/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : client.c
*		Functions for performing 'Client' things.
*
*       Updated :
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#ifdef __WIN32__
#include <winsock.h>
#include <winsock2.h>
#define SOCK_TYPE PF_INET
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#define SOCK_TYPE AF_INET
#endif

#include "../Commoninclude.h"

extern int LOG_SOCKET;
extern struct XMLLeaf * ConfigXMLTree;
extern char MIDAS_EOL[];

/******************************************************************************
*	char * SendToServer(char * SendBuff)       
*		Sends a string buffer to MIDASd. Uses MIDAS_EOL as the EOL char.
*
*	char * SendBuff		String Buffer to send.
*
*       Updated :
******************************************************************************/
char * SendToServer(char * SendBuff)
{
	struct XMLLeaf * LocalLeaf;
	int sock, connected;
	char * IP = 0;
	unsigned Port = 0;
	char * RecvBuff = 0;
	struct sockaddr_in addr;
	struct hostent * hostinfo = 0;
	
	if(SendBuff)
	{
		LocalLeaf = ConfigXMLTree;
		while(LocalLeaf)
		{
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.SERVERIP.IP");
			if(LocalLeaf)
			{
				IP = dstrcpy(IP, LocalLeaf->Value);
				Port = Myatol(ReturnXMLLeafValue(LocalLeaf, "CONFIG.SERVERIP.IP.PORT"));
				if(IP && Port)
				{
					MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToServer]-[Connecting to : %s:%u]", IP, Port);

					sock = socket(SOCK_TYPE, SOCK_STREAM, 0);
        				if(sock != -1)
        				{
						hostinfo = gethostbyname(IP);

						addr.sin_family = AF_INET;
						addr.sin_port = htons(Port);
						addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
						bzero(&(addr.sin_zero), 8);
					
						connected = connect(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr));
						
						if(connected != -1)
						{
							// [TCC] Grab the welcome line
							RecvBuff = RecvFromSocket(sock, MIDAS_EOL, 0);
							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendtoServer]-[Welcome:%s]", dstrclean(RecvBuff));
							RecvBuff = dfree(RecvBuff);
							
							SendToSocket(sock, SendBuff, MIDAS_EOL, 1);
							
							RecvBuff = RecvFromSocket(sock, MIDAS_EOL, 1);
							
							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendtoServer]-[RecvLen:%d]", dstrlen(RecvBuff));
							
							SendToSocket(sock, "<CLIENT><CLOSE></CLOSE></CLIENT>", MIDAS_EOL, 1);
							sleep(1);
						}

					} else {

						LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.SERVERIP.IP.FAILOVER");
						if(LocalLeaf)
						{
							if(!Myatoi(LocalLeaf->Value))
							{
								break;
							}
						}
					}
#ifdef __WIN32__
					closesocket(sock);
#else
					close(sock);
#endif
				}
				IP = dfree(IP);	
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
	}

	return(RecvBuff);
}
