/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : snmp.c
* 		Function for performing snmp connections and Queries. This is
*		VERY stripped down and can only perform single OID SNMP requests.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <math.h>

#ifdef __WIN32__
//	Windows socket headers
#include <winsock.h>
#include <winsock2.h>
#else
//	User Standard Header files.
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#endif

#include "../Commoninclude.h"

int MAXSNMPPACKETSIZE = 1024;

extern int LOG_CHECK;

/******************************************************************************
*       struct SNMPData * GetSNMPResponse(char *serverip, char *community, char *oid)
*		Builds SNMP udp packet, sends to device collects results and
*		returns results in an SNMPData struct (along with connection
*		status). This is VERY kludgy and should be re-written at some
*		point.
*
*	Args :
*		char *serverip		IP of device to query.
*		char *community		Community to use in query.
*		char *oid		Numeric OID to query.
*
*       Updated :
******************************************************************************/
struct SNMPData * GetSNMPResponse(char *serverip, char *community, char *oid)
{
	struct sockaddr_in adr_srvr;
	struct sockaddr_in adr;
	struct hostent * hostinfo = 0;
	
	struct SNMPData * SnmpData = 0;
	int len_inet;
	int z;
	int x;
	int sock;
	char dgram[MAXSNMPPACKETSIZE];
	
	int OIDLen = 0;

	unsigned char * s_buff = 0;
	char tOID[255];

	int b_pos = 0;
	int eo_comm = 0;
	int eo_oid = 0;
	
	unsigned char header1[] = 	{	'\x30',	// Sequence
					'\x2b',	// Len : 	pos = 1	val = b_pos - 2
					'\x02', // Type : 	VERSION (2)
					'\x01', // Len  :	1 
					'\x00', // Value :	0 (Version 1)
				};
	int header1_len = 5;
	
	// Community goes here. 
	
	unsigned char getrequest[] = 	{	'\xa0',	// Type : A0 = GET-REQUEST
					'\x1d',	// Len : 	pos = eo_comm + 1 val = b_pos - (eo_comm + 1)
					'\x02',	// Type : Request-ID 
					'\x01',	// Len : 1
					'\x08',	// Value : 8
					'\x02',	// Type : Error-status
					'\x01',	// Len : 1
					'\x00',	// Value : 0
					'\x02',	// Type : Error-Index
					'\x01',	// Len : 1
					'\x00',	// Value : 0
					'\x30', // Sequence: varbind-list
					'\x0e',	// Len 		pos = eo_comm + 12 val = b_pos - (eo_comm + 13)
				 	'\x30', // Sequence: varbind
				 	'\x0e', // Len		pos = eo_comm + 14 val = b_pos - (eo_comm + 15)
				};
	int getrequest_len = 15;
	
	// OID Goes here	
	
	unsigned char footer[] =		{	'\x05',	// Type : Value
					'\x00'  // Len : 0
				};
	int footer_len = 2;
	
	SnmpData = (struct SNMPData *) drealloc((char *)SnmpData, sizeof(struct SNMPData));
	
	SnmpData->Status = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[GetSNMPResponse]-[serverip:%s]-[community:%s]-[oid:%s]\n", serverip, community, oid);
	
	s_buff = (unsigned char *) malloc(MAXSNMPPACKETSIZE);
	b_pos = AddToBuff(s_buff, header1, header1_len, b_pos, 0);
	b_pos = AddToBuff(s_buff, community, strlen(community), b_pos, 4);
	eo_comm = b_pos;
	b_pos = AddToBuff(s_buff, getrequest, getrequest_len, b_pos, 0);
	OIDLen = MakeOID(oid, tOID);
	b_pos = AddToBuff(s_buff, tOID, OIDLen, b_pos, 6);
	eo_oid = b_pos;
	b_pos = AddToBuff(s_buff, footer, footer_len, b_pos, 0);
	
	// Fix all the len's in s_buff
	s_buff[1] = b_pos - 2;
	s_buff[eo_comm + 1] = b_pos - (eo_comm + 2);
	s_buff[eo_comm + 12] = b_pos - (eo_comm + 13);
	s_buff[eo_comm + 14] = b_pos - (eo_comm + 15);

	//
	//	UDP Socket
	//
	hostinfo = gethostbyname(serverip);
	
	if(hostinfo)
	{
		memset(&adr_srvr, 0, sizeof adr_srvr);
		adr_srvr.sin_family = AF_INET;
		adr_srvr.sin_port = htons(161);
		adr_srvr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;;
		
		len_inet = sizeof adr_srvr;				 
		
		sock = socket(AF_INET, SOCK_DGRAM, 0);
		if(sock)
		{
			z = sendto(sock, s_buff, b_pos, 0, (struct sockaddr *)&adr_srvr, len_inet);
			if(z)
			{
				sleep(1);
				fcntl(sock, F_SETFL, O_NONBLOCK);
							
				x = sizeof adr;
				z = recvfrom(sock, dgram, sizeof dgram, 0, (struct sockaddr *)&adr, &x);
				if(z)
				{
					
					SnmpData->Data = FindSNMPValue(dgram, z);
					if(SnmpData->Data)
					{
						SnmpData->Status = 1;
					} else {
						SnmpData->Status = -2;
					}
				} else {
					SnmpData->Status = -2;
				}
			} else {
				SnmpData->Status = -1;
			}
			
		} else {
			SnmpData->Status = -1;
		}
	
		close(sock);
	} else {
		SnmpData->Status = -1;
	}
	
	free(s_buff);	

	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[GetSNMPResponse]-[Status:%d]-[response:%s]", SnmpData->Status, SnmpData->Data);
	return(SnmpData);
}

/******************************************************************************
*       int AddToBuff(char * buff, char *string, int slen, int pos, int type)
*		Simple function to add one buffer to another with the Type/
*		len prefix.
*
*	Args :
*		char * buff		Buffer to append to.
*		char *string		String to append.
*		int slen		string len.
*		int pos			Current position in buff.
*		int type		Type Prefix value.
*
*       Updated :
******************************************************************************/
int AddToBuff(char * buff, char *string, int slen, int pos, int type)
{
	if(type)
	{
		buff[pos] = type;
		pos++;
		buff[pos] = slen;
		pos++;
	}
	
	memcpy(buff + pos, string, slen);
	
	pos = pos + slen;
	
	return(pos);
}

/******************************************************************************
*       int MakeOID(char * nOID, char *OID)
*		Builds binary OID value from numeric OID value.
*
*	Args :
*		char * nOID	Buffer to store binary OID in.
*		char * OID	Numeric OID to convert from.
*
*       Updated :
******************************************************************************/
int MakeOID(char * nOID, char * OID)
{
	int i = 0;
	int ii = 0;
	char *p;
	char *tP;
	char *tOID = 0;
	unsigned OIDNum	= 0;
	int len = 0;
	
	
	if(nOID)
	{
		tOID = dstrcpy(tOID, nOID);
		if(!strncmp(nOID, "1.3.", 3))
		{
			OID[i] = '\x2b';
			i++;
			p = nOID + 4;
		} else {
			p = nOID;
		}
		
		do
		{
			tP = strsep(&p, ".");
			if(tP)
			{
				OIDNum = Myatoll(tP);

				for (len=0;(OIDNum >>= 7) != 0;len++);
				
				OIDNum = Myatoll(tP);
				for (ii=0;ii<len;ii++)
				{
					OID[i] = 0x80 | (OIDNum >> ((len - ii) * 7));
					i++;
				}
				
				OID[i] = 0x7f & OIDNum;
				i++;
			}
		}while(tP);
		strcpy(nOID, tOID);
		tOID = dfree(tOID);
	}
	return(i);
}

/******************************************************************************
*       char * FindSNMPValue(unsigned char *resp, int rlen)
*		Finds the SNMP value in device response and returns the a string
*		version of value.
*
*	Args :
*		unsigned char *resp		SNMP response from device.
*		int rlen		Response length.
*
*       Updated :
******************************************************************************/
char * FindSNMPValue(unsigned char *resp, int rlen)
{
	int i;
	char *tVALUE = 0;
	unsigned int tLEN = 0;
	unsigned char tTYPE = 0;
	int FoundObj = 0;
	int Skip = 0;
	
	#define BOOLEON 	0x01
	#define	INTEGER 	0x02
	#define	BIT_STR		0x03
	#define	OCT_STR		0x04
	#define	NULL_STR 	0x05
	#define	OBJ_ID 		0x06
	#define	SNMP_SEQ 	0x07
	#define SNMP_SET 	0x08
	#define PRT_STR 	0x09
	#define	T61_STR 	0x10
	#define	IA5_STR 	0x11
	#define UTC_TME 	0x12
	#define IPADDR 		0x40
	#define COUNT32 	0x41
	#define GAUGE 		0x42
	#define UNSIGN32 	0x42
	#define TIMETICK	0x43
	//#define OPAQUE		0x44
	#define COUNT64		0x46
	
	for(i=0 ;i < rlen;i++)
	{
		tTYPE = resp[i];
		tLEN = resp[i+1];

		switch(tTYPE)
		{
			case INTEGER :
			case COUNT32 :
			case UNSIGN32 :
			case TIMETICK :
			case COUNT64 :
				if(FoundObj == 1)
				{
					tVALUE = SNMP_Numeric(&resp[i], tLEN);
					return(tVALUE);
				}
				i = i + tLEN + 1;
				break;
			case BIT_STR :
			case OCT_STR :
			case PRT_STR :
			case T61_STR : 
			case IA5_STR :
				if(FoundObj == 1)
				{
					Skip = 2;
					if(tLEN > 128)
					{
						tLEN = resp[i + 2];
						Skip = 3;
					}
					
					tVALUE = dstrncpy(tVALUE, &resp[i + Skip], tLEN);
					return(tVALUE);
				}
				i = i + tLEN + 1;
				break;
			case NULL_STR :
				if(FoundObj == 1)
				{
					tVALUE = dstrcpy(tVALUE, "0");
					return(tVALUE);
				}
				break;
			case OBJ_ID :
				FoundObj = 1;
				i = i + tLEN + 1;
				break;
			case IPADDR :
				if(FoundObj == 1)
				{
					tVALUE = SNMP_IpAddr(&resp[i], tLEN);
					return(tVALUE);
				}		
			default :	
				i = i + 1;
				break;
		}	
	}
	return(tVALUE);
}

/******************************************************************************
*       char * GenHexDump(unsigned char *string, int len)
*		Produce a hex dump display of binary string.
*
*	Args :
*		unsigned char * string		Binary string to produce display from.
*		int len			Length of binary string.
*
*       Updated :
******************************************************************************/
char * GenHexDump(unsigned char *string, int len)
{
	int i, ii;
	char *tCHAR = 0;
	char *tOUT = 0;
	
	for(i=0;i<len;i=i+8)
	{
		for(ii=0;ii<8;ii++)
		{
			tCHAR = dsprintf(tCHAR, "%.3X ", string[i+ii]);
			tOUT = dstrcat(tOUT, tCHAR);
			tCHAR = dfree(tCHAR);
		}
		tOUT = dstrcat(tOUT, "\n");
	}
	return(tOUT);
}

/******************************************************************************
*       char * SNMP_Numeric(char *buf, int len)
*		Converts a SNMP numeric value to a string value.
*
*	Args :
*		char *buf	Buffer to read from.
*		int len		Length of buffer.
*
*       Updated :
******************************************************************************/
char * SNMP_Numeric(char *buf, int len)
{
	char *tOUT = 0;
	unsigned value = 0;
	int i;
	int tBIT = 0;
	unsigned tVAL = 0;
	
	for(i=1;i<len+1;i++)
	{
		tBIT = MyPow(0xFF, (len - i));
		tVAL = (buf[i + 1] & 0xff);
		value = value + tBIT * tVAL;
	}

	tOUT = dsprintf(tOUT, "%u", value); 
	return(tOUT);

}

/******************************************************************************
*       char * SNMP_IpAddr(char *buf, int len)
*		Converts a SNMP IP Address value to a String value.
*
*	Args :
*		char *buf	Buffer to read from.
*		int len		Length of buffer.
*
*       Updated :
******************************************************************************/
char * SNMP_IpAddr(char *buf, int len)
{
	char *tOUT = 0;
	
	tOUT = dsprintf(tOUT, "%u.%u.%u.%u", (buf[2] & 0xff), 
				       (buf[3] & 0xff),
				       (buf[4] & 0xff), 
				       (buf[5] & 0xff));
	return(tOUT);
}
