/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : util.c
* 		Misc. utility functions used through out MIDAS
*
*       Updated :
*               11/11/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <time.h>
#include <regex.h>

#ifdef __WIN32__
#include <winsock.h>
#include <winsock2.h>

// Win32 aint got no syslog functions...
#define openlog(x, y, z) FILE * fb = fopen("MIDAScNT.log", "a")
#define syslog(x, y) if(fb) fprintf(fb, "%s", y)
#define closelog() fclose(fb);
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <regex.h>
#include <syslog.h>
#endif

#include "../Commoninclude.h"
extern int DEBUGLEVEL;
extern int LOGTOSTDERR;

/******************************************************************************
*       long CurrentHour()
*		Returns the time stamp for the start of the current hour
*
*
*       Updated :
******************************************************************************/
long CurrentHour()
{
	time_t tm;
	struct tm *tms;
	long timeval;
	
	tm = time(NULL);
	tms = localtime(&tm);
	timeval = ((tms->tm_hour - 1) * (60 * 60)) + (tms->tm_min * 60) + tms->tm_sec;
	return(timeval);
}

/******************************************************************************
*       long StartOfDay()
*		Returns the time stamp for the start of the current day.
*
*
*       Updated :
******************************************************************************/
long StartOfDay()
{
	time_t tm;
	struct tm *tms;
	
	tm = time(NULL);

	tms = localtime(&tm);
	
	tms->tm_hour = 1;
	tms->tm_min = 0;
	tms->tm_sec = 0;
	
	tm = mktime(tms);
	return(tm);
}
	
/******************************************************************************
*       int Myatoi(char * value)
*		Safe version of atoi.
*
*	Args :
*		char * value	String value to convert
*
*       Updated :
******************************************************************************/			                                                                
int Myatoi(char * value)
{
	if(value)
	{
		return(atoi(value));
	} else {
		return(0);
	}
}

/******************************************************************************
*       int Myatol(char * value)
*		Safe version of atol.
*
*	Args :
*		char * value	String value to convert
*
*       Updated :
******************************************************************************/			                                                                
long Myatol(char * value)
{
	if(value)
	{
		return(atol(value));
	} else {
		return(0);
	}
}

/******************************************************************************
*       int Myatof(char * value)
*		Safe version of atof.
*
*	Args :
*		char * value	String value to convert
*
*       Updated :
******************************************************************************/			                                                                
float Myatof(char * value)
{
	if(value)
	{
		return(atof(value));
	} else {
		return(0);
	}
}

/******************************************************************************
*       int Myatoll(char * value)
*		Safe version of atoll.
*
*	Args :
*		char * value	string value to convert
*
*       Updated :
******************************************************************************/			                                                                
unsigned Myatoll(char * value)
{
	if(value)
	{
		return(atoll(value));
	} else {
		return(0);
	}
}

/******************************************************************************
*     	char * ConvertL2IP(unsigned ip)
*		Converts a numeric IP address into its dotted string form
*
*	Args :
*		unsigned ip	Numeric IP address.
*
*       Updated :
******************************************************************************/
char * ConvertL2IP(unsigned ip)
{
	char *IP = 0;
	struct in_addr Address;

	Address.s_addr = ip;
	IP = dsprintf(IP, "%s", inet_ntoa(Address));
	return(IP);
}

/******************************************************************************
*       char * ConvertIP2L(unsigned ip)
*               Converts a dotted IP address to its numeric form.
*
*       Args :
*		char * IP	IP address in dotted form.
*
*       Updated :
******************************************************************************/
unsigned ConvertIP2L(char *IP)
{
        if(IP)
        {
        	return(inet_addr(IP));
        } else {
        	return(0);
	}
}      

/******************************************************************************
*       int FilePresent(char *file)
*		Determinds if a file is present (and readable). Returns 1 if 
*		true, 0 if not.
*
*	Args :
*		char * file	Path and filename of file being checked.
*
*       Updated :
******************************************************************************/
int FilePresent(char *file)
{
	FILE *fp;
	int status = 0;
	
	fp = fopen(file, "r");
	if(fp)
	{
		status = 1;
		fclose(fp);
	} else {
		status = 0;
	}
	return(status);
}

/******************************************************************************
*       int CheckStringLogic(char *line, char *regex, char *logic, char *expect)
*		Checks the 'Logic' of a string. It will either compare the 
*		line and expect directly (if a regex is not provided) or 
*		run the line through a regex and compare the results to what
*		is expected.
*
*	Args :
*		char *line	String to compare to.
*		char *regex	Regular expression to use on line (Optional).
*		char *logic	Logic to expect if true (EQUAL, GREATER, LESS,
*				NOTEQUAL).
*		char *expect	What to compare line to.
*
*       Updated :
******************************************************************************/
int CheckStringLogic(char *line, char *regex, char *logic, char *expect)
{
	regex_t re;
	regmatch_t pm;
	int re_status = -1;
	int rg_status;
	int Status = 0;
	char * REGEX_Parse = 0;

	pm.rm_so = 0;
	pm.rm_eo = 0;

	if(regex && dstrlen(regex))
	{	
		if(line && logic && expect)
		{
			rg_status = regcomp(&re, regex, REG_ICASE | REG_EXTENDED);
			re_status = regexec(&re, line, 1, &pm, 0);
			while(re_status == 0 && Status == 0 && pm.rm_so > -1 && pm.rm_eo > 0)
			{
				if(re_status == 0)
				{
					REGEX_Parse = dstrncpy(REGEX_Parse, line + pm.rm_so, (pm.rm_eo - pm.rm_so));
					if(REGEX_Parse)
					{
						Status = CompareValues(REGEX_Parse, expect, logic);
					}
					REGEX_Parse = dfree(REGEX_Parse);
				}
				re_status = regexec(&re, line + pm.rm_eo, 1, &pm, REG_NOTBOL);	
			}
			regfree(&re);
		}
	} else {
		Status = CompareValues(line, expect, logic);
	}

	return(Status);
}

/******************************************************************************
*       int CompareValues(char *text1, char *text2, char *logic)
*		Compares two strings together based on the logic specified.
*
*	Args :
*		char * text1	String to compare.
*		char * text2	String to compare.
*		char * logic	Logic to use: EQUAL, NOTEQUAL, GREATER, LESS.
*
*       Updated :
******************************************************************************/
int CompareValues(char *text1, char *text2, char *logic)
{
	int Status = 0;
	
	if(text1 && text2 && logic)
	{
		if(!strcasecmp(logic, "EQUAL"))
		{
			if(!strcasecmp(text1, text2))
			{
				Status = 1;
			}
		}
			
		if(!strcasecmp(logic, "NOTEQUAL"))
		{
			if(strcasecmp(text1, text2))
			{
				Status = 1;
			}
		}
			
		if(!strcasecmp(logic, "GREATER"))
		{
			if(Myatof(text1) > Myatof(text2))
			{
				Status = 1;
			}
		}
			
		if(!strcasecmp(logic, "LESS"))
		{
			if(Myatof(text1) < Myatof(text2))
			{
				Status = 1;
			}
		}
	}
	
	return(Status);
}

// [TCC] Need to remove from source if we define NODEBUG. Kills function def otherwise.
#ifndef NODEBUG					
/******************************************************************************
*       void MyDebug(int line, char * file, int loglevel, char * msg, ...)
*		All in one syslog logger. Will Cause a log entry to be sent
*		to the DEBUG syslog facilities if loglevel is specified within
*		DEBUGLEVEL. 
*
*	Args :
*		int line	Source code line number (__LINE__)
*		char * file	Source file name (__FILE__)
*		int loglevel	The log level of this entry.
*		char * msg	Message/Formating string to log.
*		...		Any args to use with formatting string.
*
*       Updated :
******************************************************************************/	
void MyDebug(int line, char * file, int loglevel, char * msg, ...)
{
	char *tMSG = 0;
	char *tOUT = 0;
	va_list arg_list;
	int Len = 0;
	
	
        if(DEBUGLEVEL)
        {
                if(loglevel & DEBUGLEVEL)
                {
                	va_start(arg_list, msg);
                	Len = valen(msg, arg_list);
                	va_end(arg_list);
			
			tMSG = drealloc(tMSG, Len);
			
			va_start(arg_list, msg);
			vsprintf(tMSG, msg, arg_list);
			va_end(arg_list);
			
			tOUT = dsprintf(tOUT, "[%s:%u:%d] - %s\n", file, getpid(), line, tMSG);
			
			if(LOGTOSTDERR)
			{
				fprintf(stdout, "%s", tOUT);
			} else {
				openlog(NULL, LOG_NOWAIT, LOG_USER);
				syslog(LOG_USER | LOG_DEBUG, tOUT);
				closelog();
			}
			
			tOUT = dfree(tOUT);
			tMSG = dfree(tMSG);
                }
        }
}
#endif

/******************************************************************************
*       unsigned long MyPow(int x, int y)
*		Portable version of pow function.
*
*	Args :
*		int x	Number to raise
*		int y	Raise x by this number
*
*       Updated :
******************************************************************************/
unsigned long MyPow(int x, int y)
{
	int i;
	unsigned long out;
	
	if(y > 0)
	{
		out = x;
		for(i=1;i<y;i++)
		{
			out = out * x;
		}
	} else {
		out = 1;
	}

	return(out);
}

/******************************************************************************
*	char * RegexString(char * Output, char * String, char * Regex)
*		Generic Regular expresion string parser. Will return the first 
*		match.
*
*	Args :
*		char * Output		Output string pointer.
*		char * String		String to parse on.
*		char * Regex		Regular Expression to parse with.
*
*       Updated :
******************************************************************************/
char * RegexString(char * Output, char * String, char * Regex)
{
	regex_t re;
	regmatch_t pm;
	int re_status = -1;

	pm.rm_so = 0;
	pm.rm_eo = 0;

	if(Regex && dstrlen(Regex))
	{	
		regcomp(&re, Regex, REG_ICASE | REG_EXTENDED);
		re_status = regexec(&re, String, 1, &pm, 0);
		
		if(re_status == 0)
		{
			Output = dstrncpy(Output, String + pm.rm_so, (pm.rm_eo - pm.rm_so));
		}
		regfree(&re);
	} 

	return(Output);
}

/******************************************************************************
*	unsigned UnixTime2SQLTime(unsigned UnixTime)
*		Converts a Unix timestamp to a SQL Timestamp (YYYYMMDDHHMMSS)
*
*	Args :
*		unsigned UnixTime	The Unix time stamp to convert
*
*       Updated :
******************************************************************************/
char * UnixTime2SQLTime(unsigned UnixTime)
{
	struct tm * tm;
	char * Out = 0;
		
	if(UnixTime)
	{
		tm = localtime((time_t *) &UnixTime);
	
		if(tm)
		{
			Out = dsprintf(Out, "%.4d%.2d%.2d%.2d%.2d%.2d", 
					    (tm->tm_year + 1900), tm->tm_mon, tm->tm_mday,
					    tm->tm_hour, tm->tm_min, tm->tm_sec);
		}
	}
	
	return(Out);
}	

/******************************************************************************
*	int IsStringNumeric(char * string)
*		Returns 1 if string is a number, 0 if alpha numeric
*
*	Args :
*		char * string	String to check
*       Updated :
******************************************************************************/
int IsStringNumeric(char * string)
{
	char Valid[] = "1234567890.";
	int i = 0;
	int Status = 1;
		
	if(string)
	{
		for(i=0;i<dstrlen(string);i++)
		{
			if(!strchr(Valid, string[i]))
			{
				Status = 0;
				break;
			}
		}
	}

	return(Status);
}

void PrintBin(unsigned Decimal)
{
	int i = 0;
	int ii = 0;
	
	printf("[");
	
	for(i=0;i<32;i=i+8)
	{
		for(ii=0;ii<8;ii++)
		{
	
			if((Decimal & MyPow(2, i+ii)) == MyPow(2, i+ii))
			{
				printf("1");
			} else {
				printf("0");
			}
		}
		printf(" ");
	}

	printf("]\n");
}

char * UnEscapeChars(char * string)
{
	string = dsubstrreplace(string, "$[LT]$", "<");
	string = dsubstrreplace(string, "$[GT]$", ">");
	string = dsubstrreplace(string, "$[LF]$", "\n");
	string = dsubstrreplace(string, "$[CRLF]$", "\r\n");
	
	return(string);
}
