/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : rrd.c
* 		Functions for storing statistical data in a Round Robin style 
*		SQL tables.
*
*       Updated :
*               11/11/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "../Commoninclude.h"

extern struct SQLCFG * SQLCFG;
extern int LOG_INFO;

/******************************************************************************
*       int RRD_UpdateDSValue(int DSID, unsigned TimeStamp, float Value)
*		Update a Data source table. Also updates Archive tables if
*		requred.
*
*	Args :
*		int DSID		Data Source ID.
*		unsigned TimeStamp	Unix time stmap of update.
*		float Value		Value to insert into updated record.
*
*       Updated :
******************************************************************************/
int RRD_UpdateDSValue(int DSID, unsigned TimeStamp, float Value, MYSQL * mysql)
{
	MYSQL_ROW qhres;
	struct DBR * dbr = 0;
        struct DBR * dbr2 = 0;
	MYSQL_ROW qhres2;
	int Status = 0;
	char * query = 0;
	unsigned Sequences = 0;
	unsigned SeqInterval = 0;
	unsigned LastTimeStamp = 0;
	unsigned BaseTimeStamp = 0;	
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[RRD_UpdateDSValue]-[enter]");
	
	if(DSID && TimeStamp)
	{
		query = dsprintf(query, "SELECT Sequences, Intervals FROM RRD_DataSource WHERE DSID='%d'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			if(dbr->num)
			{
				qhres = dbResult(dbr);
				
				Sequences = Myatoll(qhres[0]);
				SeqInterval = Myatoll(qhres[1]);
				
				query = dsprintf(query, "SELECT TimeStamp FROM RRD_DataValues WHERE DSID='%d' AND Sequence='%u'", DSID, Sequences);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				
				if(dbr2)
				{
					if(dbr2->num)
					{
						qhres2 = dbResult(dbr2);
						LastTimeStamp = Myatoll(qhres2[0]);
					} else {
						LastTimeStamp = 0;
					}
				} else {
					LastTimeStamp = 0;
				}
			
				dbr2 = dbFree(dbr2);
	
				if(TimeStamp >= (LastTimeStamp + SeqInterval))
				{
					RRD_ShiftValues(DSID, TimeStamp, SeqInterval, Sequences, "RRD_DataValues", "DSID", mysql);

					BaseTimeStamp = (TimeStamp / SeqInterval) * SeqInterval;
					
					query = dsprintf(query, "UPDATE RRD_DataValues SET Value='%f', TimeStamp='%u' WHERE DSID='%d' AND Sequence='%u'",
								Value, 
								BaseTimeStamp,
								DSID,
								Sequences);
					dbr2 = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr2 = dbFree(dbr2);
					
						
					query = dsprintf(query, "UPDATE RRD_DataValues SET AverageValue='%f', DerivedValue='%f' WHERE DSID='%d' AND Sequence='%u'",
								RRD_AverageValue(DSID, "RRD_DataValues", "DSID", 0, 0, mysql),
								RRD_DerivedValue(DSID, Sequences, "RRD_DataValues", "DSID", mysql),
								DSID,
								Sequences);
					dbr2 = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr2 = dbFree(dbr2);
		
					RRD_UpdateArchValues(DSID, TimeStamp, Value, mysql);
					Status = 1;
					
				}
				dbr2 = dbFree(dbr2);
			}
			dbr = dbFree(dbr);
		}
	}

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[RRD_UpdateDSValue]-[exit]");
	
	return(Status);
}

/******************************************************************************
*       void RRD_UpdateArchValues(int DSID, unsigned TimeStamp, float Value)
*		Updates the Archived Data source values.
*
*	Args :
*		int DSID		Data Source ID.
*		unsigned TimeStamp	Time stamp of update.
*		float Value		
*
*       Updated :
******************************************************************************/
void RRD_UpdateArchValues(int DSID, unsigned TimeStamp, float Value, MYSQL * mysql)
{
	MYSQL_ROW qhres;
	struct DBR * dbr = 0;
        struct DBR * dbr2 = 0;
	MYSQL_ROW qhres2;
	char * query = 0;
	unsigned Sequences = 0;
	unsigned SeqInterval = 0;
	unsigned LastTimeStamp = 0;
	unsigned BaseTimeStamp = 0;	
	int i = 0;
	int DAID = 0;	
	
	if(DSID && TimeStamp)
	{
		query = dsprintf(query, "SELECT DAID, Sequences, Intervals FROM RRD_ArchiveSource WHERE DSID='%d'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			for(i=0;i<dbr->num;i++)
			{
				qhres = dbResult(dbr);
				
				DAID = Myatoi(qhres[0]);
				Sequences = Myatoll(qhres[1]);
				SeqInterval = Myatoll(qhres[2]);
				
				query = dsprintf(query, "SELECT TimeStamp FROM RRD_ArchiveValues WHERE DAID='%d' AND Sequence='%u'", DAID, Sequences);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				
				if(dbr2)
				{
					if(dbr2->num)
					{
						qhres2 = dbResult(dbr2);
						LastTimeStamp = Myatoll(qhres2[0]);
					} else {
						LastTimeStamp = 0;
					}
				} else {
					LastTimeStamp = 0;
				}
				dbr2 = dbFree(dbr2);
	
				if(TimeStamp >= (LastTimeStamp + SeqInterval))
				{
					RRD_ShiftValues(DAID, TimeStamp, SeqInterval, Sequences, "RRD_ArchiveValues", "DAID", mysql);

					BaseTimeStamp = (TimeStamp / SeqInterval) * SeqInterval;
					
					query = dsprintf(query, "UPDATE RRD_ArchiveValues SET Value='%f', TimeStamp='%u' WHERE DAID='%d' AND Sequence='%u'",
								RRD_AverageValue(DSID, "RRD_DataValues", "DSID", BaseTimeStamp, SeqInterval, mysql),
								BaseTimeStamp,
								DAID,
								Sequences);
					dbr2 = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr2 = dbFree(dbr2);
					
					query = dsprintf(query, "UPDATE RRD_ArchiveValues SET AverageValue='%f', DerivedValue='%f' WHERE DAID='%d' AND Sequence='%u'",
								RRD_AverageValue(DSID, "RRD_ArchiveValues", "DAID", 0, 0, mysql),
								RRD_DerivedValue(DSID, Sequences, "RRD_ArchiveValues", "DAID", mysql),
								DAID,
								Sequences);
					dbr2 = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr2 = dbFree(dbr2);
				}
				dbr2 = dbFree(dbr2);
			}
			dbr = dbFree(dbr);
		}
	}
}

/******************************************************************************
*	void RRD_ShiftValues(int ID, unsigned TimeStamp, unsigned Interval, unsigned Sequences, char * TableName, char * FieldName)
*		Deletes 1st record in a Data source, shifts all values one 
*		place back and inserts the new record as last record. Function 
*		works on both Data Sources and Data Archives.
*
*	Args :
*		int ID			The DSID/DAID.
*		unsigned TimeStamp	Timestamp of new data.
*		unsigned Interval	Time interval between sequences.
*		unsigned Sequences	Number of sequences in source.
*		char * TableName	SQL Table name.
*		char * FieldName	Field name to use as 'ID' field.
*		
*       Updated :
******************************************************************************/
void RRD_ShiftValues(int ID, unsigned TimeStamp, unsigned Interval, unsigned Sequences, char * TableName, char * FieldName, MYSQL * mysql)
{
	struct DBR * dbr = 0;
	struct DBR * dbr2 = 0;
	MYSQL_ROW qhres;
	unsigned NumShifts = 0;
	unsigned i;
	unsigned MaxTimeStamp = 0;
	unsigned BaseTimeStamp = 0;
	
	char * query = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[RRD_ShiftValues]-[enter]");
	
	if(ID && TimeStamp && Interval && Sequences)
	{	
		query = dsprintf(query, "SELECT TimeStamp FROM %s WHERE %s='%d' ORDER BY TimeStamp DESC LIMIT 1", TableName, FieldName, ID);
		dbr = dbQuery_P(mysql, query);
		query =	 dfree(query);
	
		if(dbr)
		{
			if(dbr->num)
			{
				qhres = dbResult(dbr);
				MaxTimeStamp = Myatoll(qhres[0]);
				BaseTimeStamp = (TimeStamp / Interval) * Interval;
				NumShifts = (TimeStamp - MaxTimeStamp) / Interval;
				
				if(NumShifts > Sequences)
				{
					NumShifts = Sequences;
				}

#ifdef MYSQL4
				// MySQL v4.x specific Statement
				query = dsprintf(query, "UPDATE %s SET Sequence=Sequence-%u TimeStamp=TimeStamp WHERE %s='%d' ORDER BY Sequence ASC", TableName, NumShifts, FieldName, ID);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
#else
				if(NumShifts)
				{
					for(i=0;i<=Sequences;i++)
					{
						query = dsprintf(query, "UPDATE %s SET Sequence=%d, TimeStamp=TimeStamp WHERE %s='%d' AND Sequence=%u", TableName, (i - NumShifts), FieldName, ID, i);

						dbr2 = dbQuery_P(mysql, query);
						query = dfree(query);
						dbr2 = dbFree(dbr2);
					}
				}
#endif
				                            
				query = dsprintf(query, "DELETE FROM %s WHERE Sequence < 1", TableName);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
				
				query = dsprintf(query, "INSERT INTO %s (%s, Sequence, Value, AverageValue, DerivedValue, TimeStamp) VALUES ('%d', '%u', 0, 0, 0, '%u')", TableName, FieldName, ID, Sequences, (BaseTimeStamp - (Interval * i)));
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
				
			} else {
				query = dsprintf(query, "INSERT INTO %s (%s, Sequence, Value, AverageValue, DerivedValue, TimeStamp) VALUES ('%d', '%u', 0, 0, 0, '%u')", TableName, FieldName, ID, Sequences, BaseTimeStamp);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
			}
		}
		dbr = dbFree(dbr);
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[RRD_ShiftValues]-[exit]");
	
}

/******************************************************************************
*	float RRD_AverageValue(int ID, char * TableName, char * FieldName, unsigned StartTimeStamp, unsigned Interval)	
*		Returns the average value for a Data Source.
*	
*	Args :
*		int ID			DSID/DAID to search.
*		char * TableName	SQL Table name to use to read source.
*		char * FieldName	Field name to use as 'ID' field.
*		unsigned StartTimeStamp	Start time stamp to start search at.
*		unsigned Interval	The interval to check to. End of Search
*					would be StartTimeStamp + Interval.
*
*       Updated :
******************************************************************************/
float RRD_AverageValue(int ID, char * TableName, char * FieldName, unsigned StartTimeStamp, unsigned Interval, MYSQL * mysql)
{
	struct DBR * dbr = 0;
	MYSQL_ROW qhres;
	float Value = 0;
	char * query = 0;
	
	if(ID && TableName && FieldName)
	{	
		if(StartTimeStamp && Interval)
		{
			query = dsprintf(query, "SELECT AVG(Value) FROM %s WHERE %s='%d' AND TimeStamp >= %u AND TimeStamp <= %u", 
						TableName, FieldName, ID,
						StartTimeStamp, (StartTimeStamp + Interval));
		} else {
			query = dsprintf(query, "SELECT AVG(Value) FROM %s WHERE %s='%d'", TableName, FieldName, ID);
		}
		
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
	
		if(dbr)
		{
			if(dbr->num)
			{
				qhres = dbResult(dbr);
				Value = Myatof(qhres[0]);
			}
		}
		dbr = dbFree(dbr);
	}
	return(Value);
}

/******************************************************************************
*	float RRD_DerivedValue(int ID, unsigned Sequences, char * TableName, char * FieldName)
*		Returns the difference between the last and 2nd to last value
*		of a Data Source.
*
*	Args :
*		int ID			DSID/DAID to search through.
*		unsigned Sequences	What is the maximum sequence for this 
*					source.
*		char * TableName	SQL Table name to search.
*		char * FieldName	Table Field to consider the 'ID'.
*
*       Updated :
******************************************************************************/
float RRD_DerivedValue(int ID, unsigned Sequences, char * TableName, char * FieldName, MYSQL * mysql)
{
	struct DBR * dbr = 0;
	struct DBR * dbr2 = 0;
	MYSQL_ROW qhres;
	float Value = 0;
	char * query = 0;
	float MaxValue = 0;
		
	if(ID && Sequences && TableName && FieldName)
	{	
		query = dsprintf(query, "SELECT Value FROM %s WHERE %s='%d' AND Sequence='%u'", 
					TableName, FieldName, ID, Sequences);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			if(dbr->num)
			{
				qhres = dbResult(dbr);
				MaxValue = Myatof(qhres[0]);
				
				query = dsprintf(query, "SELECT Value FROM %s WHERE %s='%d' AND Sequence='%u'", 
							TableName, FieldName, ID, (Sequences - 1));
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				
				if(dbr2)
				{
					if(dbr2->num)
					{
						qhres = dbResult(dbr2);
						Value = MaxValue - Myatof(qhres[0]);
					}
				}
				dbr2 = dbFree(dbr2);	
			}
		}
		dbr = dbFree(dbr);
	}
	return(Value);
}

/******************************************************************************
*	int RRD_CreateDataSource(struct RRDCreateDS * RRDDef)
*		Creates the Data and Archive table entries for a Source.
*
*	Args :
*		struct RRDCreateDS * RRDDef	The Source def struct.
*
*       Updated :
******************************************************************************/
int RRD_CreateDataSource(struct RRDCreateDS * RRDDef, MYSQL * mysql)
{
	unsigned DSID = 0;
	struct RRDCreateArch * RRDArch = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	
	if(RRDDef)
	{
		DSID = RRD_MaxID("RRD_DataSource", "DSID", mysql);
		DSID++;
		
		query = dsprintf(query, "INSERT INTO RRD_DataSource (DSID, Sequences, Intervals, Title, Format) VALUES ('%d', '%u', '%u', '%s', '%s')",
					DSID,
					RRDDef->Sequences,
					RRDDef->Interval,
					RRDDef->Title,
					RRDDef->Format);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
		
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[RRD_CreateDataSource]-[New DSID:%d]", DSID);
		
		RRDArch = RRDDef->Archive;
		while(RRDArch)
		{
			if(RRDArch)
			{
				query = dsprintf(query, "INSERT INTO RRD_ArchiveSource (DSID, Sequences, Intervals) VALUES ('%u', '%u', '%u')",
							DSID,
							RRDArch->Sequences,
							RRDArch->Interval);
				dbr = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr = dbFree(dbr);
				
				RRDArch = RRDArch->NextArch;
			}
		}
	}
	
	return(DSID);
}


/******************************************************************************
*	void RRD_DeleteDataSource(int DSID)
*		Deletes the Data and Archive table entries for a source.
*
*	Args :
*		int DSID	The Data Source ID to delete.
*
*       Updated :
******************************************************************************/
void RRD_DeleteDataSource(int DSID, MYSQL * mysql)
{
	char * query = 0;
	MYSQL_ROW qhres;
	unsigned i = 0;
	struct DBR * dbr = 0;
	struct DBR * dbr2 = 0;	
	
	if(DSID)
	{
		query = dsprintf(query, "DELETE FROM RRD_DataValues WHERE DSID='%d'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
		
		query = dsprintf(query, "DELETE FROM RRD_DataSource WHERE DSID='%d'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
		
		query = dsprintf(query, "SELECT DAID FROM RRD_ArchiveSource WHERE DSID='%d'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			for(i=0;i<dbr->num;i++)
			{
				qhres = dbResult(dbr);
				
				query = dsprintf(query, "DELETE FROM RRD_ArchiveValues WHERE DAID='%s'", qhres[0]);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
			}
			dbr2 = dbFree(dbr2);
		}
		dbr = dbFree(dbr);
	}
}

/******************************************************************************
*
*	unsigned RRD_MaxID(char * TableName, char * FieldName)
*		Returns the maximum DSID/DAIS ID value.
*
*	Args :
*		char * TableName	Table name to search.
*		char * FieldName	Field to consider the 'ID' value.
*
*       Updated :
******************************************************************************/
unsigned RRD_MaxID(char * TableName, char * FieldName, MYSQL * mysql)
{
	char * query = 0;
	MYSQL_ROW qhres;
	unsigned ID = 0;
	struct DBR * dbr = 0;
	
	query = dsprintf(query, "SELECT %s FROM %s ORDER BY %s DESC LIMIT 1", FieldName, TableName, FieldName);
	dbr = dbQuery_P(mysql, query);
	
	if(dbr)
	{
		if(dbr->num)
		{
			qhres = dbResult(dbr);
			ID = Myatoll(qhres[0]);
		}
	}
	dbr = dbFree(dbr);
		
	return(ID);
}
