<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	01/31/2004  -   Added support for determining HTTP/HTTPS sessions
	                BUG: #888300

	01/28/2004  -   Fixed error with magic_quotes function in the
	                Add/Remove Modules screen.

	01/21/2004  -   Fixed problems with data returned when magic_quotes_gpc
	                is toggled On in the php.ini file. This was affecting
	                the IDS Configuration Screens described in BUG #872507.
				-   With the discovery of the effects magic_quotes_gpc has,
				    added set_magic_quotes_runtime() to toggle off this
					feature so it does not modify data retreived during
					the script.

	01/11/2004  -   Modifed how the MIDAS version is loaded to variable.

	12/17/2003  -   Removed obsolete graphing() function. Was commented
	                out in initial version.

	11/27/2003  -   Fixed the globalize() function.

********************************************************************/

// Determining if we are using SSL over HTTP or not.
if($_SERVER['HTTPS'] == "on")
	$PROTOCOL = "https://";
else
	$PROTOCOL = "http://";
	
// Checking for the base configuration file

if(@!file_exists(dirname(__FILE__) . "/config/config.php")) {
	$autoDetect = sprintf("%s%s%s", $PROTOCOL, $_SERVER['HTTP_HOST'], dirname($_SERVER['PHP_SELF']));
	printf("<b>ERROR:</b> Configuration file not detected!<br><br>\n");
	printf("Click <A HREF='%s/install/install.php'>here</A> to run install<br>\n", $autoDetect);
	printf("If the above link does not work, direct your browser to http://midas.web.server/path/to/midas/install/install.php\n");
	exit;
} else if(@!include_once "config/config.php") {
	$autoDetect = sprintf("%s%s%s", $PROTOCOL, $_SERVER['HTTP_HOST'], dirname($_SERVER['PHP_SELF']));
	printf("<b>ERROR:</b> Configuration file could not be loaded<br><br>\n");
	printf("Please verify %s/inc/config/config.php exists, and is readable by the webserver<br><br>\n");
	printf("Click <A HREF='%s/install/install.php'>here</A> to run install<br>\n", $autoDetect);
	printf("If the above link does not work, direct your browser to http://midas.web.server/path/to/midas/install/install.php\n");
	exit;
}

$driver['auth'] = "mySQL";  // Not needed, possible revision feature (JS)

// Upload Max File Size
define("_MAX_FILE_SIZE", 3000000);

// authentication.inc.php required defines
define("_TMPDIR", "/tmp");  // Might not be needed.. (JS)


define("_APPNAME", "MIDAS WebView");
include "version.php";

define("_MODULE_CONFIG", "CONFIG.MCF");

// tables
define("_TABLE_USERS", "MIDAS_Users");
define("_TABLE_ACTIVITY", "MIDAS_User_Activity");

define("_TABLE_REG_MODULES", "MIDAS_Reg_Modules");

define("_TABLE_ALERT_GROUPS", "MIDAS_Alert_Groups");
define("_TABLE_ALERT_CONTACTS", "MIDAS_Alert_Contacts");
define("_TABLE_ALERT_CONTACT_GROUPS", "MIDAS_Alert_Contact_Groups");

define("_TABLE_HOST_GROUPS", "MIDAS_Groups");
define("_TABLE_HOSTS", "MIDAS_Hosts");
define("_TABLE_HOST_CHECKS", "MIDAS_Host_Checks");

define("_TABLE_CHECKS", "MIDAS_Checks");
define("_TABLE_CHECK_TYPES", "MIDAS_Check_Types");

define("_TABLE_CHECK_CURRENT", "MIDAS_Check_Current");
define("_TABLE_CHECK_EVENTS", "MIDAS_Check_Events");

define("_TABLE_IDS_EVENTS", "MIDAS_IDS_Events");
define("_TABLE_SNORT_CONF", "MIDAS_SNORT_Conf");
define("_TABLE_SNORT_RULES", "MIDAS_SNORT_Rules");

define("_TABLE_NETMON_PORTS", "MIDAS_NetMon_Ports");

define("_TABLE_RRD_MIDAS_ID", "RRD_MIDAS_ID");
define("_TABLE_RRD_DATASOURCE", "RRD_DataSource");
define("_TABLE_RRD_DATAVALUES", "RRD_DataValues");
define("_TABLE_RRD_ARCHIVESOURCE", "RRD_ArchiveSource");
define("_TABLE_RRD_ARCHIVEVALUES", "RRD_ArchiveValues");


// BINARY 128|64|32|16|8|4|2|1
// Security Level defines
define("_NO_ACCESS", 0);

define("_USER", 1);
define("_GLOBAL", 3);

define("_GROUP_VIEW", 5);
define("_GLOBAL_VIEW", 7);

define("_GROUP_WRITE", 13);
define("_GLOBAL_WRITE", 15);

define("_CONFIG", 16);

define("_ADMIN", 64);

// Security Level Groups
define("_NO_ACCESS", 0);
define("_GROUP_USER", _GROUP_VIEW + _CONFIG);
define("_GROUP_USER_RW", _GROUP_WRITE + _CONFIG);
define("_GROUP_USER_ADMIN", _GROUP_WRITE + _ADMIN);

define("_GLOBAL_USER", _GLOBAL_VIEW + _CONFIG);
define("_GLOBAL_USER_RW", _GLOBAL_WRITE + _CONFIG);
define("_GLOBAL_USER_ADMIN", _GLOBAL_WRITE + _ADMIN);

define("_SU", 255);

// Module Location Definitions
define(_QUICKVIEW_PRE_BODY, 1);
define(_QUICKVIEW_BODY_UPPER, 2);
define(_QUICKVIEW_BODY_LOWER, 3);

define(_SUMMARY_PRE_BODY, 4);
define(_SUMMARY_BODY_UPPER, 5);
define(_SUMMARY_BODY_LOWER, 6);

define(_VIEWHOST_PRE_BODY, 7);
define(_VIEWHOST_BODY_UPPER, 8);
define(_VIEWHOST_BODY_LOWER, 9);

define(_VIEWGROUP_PRE_BODY, 10);
define(_VIEWGROUP_BODY_UPPER, 11);
define(_VIEWGROUP_BODY_LOWER, 12);

define(_VIEWHOST_MODULES, 13);

// Status images
$checkStatus = Array("RED" => "<IMG src='images/down.gif' alt='Service Status: Down'>",
					 "GREEN" => "<IMG src='images/up.gif' alt='Service Status: Up'>",
					 "BLUE" => "<IMG src='images/none.gif' alt='Service Status: None'>",
					 "YELLOW" => "<IMG src='images/warn.gif' alt='Service Status: Warning'>");

// report_log required defines
define("_TABLE_REPORT_LOG", "report_log");  // Possible revision feature (JS)

// Common Functions used throughout
Function getGroupName($gid)
{
	switch($gid) {
		case _NO_ACCESS:
		    return "No Access";
		    break;
		case _GROUP_USER:
		    return "User (R/O)";
			break;
		case _GROUP_USER_RW:
		    return "User (R/W)";
		    break;
		case _GROUP_USER_ADMIN:
		    return "Admin";
		    break;
		case _SU:
		    return "Super User";
		    break;
		default:
		    return "Unknown Group ($gid)";
		    break;
	}
}

Function parseURL()
{
    global $_SERVER;
    global $args;
    
    parse_str($_SERVER["QUERY_STRING"], $args);
}

Function parsePost($items)
{
    global $_POST;
    global $args;

    for($x=0;$x<sizeof($items);$x++) {
        @reset($_POST);
        for($y=0;$y<sizeof($_POST);$y++) {
            $key = key($_POST);
            if($items[$x] == $key)
                $args[$key] = $_POST[$key];
            next($_POST);
        }
    }
}

Function buildURL($url, $list)
{
    global $_SERVER;

	$url .= "?";

    $size = sizeof($list);
    @reset($list);
    for($x=0;$x<$size;$x++) {
        $key = key($list);
        if(!is_null($list[$key]))
	        $url .= sprintf("%s=%s&", $key, $list[$key]);
        next($list);
    }

    $url[strlen($url)-1] = "";
    return $url;
}

Function buildHiddenValues($exclude = FALSE)
{
    global $args;
    $list = $args;
    
    $size = sizeof($args);
    @reset($list);
    for($x=0;$x<$size;$x++) {
        $key = key($list);
        if(!in_multi_array($key, $exclude))
        	$lines .= sprintf("<input type=hidden name='%s' value='%s'>\n", $key, $list[$key]);
        next($list);
    }
    return $lines;
}

Function cToDateTimeStamp($time)
{
	if($time) {
		// Format of DateTime Stamp Weekday Month/Day/Year HH:MM AM/PM (Thu 8/21/2003 12:15 PM)
		$taFormat = explode(" ", $time);
		$taDate = explode("/", $taFormat[1]);
		$taFormat[3] = strtoupper($taFormat[3]);

		$taTime = array_merge(explode(":", $taFormat[2]), $taFormat[3]);
		if($taFormat[3] == "PM" && $taTime[0] != 12)
		    $taTime[0] = $taTime[0] + 12;
		else if ($taFormat[3] == "AM" && $taTime[0] == 12) {
		    $taTime[0] = $taTime[0] + 12;
		    $taDate[1]--;
		}
		return mktime($taTime[0], $taTime[1], 0, $taDate[0], $taDate[1], $taDate[2]);
	} else
	    return FALSE;
}

Function cFromDateTimeStamp($stamp)
{
	if($stamp) {
		// Returning DateTime Stamp Weekday Month/Day/Year HH:MM AM/PM (Thu 8/21/2003 12:15 PM)
		return date("D n/j/Y g:i A", $stamp);
	} else
	    return FALSE;
}

function in_multi_array($needle, $haystack) //taken from php.net, adapted
{
	$in_multi_array = false;
	if(!is_array($haystack)) {
	    if($needle == $haystack)
	        $in_multi_array = true;
	} else {
		if(in_array($needle, $haystack)) {
			$in_multi_array = true;
		} else {
			while (list($tmpkey,$tmpval) = each ($haystack)) {
				if(is_array($haystack[$tmpkey])) {
					if (in_multi_array($needle, $haystack[$tmpkey])) {
						$in_multi_array = true;
						break;
					}
				}
			}
		}
	}
	return $in_multi_array;
}

Function globalize()
{
    global $_POST, $_GET, $_COOKIE;
    
	// remove magic_quotes_gpc effects
    $_POST   = handleMagicQuotes($_POST);
    $_GET    = handleMagicQuotes($_GET);
    $_COOKIE = handleMagicQuotes($_COOKIE);
    
    // Order $_COOKIE over $_POST over $_GET
    @reset($_GET);
    while(($key = key($_GET))) {
        global $$key;
        $$key = $_GET[$key];
        next($_GET);
    }
    @reset($_POST);
    while(($key = key($_POST))) {
        global $$key;
        $$key = $_POST[$key];
        next($_POST);
    }
    @reset($_COOKIE);
    while(($key = key($_COOKIE))) {
        global $$key;
        $$key = $_COOKIE[$key];
        next($_COOKIE);
    }
}

function build_multi_select($name, $info, $matchValue, $default = FALSE, $size = 5)
{
	$select = sprintf("<select name='%s' size=%d multiple>\n", $name, $size);

	@reset($info);
	if(!is_Array($matchValue)) {
		$matchValue = explode(" ", $matchValue);
	}

	@reset($matchValue);
	if($default)
	    $select .= "<option value=''>$default</option>\n";

	if(is_array($info)) {
		while(!is_null($key = @key($info))) {
			if(in_multi_array($key, $matchValue)) {
				$select .= sprintf("<option value='%s' selected>%s</option>\n", $key, $info[$key]);
			} else {
			    $select .= sprintf("<option value='%s'>%s</option>\n", $key, $info[$key]);
			}
			@next($info);
		}
	}
	$select .= "</select>\n";

	return $select;
}

function build_select($name, $info, $matchValue, $default = FALSE, $extra = FALSE)
{
	$select = sprintf("<select name='%s' size=1 %s>\n", $name, $extra);
	if($default)
		$select .= sprintf("<option value=''>%s</option>\n", $default);
	@reset($info);
	if(is_array($info)) {
		while(!is_null($key = @key($info))) {
			if($matchValue == $key) {
				$select .= sprintf("<option value='%s' selected>%s</option>\n", $key, $info[$key]);
			} else {
			    $select .= sprintf("<option value='%s'>%s</option>\n", $key, $info[$key]);
			}
			@next($info);
		}
	}
	$select .= "</select>\n";

	return $select;
}

Function pageinate($loc, $sArray, $variables)
{
    global $color, $args;

	// Getting variable & start/size locations
	$startKey = @key($variables['Start']);
	$sizeKey  = @key($variables['Size']);
	$start    = $variables['Start'][$startKey];
	$size     = $variables['Size'][$sizeKey];

    $nView = ($start+$size);
    $pView = ($start-$size);
    if($pView < 0)
        $pView = 0;

    if($start != 0)
        $pagePrev = sprintf("<A HREF='%s'>Prev</A> |", buildURL($loc, array_merge($args, Array($startKey => $pView , $sizeKey => $size))));
    if($nView < $sArray) {
        $pageList = sprintf("<b>%d</b> to <b>%d</b> (%d Total)", $start+1, $nView, $sArray);
        $pageNext = sprintf("<A HREF='%s'>Next</A> |", buildURL($loc, array_merge($args, Array($startKey => $nView, $sizeKey => $size))));
    } else
        $pageList = sprintf("<b>%d</b> to <b>%d</b> (%d Total)", $start+1, $sArray, $sArray);
    if($size < $sArray)
        $pageShow = sprintf("<A HREF='%s'>Show All</A> |", buildURL($loc, array_merge($args, Array($startKey => 0, $sizeKey => $sArray))));

    $pageinate = sprintf("%s %s %s %s\n", $pagePrev, $pageNext, $pageShow, $pageList);

    return html_tag("TABLE",
               html_tag("TR",
                   html_tag("TD", $pageinate, $color[12], "width=100% nobr align=right"),
               "", ""),
           $color[3], "width=100% cellspacing=0 cellpadding=0");
}

/*
	This function was found at http://www.php.net/get-magic-quotes-gpc
*/

function &handleMagicQuotes(&$value)
{
    if (get_magic_quotes_gpc()) {
        if (is_array($value)) {
            $result = array();
            foreach ($value as $k => $v) {
                if (is_array($v)) {
                    $result[$k] = handleMagicQuotes($v);
                } else {
                    $result[$k] = stripslashes($v);
                }
            }
            return $result;
        } else {
            return stripslashes($value);
        }
    }
    return $value;
}

// Removing Magic Quotes runtime so data is not modified during the script
@set_magic_quotes_runtime(0);

globalize();
parseURL();
?>
