<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/*************************************************************************
   file:	inc/module.inc.php
   date:	09/08/2003
   Author:	jsessler
   build:	09092003

   CHANGES:
   10/09/2003   - Added ability for modules to set a load priority. Before
				  modules were loaded via Alphabetical order. Gives ability
				  to have modules load after dependencies, and load in a more
				  controlled order. (JS)

   TODO:

 *************************************************************************/

CLASS module
{
	var $p_dir;
	var $s_dir;
	
	var $registered_modules;
	var $unregistered_modules;

	var $module_details;

	var $init_scripts;
	var $message;

	function module($dir = "/modules")
	{
		$abs_dir = _ABS_ROOT . $dir;

		if(!($this->p_dir = @opendir($abs_dir))) {
			$this->message = sprintf("Unable to open module directory: [%s]", $abs_dir);

			return FALSE;
		} else {
			$this->message = "Module directory opened successfully";
			$this->s_dir = $abs_dir;

			// Loading all registered modules into memory
			$this->load_registered_modules();

			return TRUE;
		}
	}

	function load_registered_modules()
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_REG_MODULES);
		$db->add_extra("ORDER by module_priority,module_name ASC");
		$ret = $db->compile_query("SELECT");
		if($ret['return']) {
			$db->run_query();
			if($db->rows) {
				for($x=0;$x<$db->rows;$x++) {
					$db->step();
					$this->registered_modules[$x] =
						Array("name" => $db->res['module_name'],
							  "include" => $db->res['module_loc'],
							  "require" => $db->res['module_req']);
					$this->init_scripts[$x] = $db->res['module_init'];
				}
			} else {
				$this->message = sprintf("No registered modules found");
			}
		} else {
			$this->message = $ret['errmsg'];
		}
		$db->clear_res();
		$db->db_close();
		$db = FALSE;
	}
	
	function reset_registered_modules($modules)
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_REG_MODULES);
		$ret = $db->compile_query("DELETE");
		if($ret['return']) {
			$db->run_query();
			$this->registered_modules = NULL;
			for($x=0;$x<sizeof($modules);$x++) {
				$this->add_registered_module($modules[$x]);
			}
			$this->load_registered_modules();
		} else {
			$this->message = $ret['errmsg'];
		}
		$db->clear_res();
		$db->db_close();
		$db = FALSE;
	}

	function add_registered_module($config)
	{
		if($module_info = $this->get_module_info($config)) {
			global $credentials;
			$db = new conn_mySQL($credentials);
			$db->db_new_conn();
			$db->add_table(_TABLE_REG_MODULES);
			$db->add_data("module_name",$module_info['NAME']);
			$db->add_data("module_loc",$module_info['INCLUDE']);
			$db->add_data("module_req",$module_info['REQUIRE']);
			if($module_info['PRIORITY'])
				$db->add_data("module_priority",$module_info['PRIORITY']);
			$db->add_data("module_init",$module_info['INIT']);
			$ret = $db->compile_query("INSERT");
			if($ret['return']) {
				$db->run_query();
				$this->registered_modules = NULL;
				$this->load_registered_modules();
			} else {
				$this->message = $ret['errmsg'];
			}
			$db->clear_res();
			$db->db_close();
			$db = FALSE;
		}
	}
	
	function remove_registered_module($config)
	{
		if($module_info = $this->get_module_info($config)) {
			global $credentials;
			$db = new conn_mySQL($credentials);
			$db->db_new_conn();
			$db->add_table(_TABLE_REG_MODULES);
			$db->clause = sprintf("where module_name=\"%s\"", $module_info['NAME']);
			$ret = $db->compile_query("DELETE");
			if($ret['return']) {
				$db->run_query();
				$this->registered_modules = NULL;
				$this->load_registered_modules();
			} else {
				$this->message = $ret['errmsg'];
			}
			$db->clear_res();
			$db->db_close();
			$db = FALSE;
		}
	}
	
	function run_registered_modules_inits()
	{
		for($x=0;$x<sizeof($this->registered_modules);$x++) {
			if(function_exists($this->init_scripts['name']))
				call_user_func($this->init_scripts['name']);
			else {
				$this->message = sprintf("Unable to find function: [%s::%s]", $this->registered_modules[$x]['name'], $this->init_scripts['name']);
				return FALSE;
			}
		}
		return TRUE;
	}
	
	function list_modules()
	{
		while(($read = @readdir($this->p_dir)) !== FALSE) {
			$module_info = NULL;
			if($read != "." && $read != "..") {
				$abs_dir = sprintf("%s/%s", $this->s_dir, $read);
				if(is_dir($abs_dir)) {
					$module_config_file = $abs_dir . "/" . _MODULE_CONFIG;
					if(is_file($module_config_file)) {
						if($fp = @fopen($module_config_file,"r")) {
							while(!feof($fp)) {
								$line = rtrim(fgets($fp, 1024));
								if($line[0] == "#" || $line[0] == " " || !$line)
								    continue;
								else if (strpos($line,"\t") !== FALSE) {
									$module_info[$config[0]] .= str_replace("\t", " ", $line);
								} else {
								    $config = explode(":", $line);
								    $module_info[$config[0]] = $config[1];
								}
							}
							$mod_name = $module_info['NAME'];
							$module_info['NAME'] = NULL;
							for($x=0;$x<sizeof($module_info);$x++) {
								if(@in_multi_array($mod_name, $this->registered_modules))
									$this->module_details['registered'][$mod_name] = $module_info;
								else
								    $this->module_details['unregistered'][$mod_name] = $module_info;
							}
						}
					}
				}
			}
		}
		@ksort($this->module_details['registered']);
		@ksort($this->module_details['unregistered']);
		@reset($this->module_details);

		@rewinddir($this->dir);
	}
	
	function get_module_info($config)
	{
		$abs_location_config = $this->s_dir . "/" . $config;
		if(!($fp = @fopen($abs_location_config,"r"))) {
			$this->message = sprintf("Unable to open module config file: [%s]:[%s]", $this->s_dir, $config);

			return FALSE;
		} else {
			while(!feof($fp)) {
				$line = rtrim(@fgets($fp, 1024));
				if($line[0] == "#" || $line[0] == " " || !$line)
					continue;
				else if (strpos($line,"\t") !== FALSE) {
					$module_info[$config[0]] .= str_replace("\t", " ", $line);
				} else {
					$config = explode(":", $line);
					$module_info[$config[0]] = $config[1];
				}
			}
			return $module_info;
		}
	}

	function clear_res()
	{
		@closedir($this->p_dir);

		$s_dir                = NULL;
		$registered_modules   = NULL;
		$unregistered_modules = NULL;
		$module_details       = NULL;
		$init_scripts         = NULL;
		$message              = FALSE;
	}
}

function addModuleAction($loc, $id, $function)
{
	global $_MOD_ACTION;
	if(isset($_MOD_ACTION))
		$_MOD_ACTION[$loc][$id] = $function;
}

function checkForModuleAction($loc)
{
	global $_MOD_ACTION;
	$returnInfo = null;

	if($_MOD_ACTION[$loc]) {
		$tMOD = $_MOD_ACTION[$loc];
		if(is_array($tMOD)) {
			foreach($tMOD as $key => $module_func) {
				if(@function_exists($module_func))
					if(!($returnInfo[$key] = @call_user_func($module_func)))
					    printf("<SPAN class='errmsg'>ERROR: Unable to read module function [%s] <br>\n", $module_func);
			}
		}
	}
	return $returnInfo;
}

$_MODULES = new module();
$_MOD_ACTION = Array();

?>
