<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	11/30/2003  -   rewrote function getTimeLineEvents(). Function
	                was built around a few unknown bugs in how MIDASd
	                updates the MIDAS_Current_Checks and
	                MIDAS_Check_Events tables.

********************************************************************/

function buildDayAvgGraph($HID, $TID = FALSE, $CID = FALSE)
{
	global $color;

	$current = time();
	for($x=0;$x<24;$x++) {
		$offset = $current - ((24 - $x) * 3600);
		$hourStep = $current - ((24 - ($x+1)) * 3600);

		if($current < $hourStep)
		    $hourStep = $current;
		    
		$info = getTimeLineEvents(date("Y-m-d H:00:00", $offset), date("Y-m-d H:00:00", $hourStep), $HID, $TID, $CID);

		@arsort($info);
		@reset($info);
		$heading .= html_tag("TD", sprintf("<nobr>%s</nobr>\n",
						date("gA", $offset)),
					$color[0], "class=blacksmall align=center");
		switch(@key($info)) {
			case "GREEN":
			    $gColor = "#00FF00";
			    break;
			case "RED":
			    $gColor = "#FF0000";
			    break;
			case "YELLOW":
			    $gColor = "#FFFF00";
			    break;
			case "BLUE":
			    $gColor = "#0000FF";
			    break;
		}

		$graph   .= html_tag("TD", "", $gColor, sprintf("height=20 Title='UP: %d%% ALERT: %d%% WARN: %d%%'", $info['GREEN'], $info['RED'], $info['YELLOW']));
	}
	
	return html_tag("TABLE",
			   html_tag("TR", $heading, "", "") .
			   html_tag("TR", $graph, "", ""),
		   $color[3], "width=100% cellspacing=0 cellpadding=0");
}

function buildWeekAvgGraph($HID, $TID = FALSE, $CID = FALSE)
{
	global $color;

	$current = mktime(23,59,59)-(24 * 3600);
	for($x=0;$x<7;$x++) {
		$offset = $current - ((7 - $x) * (24 * 3600));
		$dayStep = $current - ((7 - ($x+1)) * (24 * 3600));

		if($current < $dayStep)
		    $dayStep = $current;
		    
		$info = getTimeLineEvents(date("Y-m-d H:00:00", $offset), date("Y-m-d H:00:00", $dayStep), $HID, $TID, $CID);

		@arsort($info);
		@reset($info);

		$heading .= html_tag("TD", sprintf("<nobr>%s</nobr>\n",
						date("m/d/Y", $dayStep)),
					$color[0], "class=blacksmall align=center");
		switch(@key($info)) {
			case "GREEN":
			    $gColor = "#00FF00";
			    break;
			case "RED":
			    $gColor = "#FF0000";
			    break;
			case "YELLOW":
			    $gColor = "#FFFF00";
			    break;
			case "BLUE":
			    $gColor = "#0000FF";
			    break;
		}

		$graph   .= html_tag("TD", "", $gColor, sprintf("height=20 Title='UP: %d%% ALERT: %d%% WARN: %d%%'", $info['GREEN'], $info['RED'], $info['YELLOW']));
	}

	return html_tag("TABLE",
			   html_tag("TR", $heading, "", "") .
			   html_tag("TR", $graph, "", ""),
		   $color[3], "width=100% cellspacing=0 cellpadding=0");
}

function buildMonthAvgGraph($HID, $TID = FALSE, $CID = FALSE)
{
	global $color;

	$current = time(); 
	$weeks = 12; 

	for($x=0;$x<$weeks;$x++) {
		$offset = $current - ((($weeks - $x) * 7) * (24 * 3600));
		$weekStep = $current - ((($weeks - ($x+1)) * 7) * (24 * 3600));

		if($current < $weekStep)
		    $weekStep = $current;

		$info = getTimeLineEvents(date("Y-m-d H:00:00", $offset), date("Y-m-d H:00:00", $weekStep), $HID, $TID, $CID);

		@arsort($info);
		@reset($info);

		$heading .= html_tag("TD", sprintf("<nobr>%s</nobr>\n",
						date("m/d/y", $offset)),
					$color[0], "class=blacksmall align=center");
		switch(@key($info)) {
			case "GREEN":
			    $gColor = "#00FF00";
			    break;
			case "RED":
			    $gColor = "#FF0000";
			    break;
			case "YELLOW":
			    $gColor = "#FFFF00";
			    break;
			case "BLUE":
			    $gColor = "#0000FF";
			    break;
		}

		$graph   .= html_tag("TD", "", $gColor, sprintf("height=20 Title='UP: %d%% ALERT: %d%% WARN: %d%%'", $info['GREEN'], $info['RED'], $info['YELLOW']));
	}

	return html_tag("TABLE",
			   html_tag("TR", $heading, "", "") .
			   html_tag("TR", $graph, "", ""),
		   $color[3], "width=100% cellspacing=0 cellpadding=0");
}

function getTimeLineEvents($stime, $etime, $HID, $TID = FALSE, $CID = FALSE)
{
	global $db;
	
	if($db->dbc) {
		$db2 = new conn_mySQL();
		$db2->db_use_conn($db->dbc, $db->db);
		$closeit = FALSE;
	} else {
		global $credentials;
		$db2 = new conn_mySQL($credentials);
		$db2->db_new_conn();
		$closeit = TRUE;
	}
	
	$current      = null;
	$status       = null;
	$statusTotals = null;

	$start = strtotime($stime);
	$end   = strtotime($etime);
	$total = $end - $start;

	$clause = "where HID='$HID'";

	if($TID)
	    $clause .= " AND TID='$TID'";
	if($CID)
	    $clause .= " AND CID='$CID'";

	$db2->add_table(_TABLE_CHECK_CURRENT);
	$db2->add_select_item(Array("Status", "UNIX_TIMESTAMP(LastCheck) as LastCheck", "UNIX_TIMESTAMP(FirstCheck) as FirstCheck", "CID", "TID"));
	$db2->clause = "$clause";
	$db2->add_extra("ORDER by HID,TID,CID ASC");
	$ret = $db2->compile_query("SELECT");

	if($ret['return']) {
		$db2->run_query();
		$num = $db2->rows;
		for($x=0;$x<$db2->rows;$x++) {
			$db2->step();
			if($db2->res['FirstCheck'] <= $start && $db2->res['LastCheck'] >= $end) {
				$status[$db2->res['TID']][$db2->res['CID']]['status']  = $db2->res['Status'];
				$status[$db2->res['TID']][$db2->res['CID']]['current'] = $end;
				$status[$db2->res['TID']][$db2->res['CID']]['total'] = $total;
			} else if($db2->res['LastCheck'] > $end) {
				$status[$db2->res['TID']][$db2->res['CID']]['statusThru'] = TRUE;
				$status[$db2->res['TID']][$db2->res['CID']]['total'] = 0;
			} else if($db2->res['LastCheck'] > $start && $db2->res['LastCheck'] < $end) {
				$current[$db2->res['TID']][$db2->res['CID']]['status'] = $db2->res['Status'];
				$current[$db2->res['TID']][$db2->res['CID']]['current'] = $db2->res['LastCheck'];
				$status[$db2->res['TID']][$db2->res['CID']]['statusThru'] = FALSE;
				$status[$db2->res['TID']][$db2->res['CID']]['total'] = 0;
			} else if($db2->res['LastCheck'] < $start) {
				$status[$db2->res['TID']][$db2->res['CID']]['status']  = "BLUE";
				$status[$db2->res['TID']][$db2->res['CID']]['current'] = $end;
				$status[$db2->res['TID']][$db2->res['CID']]['total'] = $total;
			} else {
				$status[$db2->res['TID']][$db2->res['CID']]['status']  = "BLUE";
				$status[$db2->res['TID']][$db2->res['CID']]['current'] = $start;
				$status[$db2->res['TID']][$db2->res['CID']]['total'] = 0;
			}
			$status[$db2->res['TID']][$db2->res['CID']]['last'] = $start;
		}
	}

	$db2->clear_res();

	if(is_array($status)) {
		foreach($status as $tKey => $curTID) {
		    if(is_array($curTID)) {
		    	foreach($curTID as $cKey => $curCID) {
		    		if($status[$tKey][$cKey]['total'] == 0) {
						$db2->add_table(_TABLE_CHECK_EVENTS);
						$db2->add_select_item(Array("TimeStamp", "Status"));

						$db2->clause = "where HID='$HID' AND TID='$tKey' AND CID='$cKey' AND TimeStamp<'$stime'";
						$db2->add_extra("ORDER by TimeStamp DESC");
						$db2->add_extra("LIMIT 0, 1");

						$ret = $db2->compile_query("SELECT");
						if($ret['return']) {
							$db2->run_query();
					 		for($x=0;$x<$db2->rows;$x++) {
								$db2->step();
								$status[$tKey][$cKey]['status'] = $db2->res['Status'];
								$status[$tKey][$cKey]['current'] = $start;
							}
							
							if($db2->rows == 0) {
								$status[$tKey][$cKey]['status'] = 'BLUE';
								$status[$tKey][$cKey]['current'] = $start;
							}
						}
						$db2->clear_res();
					}	
				}
			}
		}
	}


	if(is_array($status)) {
		foreach($status as $TID => $curTID) {
			if(is_array($curTID)) {
				foreach($curTID as $CID => $curCID) {
					if($curCID['total'] < $total) {
						$db2->add_table(_TABLE_CHECK_EVENTS);
						$db2->add_select_item(Array("Status", "UNIX_TIMESTAMP(TimeStamp) as TimeStamp"));

						$db2->clause = sprintf("where (HID='%s' AND TID='%s' AND CID='%s') AND (TimeStamp>'%s' AND TimeStamp<='%s')",
							$HID, $TID, $CID, date("Y-m-d H:00:00", $status[$TID][$CID]['current']), $etime);
						$db2->add_extra("ORDER by TimeStamp ASC");
						$ret = $db2->compile_query("SELECT");
						if($ret['return']) {
							$db2->run_query();
							for($x=0;$x<$db2->rows;$x++) {
								$db2->step();
								$lastStatus = $status[$TID][$CID]['status'];
								if($lastStatus)
									$status[$TID][$CID]['stats'][$lastStatus] += $db2->res['TimeStamp'] - $status[$TID][$CID]['current'];
								$status[$TID][$CID]['status'] = $db2->res['Status'];
								$status[$TID][$CID]['current'] = $db2->res['TimeStamp'];
								$status[$TID][$CID]['last'] = $status[$TID][$CID]['current'];
								$status[$TID][$CID]['total'] = $db2->res['TimeStamp'] - $start;
							}
						}
						$db2->clear_res();
					}
				}
			}
		}
	}

	$db2->clear_res();

	if(is_array($status)) {
		foreach($status as $tKey => $tester) {
			if(is_array($tester)) {
				foreach($tester as $cKey => $check) {
					if(is_array($check['stats'])) {
						foreach($check['stats'] as $state => $value) {
							$stateTotals[$state] += $value;
						}
					}
					$stateTotals[$check['status']] += $check['current'] - $check['last'];

					if($status[$tKey][$cKey]['statusThru'])
						$stateTotals[$check['status']] += $end - $check['current'];
				}
			}
		}
	}

	if(is_array($current)) {
		foreach($current as $TID => $tester) {
			if(is_array($tester)) {
				foreach($tester as $CID => $check) {
					if($check['status'])
					    if($status[$TID][$CID]['current'])
							$stateTotals[$check['status']] += $check['current'] - $status[$TID][$CID]['current'];
						else
						    $stateTotals[$check['status']] += $check['current'] - $start;
				}
			}
		}
	}

	$upTime   = @round((($stateTotals['GREEN']/$num) / $total) * 100);
	$downTime = @round((($stateTotals['RED']/$num) / $total) * 100);
	$warnTime = @round((($stateTotals['YELLOW']/$num) / $total) * 100);
	$noTime   = 100 - ($upTime + $downTime + $warnTime);

	if($closeit)
	    $db2->db_close();

	$db2 = null;

	return Array('GREEN' => $upTime, 'RED' => $downTime, 'YELLOW' => $warnTime, 'BLUE' => $noTime);
}

function buildTimeLineTablePercent($info)
{
	global $color;

	if($info['GREEN'])
		$table = html_tag("TD", "", "#00FF00", sprintf("width=%s%% height=20",$info['GREEN']));
	if($info['RED'])
		$table .= html_tag("TD", "", "#FF0000", sprintf("width=%s%% height=20",$info['RED']));
	if($info['YELLOW'])
		$table .= html_tag("TD", "", "#FFFF00", sprintf("width=%s%% height=20",$info['YELLOW']));
	if($info['BLUE'])
		$table .= html_tag("TD", "", "#0000FF", sprintf("width=%s%% height=20",$info['BLUE']));
		
	return html_tag("TABLE", html_tag("TR", $table, "", ""), $color[10], "width=200 height=20 cellspacing=1 cellpadding=0");
}

?>
