<?php
/********************************************************************
Copyright (c) 2004 Rodney Caudle

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	3/9/2004  -  Initial build

********************************************************************/
/********************************************************************
* Class for a rule in the Rulebase
*  This class includes interactions with the database for 
*  doublechecking some values in various functions.
********************************************************************/
class rule {
  var $RID;		// unique RID associated with this rule
  var $Name;		// name for this rule
  var $Description;	// description for this rule
  var $Category;	// category for this rule, verified against DB
  var $Enabled;		// set to 0 if disabled and 1 if enabled
  var $versions;	// an array of the available statements by version
  var $debug;		// set to true to see debug information

/********************************************************************
* Constructors
********************************************************************/
  function rule() {
    $this->RID = "";
    $this->Name = "";
    $this->Description = "";
    $this->Category = "";
    $this->Enabled = 0;
    $this->versions = array();
    $this->debug = false;
  }

/****************************************************
* function setRID($value)
*  set the RID to $value only if the value is numeric
****************************************************/
  function setRID($value) {
    if (is_numeric($value)) { 
      $this->RID = $value; 
      return(true);
    } else { 
      return(false); 
    }
  } 
/****************************************************
* function getRID()
*  return the current value of RID
****************************************************/
  function getRID() { return($this->RID); }
/****************************************************
* function setName($value)
*  set the Name to whatever is in $value
****************************************************/
  function setName($value) { $this->Name = $value; return(true); }
/****************************************************
* function getName()
*  return the current value of Name
****************************************************/
  function getName() { return($this->Name); }
/****************************************************
* function setCategory($value)
*   set Category to whatever is in $value provided
*   that $value is an available group in the DB
*
* Requires:
*   initialization of $db somewhere prior to using
*   this function.
****************************************************/
  function setCategory($value) { 
    global $db;
    $res = false;

// First things, clear the db
   $db->clear_res();

    $db->add_table(_TABLE_SNORT_GROUPS);
    $db->clause = " where Name='$value'";
    $ret = $db->compile_query("SELECT");
    if ($this->debug) { printf("%s<hr>\n", $db->query); }
    if ($ret['return']) {
      $db->run_query();
      if ($db->rows > 0) {
        $this->Category = $value; 
        $res = true;
        if ($this->debug) { printf("Category %s was good, setting the Category now.<hr>\n", $value); }
      }
    }
    $db->clear_res(); 
    return($res); 
  }
/****************************************************
* function getCategory()
*   return the current value of Category
****************************************************/
  function getCategory() { return($this->Category); }
/****************************************************
* function setDescription
*  set Description to $value unless it exceeds 2^32
****************************************************/
  function setDescription($value) {
    if ($this->debug) { printf("Setting description for Rule %d.\n", $this->getRID()); }
    if (strlen($value) <= 4294967295 ) {
      $this->Description = $value;
      return(true);
    } else { return(false); }
  }
/****************************************************
* function getDescription()
*  return the current value of Description
****************************************************/
  function getDescription() { return($this->Description); }
/****************************************************
* function toggleEnabled()
*  set Enabled to the opposite of whatever it used
*    to be (0 -> 1, 1 -> 0)
****************************************************/
  function toggleEnabled() { 
    if ($this->Enabled == 0)
      $this->Enabled = 1;
    else
      $this->Enabled = 0;
  }
/****************************************************
* function toggleDebug()
*  set debug to the opposite of whatever it used
*    to be (true -> false, false -> true)
****************************************************/
  function toggleDebug() { $this->debug = !($this->debug); }
/****************************************************
* function getEnabled() 
*  return the current value of Enabled 
****************************************************/
  function getEnabled() { return($this->Enabled); }
/****************************************************
* function setEnabled($value) 
*  sets the value of Enabled to $value 
****************************************************/
  function setEnabled($value) { $this->Enabled = $value; return(true);}
/****************************************************
* function addVersion($ver, $value)
*  return false if the version exists otherwise
*    insert the version and return true
****************************************************/
  function addVersion($ver, $value) {
    if (strlen($this->versions{"$ver"}) > 0) {
      return(false);
    } else {
      $this->versions{"$ver"} = $value;
      return(true);
    }
    return(true);
  }
/****************************************************
* function updateVersion($ver, $value)
*  return false if the version doesn't exist
*    otherwise update the value and return true
*
*  Need to verify against the DB that the version
*    requested is a valid field
****************************************************/
  function updateVersion($ver, $value) {
//  Verify validity of $ver
    $fields = mysql_list_fields($credentials[$driver['auth']]['db'], _SNORT_TABLE_RULEBASE);
    if ( !in_array($fields, $ver) ) {
      return(false);
    }
//  So we found it now continue on!!
    if (strlen($this->versions{"$ver"}) > 0) {
      $this->versions{"$ver"} = $value;
      return(true);
    } else { return(false); }
  }
/****************************************************
* function delVersion($ver)
*   return true if the version doesn't exist
*     or if unsetting made a difference
*     otherwise return false
****************************************************/
  function delVersion($ver) {
    if (strlen($this->versions{"$ver"}) > 0) {
      unset($this->versions{"$ver"});
      if(strlen($this->versions{"$ver"}) > 0) {
        return(false);
      } else { return(true); }
    } else { return(true); }
  }
/****************************************************
* function getVersions()
*   return a pointer to the beginning of the versions
*     array
****************************************************/
  function getVersions() { return($this->versions); }
/****************************************************
* function countVersions()
*  return a count of the elements in versions
****************************************************/
  function countVersions() { return(count($this->versions)); }
/****************************************************
* function updateDB()
*  updates the DB with the information for this rule
*    first verify that the rules exists or not
*    then either UPDATE or INSERT
****************************************************/
  function updateDB() {
    global $db;
   
//  Figure out if a record exists (UPDATE or INSERT?) 
    $db->add_table(_TABLE_SNORT_RULEBASE);
    $db->clause = " where RID=" . $this->RID;
    $ret = $db->compile_query("SELECT");
    if ($this->debug) { printf("%s<hr>\n", $db->query); }
    if ($ret['return']) {
      $db->run_query();
      if ($db->rows > 0) {
        $type = "UPDATE";
      } else {
        $type = "INSERT";
      }
    }
    $db->clear_res();

// Populate the new INSERT or UPDATE query
    $db->add_table(_TABLE_SNORT_RULEBASE);
    if (strpos($type, "UPDATE") !== false){ 
      $db->clause = " where RID=" . $this->RID; 
    } else { 
      $db->add_data("RID", $this->RID); 
    }
    $db->add_data("Name", $this->Name);
    $db->add_data("Description", $this->Description);
    $db->add_data("Category", $this->Category);
    $db->add_data("Enabled", $this->Enabled);
    foreach($this->versions as $key => $value) {
      $db->add_data($key, $value);
    }

//  Now to compile and execute the query
    $ret = $db->compile_query($type);
    if ($this->debug) { printf("%s<hr>\n", $db->query); }
    if ($ret['return']) {
      $db->run_query();
    }
    $db->clear_res();
    return(true);
  }
/********************************************************************
* function printRule()
*  prints out a copy of the rule to STDIN
********************************************************************/
  function printRule() {
    print ("<!-- *********************************************************************\n");
    printf("* RID: %d\n", $this->RID);
    printf("* Name: %s\n", $this->Name);
    if ($this->debug) { printf("* Description: %s\n", $this->Description); }
    printf("* Category: %s\n", $this->Category);
    printf("* Enabled: %d\n", $this->Enabled);
    printf("* Versions: \n");
    foreach ($this->versions as $key => $value) {
      printf("* %s: %s\n", $key, $value);
    }
    print ("********************************************************************* -->\n\n");
  } 
/********************************************************************
* function retrieveRule()
*  retrieves the rule from the DB based on the RID
*    if the RID is not set the retrieval will fail
********************************************************************/
  function retrieveRule() {
    global $db;

//  Verify that we have a good RID or at least a numeric one
    if (!is_numeric($this->RID)) {
      return(false);
    }
//  Figure out if a record exists (UPDATE or INSERT?) 
    $db->add_table(_TABLE_SNORT_RULEBASE);
    $db->clause = " where RID=" . $this->RID;
    $ret = $db->compile_query("SELECT");
    if ($this->debug) { printf("%s<hr>\n", $db->query); }
    if ($ret['return']) {
      $db->run_query();
      if ($db->rows > 0) {
        $db->step();

//      Add the basic information.. RID is read-only
        $this->setName($db->res['Name']);
        $this->setDescription($db->res['Description']);
        $this->Enabled = $db->res['Enabled'];

//      Add the versions, if they fail then try to update the info
        if (!$this->addVersion("Rule2_0", $db->res['Rule2_0'])) 
          $this->updateVersion("Rule2_0", $db->res['Rule2_0']);
        if (!$this->addVersion("Rule2_1", $db->res['Rule2_1']))
          $this->updateVersion("Rule2_0", $db->res['Rule2_0']);
 
//      Set the Category last so we can clear the db inside it.
        $this->setCategory($db->res['Category']);
      }
    } else {
      $db->clear_res();
      return(false);
    }
    $db->clear_res();
    if ($this->debug) { $this->printRule(); }
    return(true);
  }
/********************************************************************
* function printEdit()
*  prints out a collection of form fields to use when editing this
*    rule's information
*  No required information, all blanks can still return a blank
*    template to use.
*  Uses a template from the $templates hash
********************************************************************/
  function printEdit() {
    global $templates;
    $res = "";

    $res = str_replace("<<RID>>", $this->getRID(), $templates{"edit_rule_details"});
    $res = str_replace("<<NAME>>", $this->getName(), $res);
    $res = str_replace("<<CATEGORY>>", $this->getCategory(), $res);
    $res = str_replace("<<DESCRIPTION>>", $this->getDescription(), $res);
    $res = str_replace("<<ENABLED>>", $this->getEnabled(), $res);
    foreach ($this->getVersions() as $ver => $rule) {
      $res = str_replace("<<".strtoupper($ver).">>", $rule, $res);
    }
    return($res);
  }
}
?>
