<?php
/********************************************************************
Copyright (c) 2004 Rodney Caudle

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	3/9/2004  -  Initial build
********************************************************************/
/********************************************************************
* Class rulebase
*  defines a collections of rules that can be pulled from or added 
*    to a database
********************************************************************/
class rulebase {
  var $rules;		// array of rules
  var $name;		// name of this rulebase
  var $debug;		// debug info?
   
/********************************************************************
* Constructors
********************************************************************/
  function rulebase() {
    $this->rules = array();
  }
/********************************************************************
* function addRule($num)
*  adds a new item of class rule to the rules array and set the RID
*    to the value in $num
********************************************************************/
  function addRule($num) {
    if (is_numeric($num)) {
      if (empty($this->rules{$num})) {
        $this->rules{$num} = new rule;
      }
      $this->rules{$num}->setRID($num);
      return(true);
    } else {
      return(false);
    }
  }
/********************************************************************
* function setName($value)
*  sets the name to whatever is in $value
********************************************************************/
  function setName($value) {
    $this->name = $value;
    return(true);
  }
/****************************************************
* function toggleDebug()
*  set debug to the opposite of whatever it used
*    to be (true -> false, false -> true)
****************************************************/
  function toggleDebug() { $this->debug = !($this->debug); }
/********************************************************************
* function getName()
*  returns the current value of $name
********************************************************************/
  function getName() {
    return($this->name);
  }
/********************************************************************
* function updateDB()
*  updates the DB with the current values stored in the rules array
********************************************************************/
  function updateDB() {
    foreach($this->rules as $key => $value) {
      $this->rules{$key}->updateDB();
      if ($this->debug) { $this->rules{$key}->printRule(); }
    }
  }
/********************************************************************
* function getCount()
*  returns the current count of items in the rules array
********************************************************************/
  function getCount() {
    return(count($this->rules));
  }
/********************************************************************
* function retrieveRulebase()
*  pulls a copy of the current rulebase from the DB for use
********************************************************************/
  function retrieveRulebase() {
    global $db;
    
    $db->add_table(_SNORT_TABLE_RULEBASE);
    $ret = $db->compile_query("SELECT");
    if ($this->debug) { printf("%s<hr>\n", $db->query); }
    if ($ret['return']) {
      $db->run_query();
      for($x=0; $x<$db->rows; $x++) {
        $db->step();
        $this->addRule($db->res['RID']);
        $this->rules{$db->res['RID']}->setDescription($db->res['Description']);
        $this->rules{$db->res['RID']}->setName($db->res['Name']);
        $this->rules{$db->res['RID']}->setCategory($db->res['Category']);
        $this->rules{$db->res['RID']}->setEnabled($db->res['Enabled']);
        $this->rules{$db->res['RID']}->addVersion("Rule2_0", $db->res['Rule2_0']);
        $this->rules{$db->res['RID']}->addVersion("Rule2_1", $db->res['Rule2_1']);
      }
    }
    $db->clear_res();
  }
}
?>
