<?php
/******************************************************************************
*	function host_options($db)
*		returns a list of hosts to choose from
******************************************************************************/
function host_options($db) {
  $db->add_table(_TABLE_HOSTS);
  $db->clause .= "where IsSniffer=1";
  $ret = $db->compile_query("SELECT");
  if($ret['return']) {
    $db->run_query();
    $temp = "";
    for($x=0;$x<$db->rows;$x++) {
      $db->step();
      $temp .= "<option value=\"" . $db->res['HID'] . "\">" . $db->res['HostName'] . "</option>\n";
    }
  } else {
    $temp = "<option>-- NO HOSTS FOUND --</option>\n";
  }
  $db->clear_res();
  return($temp);
}

/******************************************************************************
*	function host_name($db, $hid)
*		returns a hostname for the HID given
******************************************************************************/
function host_name($db, $hid) {
  $db->add_table(_TABLE_HOSTS);
  $db->clause .= "where HID=$hid";
  $ret = $db->compile_query("SELECT");
  if($ret['return']) {
    $db->run_query();
    $db->step();
    $temp = $db->res['HostName'];
  } else {
    $temp = "-- NO HOSTS FOUND --";
  }
  $db->clear_res();
  return($temp);
}

/******************************************************************************
*	function get_mapping($db, $version, $short, $type)
*		returns a listing for the type of  section depending
*		on the version of SNORT passed
*		Either the normal or short version can be chosen
*	TYPE = 'classification', 'preprocessor' or 'outputplugin'
******************************************************************************/
function get_mapping($db, $version, $short, $type) {
  $db->add_table(_TABLE_SNORT_MAPPING);
  $db->clause .= "where Version='$version' AND MTYPE='$type' ORDER BY NORDER";
  $ret = $db->compile_query("SELECT");
  printf("%s<hr>\n", $db->query);
  if($ret['return']) {
    $db->run_query();
    $temp = "";
    for ($x=0;$x<$db->rows; $x++) {
      $db->step();
      if ($short) {
        $temp .= $db->res['Short'] . "\n\n";
      } else {
        $temp .= $db->res['Data'] . "\n\n";
      }
    }
    if ($x == 0) {
      $temp = sprintf("-- NO %s  FOUND --", strtoupper($type));;
    }
  } else {
    $temp = sprintf("-- NO %s  FOUND --", strtoupper($type));;
  }
  $db->clear_res();
  return($temp);
}
/******************************************************************************
*	function preprocessors($db, $version, $short)
*		returns a listing for the preprocessors section depending
*		on the version of SNORT passed
*		Either the normal or short version can be chosen
******************************************************************************/
function preprocessors($db, $version, $short) {
  $db->add_table(_TABLE_SNORT_MAPPING);
  $db->clause .= "where Version='$version' AND MTYPE='preprocessor'";
  $ret = $db->compile_query("SELECT");
  if($ret['return']) {
    $db->run_query();
    $temp = "";
    for ($x=0;$x<$db->rows; $x++) {
      $db->step();
      if ($short) {
        $temp .= $db->res['Short'] . "\n\n";
      } else {
        $temp .= $db->res['Data'] . "\n\n";
      }
    }
    if ($x == 0) {
      $temp = "-- NO PREPROCESSORS FOUND --";
    }
  } else {
    $temp = "-- NO PREPROCESSORS FOUND --";
  }
  $db->clear_res();
  return($temp);
}

/******************************************************************************
*	function preprocessors($db, $version, $short)
*		returns a listing for the preprocessors section depending
*		on the version of SNORT passed
*		Either the normal or short version can be chosen
******************************************************************************/
function outputplugins($db, $version, $short) {
  $db->add_table(_TABLE_SNORT_MAPPING);
  $db->clause .= "where Version='$version' AND MTYPE='output plugin'";
  $ret = $db->compile_query("SELECT");
  if($ret['return']) {
    $db->run_query();
    $temp = "";
    for ($x=0;$x<$db->rows; $x++) {
      $db->step();
      if ($short) {
        $temp .= $db->res['Short'] . "\n\n";
      } else {
        $temp .= $db->res['Data'] . "\n\n";
      }
    }
    if ($x == 0) {
      $temp = "-- NO OUTPUT PLUGINS FOUND --";
    }
  } else {
    $temp = "-- NO OUTPUT PLUGINS FOUND --";
  }
  $db->clear_res();
  return($temp);
}

/******************************************************************************
*	function rulebasesids($db, $version)
*		returns a listing of the available rules to choose from
* 		sorted by category
******************************************************************************/
function rulebaserids($db) {
  $res = array();
  $db->query = "select distinct a.RID,b.Rule2_0,b.Rule2_1 from " . _TABLE_SNORT_RULEBASE .
     " as a, " . _TABLE_SNORT_RULEBASE . " as b where a.RID=b.RID";
  $db->run_query();
  for ($x=0; $x<$db->rows; $x++) {
    $db->step();
    $tmp = $db->res['RID'];
    $res["$tmp"] = $db->res['Rule2_1'];
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function processChoice($db, $hid, $rid, $val)
*		add the chosen RID to the DB 
******************************************************************************/
function processChoice($db, $hid, $rid, $val) {
//  echo "Checking rule #$rid ... ";
  $db->add_table(_TABLE_SNORT_HOST_MAPPING);
  $db->clause .= " where HID=$hid and RID=$rid";
  $ret = $db->compile_query("SELECT");
  if ($ret) { $db->run_query(); }
  $tmp = $db->rows;
//  echo "$tmp <hr>\n";
  $db->clear_res();
//  Set the link up if it's not there and chosen
  if (($tmp == "0") && ($val)) {
//    echo "Enabling rule #$rid<hr>\n";
    $db->query = "insert into " . _TABLE_SNORT_HOST_MAPPING . " VALUES ($rid,$hid)";     
    $db->run_query();
    $db->clear_res();
    return($db->sqlResults);
//  Clear the link if it's there and not chosen
  } else if (($tmp > 0) && (!$val)) {
//    echo "Clearing rule #$rid<hr>\n";
    $db->query = "delete from " . _TABLE_SNORT_HOST_MAPPING . " where HID=$hid and RID=$rid";
    $db->run_query();
    $db->clear_res();
    return($db->sqlResults);
  } else {
//  Do nothing... either the link is there and chosen or the link isn't there and not chosen
  }
  $db->clear_res();
  return(false); 
}
/******************************************************************************
*	function rulebaseselect($db, $version, $rulebaseline, $rulebasehead)
*		returns a listing of the available rules to choose from
* 		sorted by category
******************************************************************************/
function rulebaseselect($db, $version, $hid, $rulebaseline, $rulebasehead) {
// Need a second connection
  $db2 = new conn_mySQL(); 
  $db2->db_use_conn($db->dbc, $db->db);
  $temp = "";
  $chosen = rulebasechoices($db,$hid);

  $db->query = "select distinct Category from " . _TABLE_SNORT_RULEBASE . " where Enabled=1";
  $db->run_query();
  for ($x=0;$x<$db->rows; $x++) {
    $db->step();
    $temp .= str_replace("<<CATEGORY>>", $db->res['Category'], $rulebasehead) . "\n";
    $temp2 = "";
    $db2->add_table(_TABLE_SNORT_RULEBASE);
    $db2->clause .="where Enabled=1 and Category='" . $db->res['Category'] . "'";
    $ret = $db2->compile_query("SELECT");
    if($ret['return']) {
      $db2->run_query();
      for ($y=0;$y<$db2->rows; $y++) {
        $db2->step();  $tmp = $db2->res['RID'];
        $temp2 .= str_replace("<<NAME>>",$db2->res['Name'],$rulebaseline) . "\n";
        $temp2 = str_replace("<<ID>>",$tmp,$temp2);
        if (isset($chosen["$tmp"])) {
          $temp2 = str_replace("<<SELECTED>>", " checked", $temp2);
        } else {
          $temp2 = str_replace("<<SELECTED>>", "", $temp2);
        }
      }
      $db2->clear_res();
    } else {
      $temp2 = "<tr><td colspan=\"2\" class=\"errmsg\">-- NO RULES FOUND --</td></tr>";
    }
    $temp = str_replace("<<RULES>>", $temp2, $temp);
    $temp2 = "";
  }
  if ($x == 0) {
    $temp = "<tr><td colspan=\"2\" class=\"errmsg\">-- NO RULES FOUND --</td></tr>";
  }
  $db->clear_res();
  return($temp);
}

/******************************************************************************
*	function view_rule_details($db, $rid, $rulebasedetails)
*		returns the details of a rule
******************************************************************************/
function view_rule_details($db,$rid,$rulebasedetails) {
  $db->add_table(_TABLE_SNORT_RULEBASE);
  $db->clause .= "where RID=$rid";
  $ret = $db->compile_query("SELECT");
  if($ret['return']) {
    $db->run_query();
    for ($x=0;$x<$db->rows; $x++) {
      $db->step();
      $temp .= str_replace("<<ID>>", $db->res['RID'], $rulebasedetails);
      $temp = str_replace("<<NAME>>", $db->res['Name'], $temp);
      $temp = str_replace("<<CATEGORY>>", $db->res['Category'], $temp);
      $temp = str_replace("<<ENABLED>>", $db->res['Enabled'], $temp);
      $temp = str_replace("<<DESCRIPTION>>", $db->res['Description'], $temp);
      $temp = str_replace("<<RULE20>>", $db->res['Rule2_0'], $temp);
      $temp = str_replace("<<RULE21>>", $db->res['Rule2_1'], $temp);
    }
  } else {
    $temp = "<span class=\"errormsg\">-- NO DETAILS FOUND --</span>";
  }
  $db->clear_res();
  return($temp);
}

/******************************************************************************
*	function rulebasechoices($db, $hid)
*		returns a list of rids for a chosen host
******************************************************************************/
function rulebasechoices($db, $hid) {
  $res = array();
  $db->add_table(_TABLE_SNORT_HOST_MAPPING);
  $db->clause .= " where HID=$hid";
  $ret = $db->compile_query("SELECT");
  if ($ret) { 
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
      $tmp = $db->res['RID'];
      $res["$tmp"] = $tmp;
    }
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function updateConfiguration($db, $hid, $config, $rules)
*		updates the configuration and rules for a selected hostid
******************************************************************************/
function updateConfiguration($db, $hostid, $config, $rules) {
  $db2 = new conn_mySQL(); 
  $db2->db_use_conn($db->dbc, $db->db);

  foreach ($config as $key => $value) {
    $db->add_table(_TABLE_SNORT_SECTIONS);
    $db->clause .= " where HID=$hostid and Section='$key'";
    $ret = $db->compile_query("SELECT");
    //echo $db->query . "<hr>\n";
    if ($ret['return']) {
      $db->run_query();
      if ($db->rows > 0) {
        $db2->query = "UPDATE " . _TABLE_SNORT_SECTIONS . " set Details='" . 
            mysql_escape_string($value) . "' where HID=$hostid and Section='$key'";
      } else {
        $db2->query = "INSERT INTO " . _TABLE_SNORT_SECTIONS . " VALUES($hostid,'$key','" . 
            mysql_escape_string($value) . "')";
      }
      //echo $db2->query . "<hr>\n";
      $db2->run_query();
    }
    $db2->clear_res();
    $db->clear_res();
  }
}

/******************************************************************************
*	function processNewRule()
*		displays screen for creating a new rule	
******************************************************************************/
function processNewRule() {
  global $db; 		// database connection
  global $rid;
  global $rname;
  global $rdesc;
  global $rcat; 
  global $rules2_0;
  global $rules2_1;
  $nextnum = 1000000;
  $present = false;

  if (strlen($rid) <= 0) {
    $db->add_table(_TABLE_SNORT_RULEBASE);
    $db->clause .= " where RID >= 1000000";
    $ret = $db->compile_query("SELECT");
  //  printf("%s<hr>\n", $db->query);
    if ($ret['return']) {
      $db->run_query();
      $nextnum += $db->rows;
    }
    $db->clear_res();
  } else {
    $db->add_table(_TABLE_SNORT_RULEBASE);
    $db->clause = " where RID = $rid";
    $ret = $db->compile_query("SELECT");
    if ($ret['return']) {
      $db->run_query();
      if ($db->rows > 0) { $present = true; } else { $present = false; }
    }
    $db->clear_res();
  }
  
  $db->add_table(_TABLE_SNORT_RULEBASE);
  if (!$present) {
    if (strlen($rid) > 0) { $db->add_data("RID", $rid); } 
     else { $db->add_data("RID", $nextnum); }
    $type = "INSERT";
  } else {
    $db->clause = " where RID=$rid";
    $type = "UPDATE";
  }
  $db->add_data("Name", $rname);
  $db->add_data("Description", $rdesc);
  $db->add_data("Category", $rcat);
  $db->add_data("Rule2_0", $rules2_0); 
  $db->add_data("Rule2_1", $rules2_1); 
  $db->add_data("Enabled", 1);
  $ret = $db->compile_query("$type");
  printf("%s<hr>\n", $db->query);
  if ($ret['return']) {
    $db->run_query();
    return(true);
  } else {
    return(false);
  }
  return(false);
}

/******************************************************************************
*	function processNewGroup()
*		saves the information for a new group entry	
******************************************************************************/
function processNewGroup() {
  global $db; 		// database connection
  global $gname;
  global $gdesc;
  $nextnum = 1000000;

  $db->add_table(_TABLE_SNORT_GROUPS);
  $db->clause .= " where GID >= 1000000";
  $ret = $db->compile_query("SELECT");
//  printf("%s<hr>\n", $db->query);
  if ($ret['return']) {
    $db->run_query();
    $nextnum += $db->rows;
  }
  $db->clear_res();

  $db->add_table(_TABLE_SNORT_GROUPS);
  $db->add_data("GID", $nextnum);
  $db->add_data("Name", $gname);
  $db->add_data("Description", $gdesc);
  $ret = $db->compile_query("INSERT");
//  printf("%s<hr>\n", $db->query);
  if ($ret['return']) {
    $db->run_query();
    return(true);
  } else {
    return(false);
  }
  return(false);
}

/******************************************************************************
*	function get_section($db, $hid, $section)
*		get a listing of the variables from the database if one was saved
*		otherwise use the default statement.
*
*	Available sections: VARIABLES, OUTPUT, PREPROC
******************************************************************************/
function get_section($db, $hostid, $section) {
  global $default_variables;
  global $short;
  global $version;
  $tmp = NULL;

  $db->add_table(_TABLE_SNORT_SECTIONS);
  $db->clause .= " where HID=$hostid and Section='$section'";
  $ret = $db->compile_query("SELECT");
//  echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    if ($db->rows > 0) {
      $db->step();
      $tmp = $db->res["Details"];
    }
  }
  $db->clear_res();
  if ($tmp == NULL) {
    switch ($section) {
      case _SECTION_VAR:
        $tmp = $default_variables;
        break;
      case _SECTION_PREPROC:
        //$tmp = preprocessors($db,$version,$short);
        $tmp = get_mapping($db,$version,$short,'preprocessor');
        break;
      case _SECTION_OUTPUT:
        //$tmp = outputplugins($db,$version,$short);
        $tmp = get_mapping($db,$version,$short,'output plugin');
        break;
      case _SECTION_CLASS:
        $tmp = get_mapping($db,$version,$short,'classification');
        break;
      default:
        $tmp = NULL;
    }
  }
//  echo $tmp . "<hr>\n";
  return($tmp);
}

/******************************************************************************
*	function save_section($db, $hid, $section)
* 		saves a section of information for retrieval later
*
*	Available sections: VARIABLES, OUTPUT, PREPROC
******************************************************************************/
function save_section($db, $hostid, $section, $details, $order) {
  $db->add_table(_TABLE_SNORT_SECTIONS);
  $db->clause .= " where HID=$hostid and Section='$section'";
  $ret = $db->compile_query("SELECT");
  //echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    if ($db->rows > 0) {
      $query = "UPDATE " . _TABLE_SNORT_SECTIONS . " set Details='" . 
          mysql_escape_string($details) . "' where HID=$hostid and Section='$section'";
    } else {
      $query = "INSERT INTO " . _TABLE_SNORT_SECTIONS . " VALUES($hostid,'$section','" . 
          mysql_escape_string($details) . "', $order)";
    }
    $db->clear_res();

    $db->query = $query;
    //echo $db->query . "<hr>\n";
    $db->run_query();
  }
  $db->clear_res();

  $db->add_table(_TABLE_SNORT_STATUS);
  $db->clause = " where HID=$hostid";
  $ret = $db->compile_query("SELECT");
  if ($ret['return']) {
    $db->run_query();
    $count = $db->rows;
  }
  $db->clear_res();
  
  if ($count == 0) {
    $db->query = sprintf("INSERT INTO %s (HID,Status,LastUpdate) VALUES (%s,'PARTIALLY_CONFIGURED',NOW())",
      _TABLE_SNORT_STATUS, $hostid);
  } else {
    $db->query = sprintf("UPDATE %s set Status='PARTIALLY_CONFIGURED',LastUpdate=NOW() where Status < 'PARTIALLY_CONFIGURED' and HID=%d", _TABLE_SNORT_STATUS, $hostid);
  }
  $db->run_query();
  $db->clear_res();
  
  return;
}
/******************************************************************************
*	function updateStatus($hostid)
*		updates the status in the DB to match the current state of 
*		configuration
******************************************************************************/
function update_status($hostid) {
  global $db;
  $STATUS = "";

  $db->query = sprintf("SELECT count(*) from %s where HID=%s", _TABLE_SNORT_SECTIONS, $hostid);
  $db->run_query();
  if ($db->rows > 0) {
    $count = $db->res[0];
  }
  $db->clear_res();

// First check the number of Sections
  if ($count == 0) {
    $STATUS = "NO_CONFIGURATION";
  } elseif ($count < 4) {
    $STATUS = "PARTIALLY_CONFIGURED";
  } else {

//  Now check the number of rules enabled
    $db->query = sprintf("SELECT count(*) from %s where HID=%s", _TABLE_SNORT_HOST_MAPPING, $hostid);
    $db->run_query();
    if ($db->rows > 0) {
      $count = $db->res[0];
    }
    $db->clear_res();

    if ($count > 1) {
      $STATUS = "FULLY_CONFIGURED";
    } else {
      $STATUS = "PARTIALLY_CONFIGURED";
    }
  }

  $db->query = sprintf("UPDATE %s set Status='%s',LastUpdate=NOW() where HID=%d", _TABLE_SNORT_STATUS, $STATUS, $hostid);
  $db->run_query();
  $db->clear_res();

  return($STATUS);
}
/******************************************************************************
*	function rulebasegroups($db)
*		returns a listing of the available rule categories 
******************************************************************************/
function chosenrulebasegroups($db) {
  $db->query = "select distinct a.Category,b.GID from " . _TABLE_SNORT_RULEBASE . " as a, " . _TABLE_SNORT_GROUPS
    . " as b where a.Category = b.Name order by a.Category";
//  $db->add_table(_TABLE_SNORT_RULEBASE);
//  $db->add_select_item("distinct Category, GID");
//  $ret = $db->compile_query("SELECT");
  //echo $db->query . "<hr>\n";
//  if ($ret['return']) {
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
      $tmp = $db->res['Category'];
      $res["$tmp"] = $db->res['GID'];
    }
//  }
  $db->clear_res();
  return($res);
}
/******************************************************************************
*	function selectedrulebasegroups($db, $hid)
*		returns a listing of the selected rule categories for $hid
******************************************************************************/
function selectedrulebasegroups($db, $hid) {
  $db->query = "select distinct b.Category from " . _TABLE_SNORT_HOST_MAPPING . " as a, " . _TABLE_SNORT_RULEBASE
    . " as b where a.RID=b.RID and a.HID=$hid";
  $db->run_query();
//  echo $db->query . "<hr>\n";
  for ($x=0; $x<$db->rows; $x++) {
    $db->step();
    $tmp = $db->res['Category'];
    $res["$tmp"] = $tmp;
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function rulebasegroups($limited)
*		returns a listing of the available rule group names
* 		if ($limited) then limit the returned names to custom groups
*			only (GID >= 1000000)
******************************************************************************/
function rulebasegroups($limited) {
  global $db;
  global $templates;
  $res = "";

  $db->add_table(_TABLE_SNORT_GROUPS);
  $db->add_select_item("distinct Name");
  if ($limited) {
    $db->clause .= " where GID >= 1000000 ORDER BY Name";
  } else {
    $db->clause .= " ORDER BY Name";
  }
  $ret = $db->compile_query("SELECT");
  //echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
      $res .= str_replace("<<NAME>>", $db->res['Name'], $templates{"new_rule_groupline"});
      if ($db->res['Name'] == _DEFAULT_CUSTOM_SNORT_GROUP) {
        $res = str_replace("<<SELECTED>>", " selected", $res);
      } else {
        $res = str_replace("<<SELECTED>>", "", $res);
      }
    }
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function rulebasegids($limited)
*		returns a list of the available group IDs
*		if ($limited) then limit the returned IDs to custom groups
*			only (GID >= 1000000)
******************************************************************************/
function rulebasegids($limited) {
  global $db;
  global $templates;
  $res = array();

  $db->add_table(_TABLE_SNORT_GROUPS);
  $db->add_select_item("GID");
  $db->add_select_item("Name");
  if ($limited) {
    $db->clause .= " where GID >= 1000000 ORDER BY GID";
  } else {
    $db->clause .= " ORDER BY GID";
  }
  $ret = $db->compile_query("SELECT");
//  echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
//      echo "Adding " . $db->res['Name'] . "[" . $db->res['GID'] . "]<hr>\n";
      $tmp = $db->res['GID'];
      $res{"$tmp"} = $db->res['Name'];
    }
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function view_rule_group_details($db, $group, $rulegroupdetails)
*		returns a listing of the selected rule categories for $hid
******************************************************************************/
function view_rule_group_details($db,$group) {
  global $templates;
  $tmp = $templates{"norulegroup"};

  $db->add_table(_TABLE_SNORT_GROUPS);
  $db->clause .= " where NAME='$group'";
  $ret = $db->compile_query("SELECT");
  //echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    if ($db->rows > 0) {
      $db->step();
      $tmp = $templates{"rulegroupdetails"};
      $tmp = str_replace("<<ID>>", $db->res['GID'], $tmp);
      $tmp = str_replace("<<NAME>>", $db->res['Name'], $tmp);
      $tmp = str_replace("<<DESCRIPTION>>", $db->res['Description'], $tmp);
    }
  } 
  $db->clear_res();
  return($tmp);
}

/******************************************************************************
*	function grouprules($group)
*		returns a list of rules for a specific category (group) 
******************************************************************************/
function grouprules($group) {
  global $db;

  $db->query = "select a.RID,a.Name from " . _TABLE_SNORT_RULEBASE . " as a, " . _TABLE_SNORT_GROUPS 
    . " as b where a.Category=b.Name and b.GID=$group";
//  echo $db->query . "<hr>\n";
  $db->run_query();
  for($x=0;$x<$db->rows;$x++) {
    $db->step();
    $tmp = $db->res['RID'];
    $res{"$tmp"} = $db->res['Name'];
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function groupschosen()
*		returns the first group found if there are any instances of
*                 group_GID in the $POST array... otherwise return -1
******************************************************************************/
function groupschosen() {
  global $_POST;

  $res = array();
  foreach ($_POST as $key => $value) {
//    echo "$key => $value ";
    if (strpos($key,"group_") !== FALSE) {
      $tmp = substr($key, 6);
      $res[] = $tmp;
    }
//    echo "<hr>\n";
  }
  return($res);  // return the first group_ ID that you find
}

/******************************************************************************
*	function rulebasechoices($db, $hid)
*		returns a list of rids for a chosen host
******************************************************************************/
function savedrules($hid,$group) {
  global $db;

  $res = array();
  $db->add_table(_TABLE_SNORT_HOST_MAPPING);
  $db->add_table(_TABLE_SNORT_RULEBASE);
  $db->add_table(_TABLE_SNORT_GROUPS);
  $db->clause .= " where " . _TABLE_SNORT_HOST_MAPPING . ".HID=$hid and " . _TABLE_SNORT_HOST_MAPPING
    . ".RID=" . _TABLE_SNORT_RULEBASE . ".RID and " . _TABLE_SNORT_RULEBASE . ".Category=" . _TABLE_SNORT_GROUPS
    . ".NAME and " . _TABLE_SNORT_GROUPS . ".GID=$group";
  $ret = $db->compile_query("SELECT");
//  echo "query = " . $db->query . "<hr>\n";
  if ($ret) { 
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
      $tmp = $db->res['RID'];
      $res["$tmp"] = $tmp;
    }
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function rulestoedit($limited)
*		returns a list of rules which can be edited... 
* 		if limited is set to true then only the custom rules are 
*			available to edit
******************************************************************************/
function rulestoedit($limited) {
  global $db;
  global $templates;
  $res = "";

  $db->add_table(_TABLE_SNORT_RULEBASE);
  $db->add_select_item("RID");
  $db->add_select_item("Name");
  $db->add_select_item("Category");
  if ($limited) {
    $db->clause .= " where RID >= 1000000 ORDER BY Category,RID";
  } else {
    $db->clause .= " ORDER BY Category,RID";
  }
  $ret = $db->compile_query("SELECT");
//  echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
//      echo "Adding " . $db->res['Name'] . "[" . $db->res['GID'] . "]<hr>\n";
      $res .= str_replace("<<RID>>", $db->res['RID'], $templates{"edit_rule_line"});
      $res = str_replace("<<NAME>>", $db->res['Name'], $res);
      $res = str_replace("<<CATEGORY>>", $db->res['Category'], $res);
      $res .= "\n";
    }
  }
  $db->clear_res();
  return($res);
}

/******************************************************************************
*	function hostsstatus()
*		Returns the text displaying the status of the sensors
******************************************************************************/
function hostsstatus() {
  global $db;
  global $templates;
  $res = "";

  $db->add_table(_TABLE_SNORT_STATUS);
  $db->add_table(_TABLE_HOSTS);
  $db->add_select_item(Array(_TABLE_HOSTS => Array("HID", "HostName", "IPAddr"),
    _TABLE_SNORT_STATUS => Array("Status", "LastUpdate", "LastSynch", "AutoUpdate")));
  $db->clause = sprintf("WHERE %s.HID=%s.HID ORDER BY %s.HID", _TABLE_SNORT_STATUS, _TABLE_HOSTS, _TABLE_SNORT_STATUS);
  $ret = $db->compile_query("SELECT");
//  echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
      $res .= str_replace("<<HID>>", $db->res['HID'], $templates{"view_status_hostline"});
      $res = str_replace("<<NAME>>", $db->res['HostName'], $res);
      $res = str_replace("<<STATUS>>", $db->res['Status'], $res);
      $res = str_replace("<<LASTUPDATE>>", $db->res['LastUpdate'], $res);
      $res = str_replace("<<LASTSYNCH>>", $db->res['LastSynch'], $res);
      $res = str_replace("<<AUTOUPDATE>>", $db->res['AutoUpdate'], $res);
      $res = str_replace("<<IPADDR>>", $db->res['IPAddr'], $res);
      
    }
  }
  return($res);
}

/******************************************************************************
*	function hostchoices()
*		Returns the text of options for deleting a host
******************************************************************************/
function hostschoices() {
  global $db;
  global $templates;
  $res = "";

  $db->add_table(_TABLE_HOSTS);
  $db->clause = "where IsSniffer=1";
  $ret = $db->compile_query("SELECT");
//  echo $db->query . "<hr>\n";
  if ($ret['return']) {
    $db->run_query();
    for ($x=0; $x<$db->rows; $x++) {
      $db->step();
      $res .= str_replace("<<HID>>", $db->res['HID'], $templates{"delete_host_line"});
      $res = str_replace("<<NAME>>", $db->res['HostName'], $res);
    }
  }
  return($res);
} 

/******************************************************************************
*	function configclear()
*		Clears the configuration of the hostid passed through 
*			- clear all HID_x_RID associations
*			- clear all Sections
*			- set status to NO_CONFIGURATION
******************************************************************************/
function configclear($hostid) {
  global $db;
  global $templates;

// Set the status to NO_CONFIGURATION to stop updates for that host
  $db->query = sprintf("UPDATE %s set Status='NO_CONFIGURATION',LastUpdate=NOW() where HID=%d", _TABLE_SNORT_STATUS, $hostid);
  $db->run_query();
  $db->clear_res();

// Clear all the sections
  $db->query = sprintf("DELETE from %s where HID=%d", _TABLE_SNORT_SECTIONS, $hostid);
  $db->run_query();
  $db->clear_res();

// Clear all the HID_x_RID associations
  $db->query = sprintf("DELETE from %s where HID=%d", _TABLE_SNORT_HOST_MAPPING, $hostid);
  $db->run_query();
  $db->clear_res();
  
  return($templates{"delete_success"});
}

function setStatus($hostid, $status) {
  global $db;

// Set the status to NO_CONFIGURATION to stop updates for that host
  $db->query = sprintf("UPDATE %s set Status='%s',LastUpdate=NOW() where HID=%d", _TABLE_SNORT_STATUS, $status, $hostid);
  $db->run_query();
  $db->clear_res();

  return(true);
}
?>
