<?php
/********************************************************************
Copyright (c) 2004 Rodney Caudle

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	3/7/2004  -  Initial build

********************************************************************/
include "../../inc/common.php";
include _ABS_ROOT . "/lib/mysql.inc.php";
include "class_rule.php";
include "class_rulebase.php";

/********************************************************************
* Definitions
********************************************************************/
$RULESDIR = "/home/rcaudle/downloads/snort/snort-2.1.0/rules";
$DOCSDIR = "/home/rcaudle/downloads/snort/snort-2.1.0/doc/signatures";

define("_TABLE_SNORT_RULEBASE", "MIDAS_SNORT_Rulebase");
define("_TABLE_SNORT_SECTIONS", "MIDAS_SNORT_Sections");
define("_TABLE_SNORT_HOST_MAPPING", "MIDAS_SNORT_HID_x_RID");
define("_TABLE_SNORT_MAPPING", "MIDAS_SNORT_Mapping");
define("_TABLE_SNORT_GROUPS", "MIDAS_SNORT_Groups");
define("_SECTION_VAR", "VARIABLES");
define("_SECTION_PREPROC", "PREPROC");
define("_SECTION_OUTPUT", "OUTPUT");
define("_DEFAULT_CUSTOM_SNORT_GROUP", "CUSTOM-RULES");

// Empty ruleset to start with
$rules = array();

/****************************************************
* function clean($item, $key)
*  Parse and organize the rules into the $rulebase
*    variable.
*
*  Tokens Needed:
*    RID - unique rule ID  "sid:"
*    Name - descriptive short name for a rule "msg:"
*    Category - where did the file come from 
****************************************************/
function clean($item, $key) { 
  global $tmpcat;
  global $rulebase;
  global $db;

// test for the presence of all the needed information
  if ((strpos($item, "sid:") !== false) && (strpos($item, "msg:") !== false)) { 
    if ($rulebase->debug) { printf("Parsing line: %s\n", $item); }

//  Parse out the SID and Name
    foreach (split("[(;]", $item) as $key => $value) {
      if (strpos($value, "sid:") !== false) {
        $tmpsid = trim(str_replace("\"", "", str_replace("sid:", "", $value)));
      } else if (strpos($value, "msg:") !== false) {
        $tmpname = trim(str_replace("\"", "", str_replace("msg:", "", $value)));
      }
    }

//  Create a new item and populate with known information
    $rulebase->addRule($tmpsid);
    if (! $rulebase->rules{$tmpsid}->setCategory(strtoupper($tmpcat)))
      printf("ERROR: %s unknown category!\n", strtoupper($tmpcat));
    $rulebase->rules{$tmpsid}->setName($tmpname);
    $rulebase->rules{$tmpsid}->addVersion("Rule2_1", $item);
    $rulebase->rules{$tmpsid}->toggleEnabled();

//  Print a little info
    if ($rulebase->debug) { 
      printf("Parsed Name: %s\n Parsed SID: %s\n--\n", $tmpname, $tmpsid);
      $rulebase->rules{$tmpsid}->printRule();
    }
  }
}
/****************************************************
* Initialize the DB connection
****************************************************/
$db = new conn_mysql($credentials);
$db->db_new_conn();

/****************************************************
* Initialize the Rulebase
****************************************************/
$rulebase = new rulebase;
$tmprules = array();
$oldval = 0;

/********************************************************************
* Open the rules directory and import the raw rules
********************************************************************/
if ($rdir = opendir($RULESDIR)) {
  while (false !== ($file = readdir($rdir))) {

//  Don't use the . and .. files
    if (strcmp(".", $file) && strcmp("..", $file)) {

//    Parse out the extension for the file, but don't shrink it
      $tag = ereg_replace(".*\.([^\.]*)$", "\\1", $file);

//    Remove "."s from the remaining filename
      $tmpcat = str_replace(".", "", basename($file, $tag));

//    If a file with the extension ".rules" is found
      if (strpos($file,".rules") !== FALSE) {
        if ($rulebase->debug) {
          printf("******************************************************\n");
          printf("Starting category: %s\n", strtoupper($tmpcat));
        }
       
//      Open the file for read-only 
        if (false !== ($handle = fopen("$RULESDIR/$file", "r"))) {
          $tmpbuff = "";  $count = 1;
          while (!feof($handle)) { 
            $tmpbuff .= fgets($handle, 4096); 
            $count++;
          }
          fclose($handle);

//        Split it into an array based on the \n or \r\n characters
          $tmparr = split("\n", $tmpbuff);

//        Clean out the empty lines and comments
          array_walk($tmparr, 'clean');

//        Count the lines
          if ($rulebase->debug) { printf("******************************************************\n\n"); }
          $oldval = count($tmprules);
        }
      }
    }
  }
  closedir($rdir);
  if ($rulebase->debug) { printf("%d rules found!\n", count($tmprules)); }
} else {
  printf("Could not open %s\n", $RULESDIR);
  exit;
}
  
/****************************************************
* Open the docs directory and import the descriptions
****************************************************/
if ($ddir = opendir($DOCSDIR)) {
  while (false !== ($file = readdir($ddir))) {

//  Don't use the . and .. files
    if (strcmp(".", $file) && strcmp("..", $file)) {

//    Parse out the extension for the file, but don't shrink it
      $tag = ereg_replace(".*\.([^\.]*)$", "\\1", $file);

//    Remove "."s from the remaining filename
      $tmpname = str_replace(".", "", basename($file, $tag));

//    Handle only if the filename is numeric and extension is .txt
      if (is_numeric($tmpname) && (false !== strcmp($tag,"txt"))) {
//      Read in the description file for use later as the Description
        $desc = "";
        if (false !== ($handle = fopen("$DOCSDIR/$file", "r"))) {
          while (!feof($handle)) {
            $desc .= fgets($handle, 4096);
          }
          fclose($handle);
       }

//      Mangle the description appropriately to format it as HTML
        $desc = htmlspecialchars($desc);
        $desc = nl2br($desc);
        $desc = sprintf("<p><span class=\"bodysmall\">%s</span></p>", $desc);

//      Create a new rule_item and populate information
        $rulebase->addRule($tmpname);
        $rulebase->rules{$tmpname}->setRID($tmpname);
        if (! $rulebase->rules{$tmpname}->setDescription($desc))
          printf("ERROR: couldn't set Description for rule ID: %d\n", $tmpname);
//      Log a message for discussion
        if($rulebase->debug) { $rulebase->rules{$tmpname}->printRule(); }
      }
    }
  }
  closedir($ddir);
} else {
  printf("Could not open %s\n", $DOCSDIR);
  exit;
}

// Give a final count!
if ($rulebase->debug) { printf("%d rules found.\n", $rulebase->getCount()); }

// Push information to the DB
$rulebase->toggleDebug();
$rulebase->updateDB();

/****************************************************
* Close the DB connection
****************************************************/
$db->db_close();


?>
