/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*	Applisation : MIDASa
*	Filename : MIDASa.c
*		MIDASa.c contains the primary source code for the MIDASa app
*		(The Alert daemon). 
*
*	Updated :
*		11/06/2003 - Added function and header comments.
*		12/16/2003 - Added CatchSignal function.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "alert_email.h"
#include "alert_external.h"

struct XMLLeaf * ConfigXMLTree = 0;
struct SQLCFG * SQLCFG;

int LOGTOSTDERR = 0;
int DEBUGLEVEL = 0;
int LOG_XML 	= 1;
int LOG_INFO 	= 2;
int LOG_CHECK	= 4;
int LOG_FORK 	= 8;
int LOG_SMTP	= 16;
int LOG_SOCKET	= 32;
int LOG_SQL 	= 64;

char * KEYFILE = 0;
char APPNAME[] = "MIDASa";
char * CFGPATH = 0;

unsigned MAXALERTS = 200;

/******************************************************************************
*	int main(int argc, char * argv[])
*		The main() application starting function
*	Args :
*		int arc		The Argument count.
*		int *argv[]	The Argunment value array.
*	
*       Updated :
******************************************************************************/
int main(int argc, char *argv[])
{
	unsigned MaxAID = 0;
	MYSQL mysql;

	(void) signal(SIGHUP, CatchSignal);
	(void) signal(SIGKILL, CatchSignal);
	(void) signal(SIGTERM, CatchSignal);
		
	CFGPATH = dsprintf(CFGPATH, "%s/etc/MIDASa.cf", APPROOT);
	ConfigXMLTree = FileToXMLTree(CFGPATH);
		
	if(!ConfigXMLTree)
	{
		printf("! Unable to read %s config file.\n", CFGPATH);
		exit(0);
	}

	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
	
	SQLCFG = (struct SQLCFG *) malloc(sizeof(struct SQLCFG));
	
	SQLCFG->IP = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.IP");
	SQLCFG->user = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.USER");
	SQLCFG->passwd = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.PASSWD");
	SQLCFG->db = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.DB");
	SQLCFG->unixsocket =  ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.UNIXSOCKET");
	SQLCFG->port = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SQL.port"));
	
	do
	{
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[MIDASa]-[Start Loop]");
		
		dbConnect(&mysql, SQLCFG);
		
		MaxAID = ReturnMaxAID(&mysql);
		
		if(MaxAID)
		{
			ProcessAlerts(&mysql);
			PurgeAlertQueue(&mysql, MaxAID);
		} else {
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[MIDASa]-[No Alerts to process]");
		}
		
		dbClose(&mysql);
		
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[MIDASa]-[End Loop]");
		
		if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
		{
			sleep(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.LOOPTIME")));
		}
	}while(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")));
	
	free(SQLCFG);
	CFGPATH = dfree(CFGPATH);
		
	exit(0);	
}

/******************************************************************************
*	void ProcessAlert(void)
*		Main function to process queued alerts. Alerts are stored in
*		the MIDAS_Alert_Queue table.
*
*	Updated :
******************************************************************************/
void ProcessAlerts(MYSQL * mysql)
{
        struct DBR * dbr = 0;
        struct DBR * dbr2 = 0;
        struct DBR * dbr3 = 0;
        MYSQL_ROW myrow;
        MYSQL_ROW myrow2;
        MYSQL_ROW myrow3;
 	char * query = 0;
        unsigned STARTOFDAY = 0;
        unsigned INTTIME = 0;
        unsigned i, ii;
        int AlertType = 0;
        struct XMLLeaf * AlertLeaf = 0;

	char SELECT[] = "SELECT MIDAS_Alert_Contacts.AlertType, MIDAS_Alert_Contacts.AlertOption FROM MIDAS_Alert_Contacts "
			"LEFT JOIN MIDAS_Alert_Contact_Groups ON (MIDAS_Alert_Contact_Groups.ACID = MIDAS_Alert_Contacts.ACID) "
			"LEFT JOIN MIDAS_Hosts ON (MIDAS_Alert_Contact_Groups.AGID = MIDAS_Hosts.AGID) "
			"WHERE (MIDAS_Alert_Contacts.OnTime + %ld) <= '%ld' AND "
			"(MIDAS_Alert_Contacts.OnTime + MIDAS_Alert_Contacts.Offtime + %ld) >= '%ld' AND "
			"MIDAS_Hosts.HID='%s'";
	
        MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessAlerts]-[Enter]");
        
	dbr = dbQuery_P(mysql, "SELECT EID FROM MIDAS_Alert_Queue");
	
	if(dbr)
	{
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessAlerts]-[Alerts to Process:%ld]", dbr->num);
		
		for(i=0;i<dbr->num;i++)
		{
			myrow = dbResult(dbr);
			
			query = dsprintf(query, "SELECT HID, CID, Status, Response, TimeStamp, TID FROM MIDAS_Check_Events WHERE EID='%s'", myrow[0]);
			dbr2 = dbQuery_P(mysql, query);
			query = dfree(query);
			
			if(dbr2)
			{
				if(dbr2->num)
				{
					myrow2 = dbResult(dbr2);
					
					STARTOFDAY = StartOfDay();
					INTTIME = CurrentHour() + STARTOFDAY;
		
					query = dsprintf(query, SELECT, STARTOFDAY, INTTIME, STARTOFDAY, INTTIME, myrow2[0]);
					dbr3 = dbQuery_P(mysql, query);
					query = dfree(query);
					
					if(dbr3)
					{
						for(ii=0;ii<dbr3->num;ii++)
						{
							myrow3 = dbResult(dbr3);
					
							AlertType = Myatoi(myrow3[0]);
							AlertLeaf = StringToXMLTree(myrow3[1]);

							MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessAlert]-[AlertType:%d]", AlertType);
								
							switch(AlertType)
							{
								case 0:	//Pager Alert
									Alert_Page(mysql, AlertLeaf, myrow2[0], myrow2[1], myrow2[2], myrow2[3], myrow2[4]);
									break;
								case 1: //Email Alert
									Alert_Email(mysql, AlertLeaf, myrow2[0], myrow2[1], myrow2[2], myrow2[3], myrow2[4]);
									break;
								case 2: //External
									Alert_External(mysql, AlertLeaf, myrow2[0], myrow2[1], myrow2[2], myrow2[3], myrow2[4]);
									break;
							}
							AlertLeaf = DelXMLLeafTree(AlertLeaf);
						}
					}
					dbr3 = dbFree(dbr3);			
				} else {
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessAlerts]-[Alert Record not found : %s]", myrow[0]);
				}
			}
			dbr2 = dbFree(dbr2);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessAlerts]-[Exit]");
	
	dbr = dbFree(dbr);	
}

/******************************************************************************
*	void Alert_Page(struct XMLLeaf * AlertLeaf, char * HID, char * CID, char * STATUS, char * Response, char * TimeStamp)
*		Performs a Pager alert. Uses the SNPP protocol to send alert (See snpp.c).
*		
*	Args :
*		struct XMLLeaf * AlertLeaf	Alert XML Linked list. 
*		char * HID			char of Host ID
*		char * CID			char of Check ID
*		char * STATUS			char of Event Status
*		char * Response			Event Details
*		char * TimeStamp		char of Time stamp of event.
*
*	Updated :
******************************************************************************/
void Alert_Page(MYSQL * mysql, struct XMLLeaf * AlertLeaf, char * HID, char * CID, char * STATUS, char * Response, char * TimeStamp)
{
	char * PAGERID = 0;
	char * HOSTNAME = 0;
	char * CHECKNAME = 0;
	char * PagerMsg = 0;
	struct XMLLeaf * LocalAlertLeaf;
	struct XMLLeaf * LocalOptionLeaf;
		
	LocalAlertLeaf = AlertLeaf;
	LocalOptionLeaf = AlertLeaf;
	
	HOSTNAME = HID2HostName(mysql, Myatoi(HID));
	CHECKNAME = CID2Name(mysql, Myatoi(CID));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Alert_Page]-[Enter]-[HostName:%s]-[CheckName:%s]", HOSTNAME, CHECKNAME);
	
	while(LocalAlertLeaf)
	{
		LocalOptionLeaf = BuildSubXMLTree(LocalAlertLeaf, ReturnXMLLeafID(LocalAlertLeaf, "PAGER.PAGERID"), 1);
		if(LocalOptionLeaf)
		{
			PAGERID = ReturnXMLLeafValue(LocalOptionLeaf, "PAGER.PAGERID");
			
			if(PAGERID && HOSTNAME && STATUS)
			{
				PagerMsg = dsprintf(PagerMsg, "%s-%s:%s", HOSTNAME, CHECKNAME, STATUS);
				SNPP_SendPage(PAGERID, PagerMsg);
				PagerMsg = dfree(PagerMsg);
			}
			LocalAlertLeaf = FindXMLLeafByID(LocalAlertLeaf, LocalOptionLeaf->ID);
			LocalOptionLeaf = DelSubXMLTree(LocalOptionLeaf);
		}
		LocalAlertLeaf = LocalAlertLeaf->NextLeaf;
	}
	
	HOSTNAME = dfree(HOSTNAME);
	CHECKNAME = dfree(CHECKNAME);
			
}

/******************************************************************************
*	void Alert_Email(struct XMLLeaf * AlertLeaf, char * HID, char * CID, char * STATUS, char * Response, char * TimeStamp)
*		Performs a Email alert. Uses the SMTP protocol to send alert (See smtp.c).
*		
*	Args :
*		struct XMLLeaf * AlertLeaf	Alert XML Linked list. 
*		char * HID			char of Host ID
*		char * CID			char of Check ID
*		char * STATUS			char of Event Status
*		char * Response			Event Details
*		char * TimeStamp		char of Time stamp of event.
*
*	Updates :
******************************************************************************/
void Alert_Email(MYSQL * mysql, struct XMLLeaf * AlertLeaf, char * HID, char * CID, char * STATUS, char * Response, char * TimeStamp)
{
	struct XMLLeaf * LocalAlertLeaf;
	struct XMLLeaf * LocalOptionLeaf;
	char * ADDRESS = 0;
	char * HOSTNAME = 0;
	char * CHECKNAME = 0;
	char * EmailString = 0;

	LocalAlertLeaf = AlertLeaf;
	LocalOptionLeaf = AlertLeaf;
	
	HOSTNAME = HID2HostName(mysql, Myatoi(HID));
	CHECKNAME = CID2Name(mysql, Myatoi(CID));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Alert_Email]-[Enter]-[HostName:%s]-[CheckName:%s]", HOSTNAME, CHECKNAME);
	
	if(HOSTNAME && CHECKNAME)
	{
		if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.EMAIL.HTML")))
		{
			EmailString = dstrcpy(EmailString, ALERT_EMAIL_HTML);
		} else {
			EmailString = dstrcpy(EmailString, ALERT_EMAIL_TEXT);
		}
			
		EmailString = dsubstrreplace(EmailString, "$[HOSTNAME]$", HOSTNAME);
		EmailString = dsubstrreplace(EmailString, "$[CHECKNAME]$", CHECKNAME);
		EmailString = dsubstrreplace(EmailString, "$[STATUS]$", STATUS);
		EmailString = dsubstrreplace(EmailString, "$[TIMESTAMP]$", TimeStamp);
		EmailString = dsubstrreplace(EmailString, "$[DETAIL]$", Response);
						
		while(LocalAlertLeaf)
		{
			LocalOptionLeaf = BuildSubXMLTree(LocalAlertLeaf, ReturnXMLLeafID(LocalAlertLeaf, "EMAIL.ADDRESS"), 1);
			if(LocalOptionLeaf)
			{
				ADDRESS = ReturnXMLLeafValue(LocalOptionLeaf, "EMAIL.ADDRESS");			
				
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[Alert_Email]-[Sending to:%s]", ADDRESS);
				
				if(ADDRESS)
				{
					if(HOSTNAME && CHECKNAME)
					{
						SMTP_SendEmail(ADDRESS, EmailString);
					}
				}
				LocalAlertLeaf = FindXMLLeafByID(LocalAlertLeaf, LocalOptionLeaf->ID);
				LocalOptionLeaf = DelSubXMLTree(LocalOptionLeaf);
			}
			LocalAlertLeaf = LocalAlertLeaf->NextLeaf;
		}
	}
		
	HOSTNAME = dfree(HOSTNAME);
	CHECKNAME = dfree(CHECKNAME);
	EmailString = dfree(EmailString);
}

/******************************************************************************
*	void Alert_External(struct XMLLeaf * AlertLeaf, char * HID, char * CID, char * STATUS, char * Response, char * TimeStamp)
*		Performs an External alert. This will use a specified command 
*		line to send an alert (Specified with the XML option Tag 
*		EXTERNAL.CMD).
*		
*	Args :
*		struct XMLLeaf * AlertLeaf	Alert XML Linked list. 
*		char * HID			char of Host ID
*		char * CID			char of Check ID
*		char * STATUS			char of Event Status
*		char * Response			Event Details
*		char * TimeStamp		char of Time stamp of event.
*
*	Updated :
******************************************************************************/
void Alert_External(MYSQL * mysql, struct XMLLeaf * AlertLeaf, char * HID, char * CID, char * STATUS, char * Response, char * TimeStamp)
{
	struct XMLLeaf * LocalAlertLeaf;
	struct XMLLeaf * LocalOptionLeaf;
	char * HOSTNAME = 0;
	char * CHECKNAME = 0;
	char * EmailString = 0;
	FILE * pp;
	char * CMD = 0;
	FILE * fp = 0;
	char * TmpOutFile = 0;
	char * TmpOutput = 0;
	
	LocalAlertLeaf = AlertLeaf;
	LocalOptionLeaf = AlertLeaf;
	
	HOSTNAME = HID2HostName(mysql, Myatoi(HID));
	CHECKNAME = CID2Name(mysql, Myatoi(CID));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Alert_External]-[Enter]-[HostName:%s]-[CheckName:%s]", HOSTNAME, CHECKNAME);
	
	if(HOSTNAME && CHECKNAME)
	{
		TmpOutFile = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.TMPEXTERN");
		
		while(LocalAlertLeaf)
		{
			LocalOptionLeaf = BuildSubXMLTree(LocalAlertLeaf, ReturnXMLLeafID(LocalAlertLeaf, "EXTERNAL.CMD"), 1);
			if(LocalOptionLeaf)
			{
				if(TmpOutFile)
				{
					fp = fopen(TmpOutFile, "w");
					if(fp)
					{
						TmpOutput = dstrcpy(TmpOutput, ALERT_EXTERNAL);
						
						TmpOutput = dsubstrreplace(TmpOutput, "$[HOSTNAME]$", HOSTNAME);
						TmpOutput = dsubstrreplace(TmpOutput, "$[CHECKNAME]$", CHECKNAME);
						TmpOutput = dsubstrreplace(TmpOutput, "$[STATUS]$", STATUS);
						TmpOutput = dsubstrreplace(TmpOutput, "$[TIMESTAMP]$", TimeStamp);
						TmpOutput = dsubstrreplace(TmpOutput, "$[DETAIL]$", Response);
				
						fprintf(fp, "%s", TmpOutput);
						fclose(fp);
						
						TmpOutput = dfree(TmpOutput);
					}
				}
				
				CMD = dstrcpy(CMD, ReturnXMLLeafValue(LocalOptionLeaf, "EXTERNAL.CMD"));
				
				CMD = dsubstrreplace(CMD, "$[HOSTNAME]$", HOSTNAME);
				CMD = dsubstrreplace(CMD, "$[CHECKNAME]$", CHECKNAME);
				CMD = dsubstrreplace(CMD, "$[STATUS]$", STATUS);
				CMD = dsubstrreplace(CMD, "$[TIMESTAMP]$", TimeStamp);
				CMD = dsubstrreplace(CMD, "$[DETAIL]$", Response);
				
				//CMD = dsubstrreplace(CMD, "&lt;", "<");
				//CMD = dsubstrreplace(CMD, "&gt;", ">");
				CMD = UnEscapeChars(CMD);
				
				pp = popen(CMD, "r");
				pclose(pp);
				
				CMD = dfree(CMD);

				if(TmpOutFile)
				{
					unlink(TmpOutFile);
				}
				
				LocalAlertLeaf = FindXMLLeafByID(LocalAlertLeaf, LocalOptionLeaf->ID);
				LocalOptionLeaf = DelSubXMLTree(LocalOptionLeaf);
			}
			LocalAlertLeaf = LocalAlertLeaf->NextLeaf;
		}
	}
		
	HOSTNAME = dfree(HOSTNAME);
	CHECKNAME = dfree(CHECKNAME);
	EmailString = dfree(EmailString);
}

/******************************************************************************
*       unsigned ReturnMaxAID(void)
*		Returns the maximum AID (Alert ID) from the MIDAS_Alert_Queue 
*		SQL table.
*
*       Updated :
******************************************************************************/	
unsigned ReturnMaxAID(MYSQL * mysql)
{
       	struct DBR * dbr = 0;
	MYSQL_ROW myrow;
	unsigned MaxEID = 0;
		
	dbr = dbQuery_P(mysql, "SELECT MAX(AID) FROM MIDAS_Alert_Queue");
	if(dbr)
	{
		myrow = dbResult(dbr);
		MaxEID = Myatoll(myrow[0]);
	}
	dbr = dbFree(dbr);

	return(MaxEID);
}

/******************************************************************************
*       void PurgeAlertQueue(unsigned MaxAID)
*		Will purge all recods from the MIDAS_Alert_Queue with AID's =< MaxAID
*
*	Args :
*		unsigned MaxAID		The Max Alert ID to Purge.
*
*       Updated :
******************************************************************************/				
void PurgeAlertQueue(MYSQL * mysql, unsigned MaxAID)
{
	struct DBR * dbr = 0;
	char * query = 0;
	
	query = dsprintf(query, "DELETE FROM MIDAS_Alert_Queue WHERE AID<='%u'", MaxAID);
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	dbr = dbFree(dbr);
}

/******************************************************************************
*       char CID2Name(int CID)
*		Takes a CID (Check ID) and returns its Check Name.
*
*	Args :
*		int CID		The Check ID to lookup.
*
*       Updated :
******************************************************************************/
char * CID2Name(MYSQL * mysql, int CID)
{
	struct DBR * dbr = 0;
	MYSQL_ROW myrow;
	char * query = 0;
	char * Out = 0;
		
	query = dsprintf(query, "SELECT Name FROM MIDAS_Checks WHERE CID='%d'", CID);
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	
	if(dbr)
	{
		myrow = dbResult(dbr);
		Out = dstrcpy(Out, myrow[0]);
	}
	dbr = dbFree(dbr);
	
	return(Out);
}

/******************************************************************************
*       char * HID2HostName(int HID)
*		Thats a HID and returns its Host name.
*
*	Args :
*		int HID		The Host ID to lookup.
*
*       Updated :
******************************************************************************/
char * HID2HostName(MYSQL * mysql, int HID)
{
	struct DBR * dbr = 0;
	MYSQL_ROW myrow;
	char * query = 0;
	char * Out = 0;
		
	query = dsprintf(query, "SELECT HostName FROM MIDAS_Hosts WHERE HID='%d'", HID);
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	
	if(dbr)
	{
		myrow = dbResult(dbr);
		Out = dstrcpy(Out, myrow[0]);
	}
	dbr = dbFree(dbr);
	
	return(Out);
}

/******************************************************************************
*       void ParseCommandLine(int argc, char *argv[])
*		Process the command line passed when starting this application
*
*	Args :
*		int arc		The Argument count.
*		int *argv[]	The Argunment value array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "VhD:TL");
		switch(opt)
		{
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D<#>\tSet Debug level\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-T\tShow Configuration\n");				
				printf("\t-V\tShow version\n");
				printf("\t-L\tSend Debug to STDERR\n");
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'L':
				LOGTOSTDERR = 1;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void CatchSignal(int sig)
*               The signal() catch function. Handles, TERM, KILL, and HUP
*
*       Args :
*               int sig         Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void CatchSignal(int sig)
{
        switch(sig)
        {
                case SIGHUP:
                        MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchSignal]-[Caught HUP Reloading config file : %s]", CFGPATH);

                        ConfigXMLTree = FileToXMLTree(CFGPATH);
                        break;
                case SIGTERM:
                case SIGKILL:
                        MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchHUP]-[Caught TERM/KILL exiting]");
                        exit(0);
                        break;
        }
}

/******************************************************************************
*       void ShowConfig(void)
*		Print out the configuration options read from the configuration 
*		file. This is done when using the -T command line sqitch.
*
*       Updated :
******************************************************************************/
void ShowConfig(void)
{
	printf("%s %s\n", APPNAME, MIDASVER);
        printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME);
        DebugXMLTree("MIDASa.cf", ConfigXMLTree, 0);
}
