/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASb
*       Filename : MIDASb.c
*		Main application for MIDASb. MIDASb handles the Big Brother
*		client to MIDASd connection/convertion. 
*
*       Updated :
*               11/06/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <signal.h>
#include <netdb.h>

#include "../Common/Commoninclude.h"

#include "includes.h"

int LOGTOSTDERR = 0;
int DEBUGLEVEL 	= 0;
int LOG_INFO 	= 1;
int LOG_SOCKET 	= 2;
int LOG_XML	= 4;
int LOG_CHECK	= 8;
int LOG_FORK	= 16;

struct XMLLeaf * ConfigXMLTree = 0;

char APPNAME[] = "MIDASb";
char * CFGPATH = 0;
char * KEYFILE = 0;
extern char MIDAS_EOL[];
char STATUS[][10] = { 	"RED",
			"GREEN",
			"YELLOW",
		    };

#ifdef __WIN32__
#define close(x) closesock(x)
#endif

/******************************************************************************
*       void main(int argc, char * argv[])
*		main() function for MIDASb
*
*	Args :
*		int argc		Argument count
*		char * argv[]		Argument array
*
*       Updated :
******************************************************************************/
int main(int argc, char *argv[])
{
	(void) signal(SIGHUP, CatchSignal);
	(void) signal(SIGCHLD, CatchSignal);
	(void) signal(SIGKILL, CatchSignal);
	(void) signal(SIGTERM, CatchSignal);
	
	CFGPATH = dsprintf(CFGPATH, "%s/etc/MIDASb.cf", APPROOT);
	ConfigXMLTree = FileToXMLTree(CFGPATH);
	
	if(!ConfigXMLTree)
	{
		printf("! Unable to open %s\n", CFGPATH);
		exit(0);
	}
	
	
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));

	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASd %s starting]", MIDASVER);
	
	StartServer();
	
	DelXMLLeafTree(ConfigXMLTree);
	
	exit(0);
}

/******************************************************************************
*       int StartServer(void)
*		Starts the BB server and waits for BB clients to connect. On 
*		connect processes client alerts. After process sends alert 
*		detail to MIDASd.
*
*       Updated :
******************************************************************************/	
int StartServer(void)
{
	int sockfd, clientsockfd, len;
	struct sockaddr_in serv_addr, cli_addr;
	char *tCIP = 0;
	int FreeChildrenCount;
	unsigned CHILDREN[255];
	unsigned BIRTH[255];
	int on = 1;
	pid_t child;
	char * SendBuff = 0;
	char * ClientString = 0;
	char * TYPE = 0;
	char * HOST_CHECK = 0;
	char * HOST = 0;
	char * CHECK = 0;
	char * STATUS = 0;
	char * DETAIL = 0;
	struct hostent * hostinfo = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Starting]");
	
	bzero(&CHILDREN, sizeof(CHILDREN));
	bzero(&BIRTH, sizeof(BIRTH));
	
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof on);
	
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	
	if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"))
	{
		hostinfo = gethostbyname(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"));
		serv_addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
	} else {
		serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	}

	serv_addr.sin_port = htons(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PORT")));
		
	if(bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr)) == -1)
	{
		MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SockServer]-[Unable to open Port for listening. Exiting.]");
		close(sockfd);
		return(-1);
	}

	if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"))
	{
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Interface %s:%s bound]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"), ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PORT"));
	} else {
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Port %s bound]",  ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PORT"));
	}
	
	listen(sockfd, 250);
	len = sizeof(cli_addr);
	for(;;)
	{
		clientsockfd = accept(sockfd,(struct sockaddr*)&cli_addr,&len);
		if(clientsockfd > 0)
		{
			tCIP = ConvertL2IP(cli_addr.sin_addr.s_addr);
			if(CheckIPAccess(cli_addr.sin_addr.s_addr))
			{
				MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[Client connected]-[%s]", tCIP);
				do
				{
					FreeChildrenCount = FreeChild(CHILDREN, BIRTH);
					if(FreeChildrenCount != -1)
					{
						child = fork();
						if(child == 0)
						{
							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Forked]");
							
							SendBuff = dsprintf(SendBuff, "%s v%s connection accepted\n", APPNAME,  MIDASVER);
							
							SendToSocket(clientsockfd, SendBuff, MIDAS_EOL, 0);
							SendBuff = dfree(SendBuff);
							
							ClientString = RecvFromSocket(clientsockfd, MIDAS_EOL, 0);
							if(ClientString)
							{
								MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Sent %s]", ClientString);
								
								TYPE = dstrcpy(TYPE, strtok(ClientString, " "));
								if(!strcmp(TYPE, "status"))
								{
									HOST_CHECK = dstrcpy(HOST_CHECK, strtok(NULL, " "));
									STATUS = dstrcpy(STATUS, strtok(NULL, " "));
									DETAIL = dstrcpy(DETAIL, strtok(NULL, "\0"));
									
									DETAIL = dsubstrreplace(DETAIL, "&green", "[GREEN]");
									DETAIL = dsubstrreplace(DETAIL, "&yellow", "[YELLOW]");
									DETAIL = dsubstrreplace(DETAIL, "&red", "[RED]");
									
									HOST = dstrcpy(HOST, strtok(HOST_CHECK, "."));
									HOST = dsubstrreplace(HOST, ",", ".");
									
									CHECK = dstrcpy(CHECK, strtok(NULL, "\0"));
									
									UpdateStatus(HOST, CHECK, STATUS, DETAIL);
									
									HOST_CHECK = dfree(HOST_CHECK);
									STATUS = dfree(STATUS);
									HOST = dfree(HOST);
									CHECK = dfree(CHECK);
									DETAIL = dfree(DETAIL);
								}
								TYPE = dfree(TYPE);								
							}

							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Closed]");
							close(clientsockfd);
							exit(0);
						} else {
							close(clientsockfd);
							AddChild(child, CHILDREN, BIRTH);
						}
					} else {
						usleep(100);
						continue;
					}
				}while(FreeChildrenCount == -1);
			} else {
				MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[IP Access denied to : %s]", tCIP);
				close(clientsockfd);
			}
			tCIP = dfree(tCIP);
		}
	}
	return(0);
}

/******************************************************************************
*       void UpdateStatus(char * BBHost, char * BBCheck, char * BBStatus, char * BBDetail)
*		Create Client checkupdate request and send to MIDASd server.
*
*	Args :
*		char * BBHost		Host name specified in the BB alert 
*		char * BBCheck		Check name specified in the BB alert
*		char * BBStatus		Status specified in the BB alert
*		char * BBDetail		The event detail specified in the BB 
*					alert.
*
*       Updated :
******************************************************************************/
void UpdateStatus(char * BBHost, char * BBCheck, char * BBStatus, char * BBDetail)
{
	int HID = 0;
	int CID = 0;
	char * Response = 0;
	char * RecvString = 0;
	char * Detail = 0;
	
	BBStatus = dstrtoupper(BBStatus);
	HID = BBHostName2HID(BBHost);
	CID = BBCheckName2CID(BBCheck);
	Detail = CleanXMLValueString(BBDetail);
	
	Response = dsprintf(Response, "<CLIENT><HID>%s</HID><TYPE>CHECKUPDATE</TYPE><CHECKUPDATE><CHECK><HID>%d</HID><CID>%d</CID><STATUS>%s</STATUS><DETAIL>%s</DETAIL></CHECK></CHECKUPDATE></CLIENT>",
				      ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"), HID, CID, BBStatus, Detail);
	
	RecvString = SendToServer(Response);
	RecvString =  dfree(RecvString);
	
	Detail = dfree(Detail);
	Response = dfree(Response);
}

/******************************************************************************
*       int BBCheckName2CID(char * Name)
*		Converts a BB check name into a MIDAS check ID. These are 
*		defined in the MIDASb.cf file. Returns CID value or -1 on error.
*
*	Args :
*		char * Name		The Check Name specified in the BB alert.
*
*       Updated :
******************************************************************************/
int BBCheckName2CID(char * Name)
{
	struct XMLLeaf * LocalLeaf = 0;
	char * BBNAME = 0;
	int CID = 0;
	
	if(Name)
	{
		LocalLeaf = ConfigXMLTree;
		while(LocalLeaf)
		{
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.BBCHECK.NAME");
			if(LocalLeaf)
			{
				BBNAME = LocalLeaf->Value;
				CID = Myatoi(ReturnXMLLeafValue(LocalLeaf, "CONFIG.BBCHECK.NAME.CID"));
				
				if(BBNAME && CID)
				{
					if(!strcasecmp(Name, BBNAME))
					{
						return(CID);
					}
				}
      				LocalLeaf = LocalLeaf->NextLeaf;
      			}
		}
	}
	return(-1);
}

/******************************************************************************
*       int BBHostName2HID(char * Name)
*		Queries the MIDASd server to convert a specific BB host name to
*		its HID value. Returns HID value, or -1 on error.
*
*	Args :
*		char * Name		The Host name specified in the BB alert.
*
*       Updated :
******************************************************************************/
int BBHostName2HID(char * Name)
{
	char * Query = 0;
	char * Response = 0;
	struct XMLLeaf * ResponseLeaf = 0;
	int HID = 0;
	
	if(Name)
	{
		Query = dsprintf(Query, "<CLIENT><HID>%s</HID><TYPE>HIDQUERY</TYPE><HOSTNAME>%s</HOSTNAME></CLIENT>", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),  Name);
		Response = SendToServer(Query);
		Query = dfree(Query);
		
		if(Response)
		{
			ResponseLeaf = StringToXMLTree(Response);
			if(ResponseLeaf)
			{
				HID = Myatoi(ReturnXMLLeafValue(ResponseLeaf, "SERVER.HIDQUERY.HID"));
			}
			ResponseLeaf = DelXMLLeafTree(ResponseLeaf);
			
		}
		Response = dfree(Response);
	}
	return(HID);
}

/******************************************************************************
*       int CheckIPAccess(unsigned clientip)
*		Verifies that clientip has access to connect. Return 1 for true,
*		0 for false.
*
*	Args :
*		unsigned clientip		The Client ip in numeric form.
*
*       Updated :
******************************************************************************/						        
int CheckIPAccess(unsigned clientip)
{
	int  status = 0;
	struct XMLLeaf * LocalXMLLeaf;
	unsigned tIP;
	unsigned tMASK;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[CheckIPAccess]-[Enter]");

	LocalXMLLeaf = ConfigXMLTree;
	
	while(LocalXMLLeaf)
	{
		LocalXMLLeaf = FindXMLLeafByTag(LocalXMLLeaf, "CONFIG.AccessIP.IP");
		if(LocalXMLLeaf)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[CheckIPAccess]-[IP:%s]", LocalXMLLeaf->Value);

			tIP = ConvertIP2L(LocalXMLLeaf->Value);

			LocalXMLLeaf = FindXMLLeafByTag(LocalXMLLeaf, "CONFIG.AccessIP.IP.MASK");
			if(LocalXMLLeaf)
			{
				tMASK = (MyPow(2, Myatoi(LocalXMLLeaf->Value)) -1);
				if((clientip & tMASK) == tIP)
				{
					status = 1;
					break;
				}
			}
		}
	}
	return(status);	
}

/******************************************************************************
*       void CatchSignal(int sig)
*               The signal() catch function. Handles, TERM, KILL, CHLD and HUP
*
*       Args :
*               int sig         Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void CatchSignal(int sig)
{
        pid_t pid = 1;

        switch(sig)
        {
		case SIGHUP:
			MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchSignal]-[Caught HUP Reloading config file : %s]", CFGPATH);

			ConfigXMLTree = FileToXMLTree(CFGPATH);
			break;
		case SIGTERM:
		case SIGCHLD:
			while(pid > 0)
			{
				pid = waitpid(0, NULL, WNOHANG);
			}
                        break;
		case SIGKILL:
			MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchHUP]-[Caught TERM/KILL exiting]");
			exit(0);
			break;
	}
}

/******************************************************************************
*	int ParseCommandLine(int argc, char *argv[])
*		Parses command line arguments.
*
*	Args :
*		int argc		Argument count.
*		char *argv[]		Argument value array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "VhD:TL");
		switch(opt)
		{
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D#\tSet the Debug level\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-T\tShow configuration\n");
				printf("\t-V\tShow version\n");
				printf("\t-L\tSend Debug to STDERR\n");
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'L':
				LOGTOSTDERR = 1;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Show the current configuration settings. Trigged with the -T 
*		command line argument.
*
*       Updated :
******************************************************************************/
void ShowConfig(void)
{
        printf("%s %s\n", APPNAME, MIDASVER);
        printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME);
        DebugXMLTree("MIDASb.cf", ConfigXMLTree, 0);
}
