/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASc
*       Filename : MIDASc.c
*               MIDASa.c contains the primary source code for the MIDASc app
*               (The Client Check daemon). It handles all check functions 
*		local and network.
*
*       Updated :
*               11/06/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>

#include <windows.h>
#include <process.h>
#include <winsvc.h>
#include "service.h"

#include "MIDASc.h"
#include "../includes.h"
#include "../../Common/Commoninclude.h"

int LOGTOSTDERR = 0;
int DEBUGLEVEL = 0;

int RUN_AS_SERVICE = 1;

int	LOG_INFO	=1;
int	LOG_ERROR	=2;
int 	LOG_CHECK	=4;
int	LOG_SOCKET	=16;
int	LOG_MD5 	=32;
int	LOG_DISK	=64;
int	LOG_LOAD	=128;
int	LOG_LOG		=256;
int	LOG_PROC	=512;
int	LOG_EXT		=1024;
int	LOG_NETWORK	=2048;
int	LOG_XML		=4096;
int	LOG_FORK	=8192;

struct XMLLeaf * ConfigXMLTree;
char APPNAME[] = "MIDAScNT";

char CFGPATH[] = "MIDASc.cf";
char * KEYFILE = 0;
char STATUS[][20] = { 	"RED",
			"GREEN",
			"YELLOW"
		    };

/******************************************************************************
*       int main(int argc, char *argv[])
*		main() function for MIDASc.
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument vlaue array.
*
*       Updated :
******************************************************************************/
int main(int argc, char *argv[])
{
	char * p = 0;
	char * APPLICATION_PATH = 0;
        SERVICE_TABLE_ENTRY dispatchTable[] = {{TEXT(SZSERVICENAME), (LPSERVICE_MAIN_FUNCTION)service_main}, { NULL, NULL }};
	WSADATA wsaData;
	WORD wVersionRequested;

	wVersionRequested = MAKEWORD(2, 0);
	WSAStartup(wVersionRequested, &wsaData);

	APPLICATION_PATH = drealloc(APPLICATION_PATH, 512);
	        
        GetModuleFileName(NULL, APPLICATION_PATH, 511);
        p = strrchr(APPLICATION_PATH, '\\');
        *p = 0;

	SetCurrentDirectory(APPLICATION_PATH);

	ConfigXMLTree = FileToXMLTree(CFGPATH);
	if(!ConfigXMLTree)
	{
		printf("! Unable to open %s\n", CFGPATH);
		exit(0);
	}
	
	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	if(RUN_AS_SERVICE)
	{
		StartServiceCtrlDispatcher(dispatchTable);
		APPLICATION_PATH = dfree(APPLICATION_PATH);
	} else {
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[MIDAScNT]-[Running From Console]");
		
		Main_Loop();
	}

	WSACleanup();

	exit(0);
}

/******************************************************************************
*	void Main_Loop(void)
*		NT Service entry function.
*
*	Updated :
*******************************************************************************/
void Main_Loop()
{
	struct XMLLeaf * CheckXMLTree = 0;
	char * SendBuff = 0;
	char * RecvBuff = 0;
	time_t tm;

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[%s v%s started]", APPNAME, MIDASVER);
	
	for(;;)
	{
		tm = time(NULL);
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[%s]-[Starting Check loop]", APPNAME);

		CheckXMLTree = QueryChecks();
		
		if(CheckXMLTree)
		{
			SendBuff = ProcessChecks(CheckXMLTree);
			if(SendBuff)
			{
				RecvBuff = SendToServer(SendBuff);
				RecvBuff = dfree(RecvBuff);
			}
			SendBuff = dfree(SendBuff);
		}
		CheckXMLTree = DelXMLLeafTree(CheckXMLTree);
			
		MyDebug(__LINE__, __FILE__,  LOG_INFO, "[%s]-[Ending Check loop]-[Check Time:%u seconds]", APPNAME, (time(NULL) - tm));
		sleep(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.CHECKINTERVAL")));
	}
}

/******************************************************************************
*       int ParseCommandLine(int argc, char *argv[])
*		Parses command line arguments.
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument vlaue array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "VThD:irCL");
		switch(opt)
		{
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D#\tSet Debug level\n");
				printf("\t-L\tLog to console instead of MIDAScNT.log\n");
				printf("\t-C\tRun as console application instead of as service\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-T\tShow Config and server query response.\n");
				printf("\t-V\tShow version\n");
				printf("\t-i\tInstall Windows Service\n");
				printf("\t-r\tRemove Windows Service\n");
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'i':
                 		CmdInstallService();
                 		Status = 0;
                                break;
			case 'r':
				CmdRemoveService();
				Status = 0;
				break;
			case 'C':
				RUN_AS_SERVICE = 0;
				break;
			case 'L':
				LOGTOSTDERR = 1;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Displays configurations settings. Also displays the result (if
*		any) of a CHECKREQUEST query to the MIDASd server
*
*       Updated :
******************************************************************************/
void ShowConfig(void)
{
	struct XMLLeaf * LocalLeaf = 0;

	printf("%s %s\n", APPNAME, MIDASVER);
	printf("------------------------------------------------------------------\n");
	printf("[Contents of %s.cf file]\n", APPNAME);
	
	DebugXMLTree("MIDASc.cf", ConfigXMLTree, 0);

	printf("------------------------------------------------------------------\n");
	printf("[Query Check Response from Server]\n");
	LocalLeaf = QueryChecks();
	if(LocalLeaf)
	{
		DebugXMLTree("MIDASc.checks", LocalLeaf, 0);
		LocalLeaf = DelXMLLeafTree(LocalLeaf);
	} else {
		printf(" !! No response from server.\n");
	}
}
