#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>

#include <windows.h>
#include <process.h>
#include <winsvc.h>

#include "service.h"
#include "../../Common/Commoninclude.h"
#include "MIDASc.h"


SERVICE_STATUS          ssStatus;       // current status of the service
SERVICE_STATUS_HANDLE   sshStatusHandle;
DWORD                   dwErr = 0;

void WINAPI service_main(DWORD dwArgc, LPTSTR *lpszArgv)
{
	SetConsoleCtrlHandler(ControlHandler, TRUE);

	sshStatusHandle = RegisterServiceCtrlHandler(TEXT(SZSERVICENAME), service_ctrl);
	if(sshStatusHandle)
	{
  		ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
		ssStatus.dwServiceSpecificExitCode = 0;
		ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 30000);

  		(VOID)ReportStatusToSCMgr(SERVICE_RUNNING, NO_ERROR, 30000);

		Main_Loop();

		(VOID)ReportStatusToSCMgr(SERVICE_STOPPED, dwErr, 0);
	}
	return;
}


VOID WINAPI service_ctrl(DWORD dwCtrlCode)
{
	switch(dwCtrlCode)
	{
		case SERVICE_CONTROL_STOP:
			ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 0);
			(VOID)ReportStatusToSCMgr(SERVICE_STOPPED, dwErr, 0);		
			return;
		case SERVICE_CONTROL_INTERROGATE:
			break;
		default:
			break;
    
  	}
	ReportStatusToSCMgr(ssStatus.dwCurrentState, NO_ERROR, 0);
  
}


int ReportStatusToSCMgr(DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwWaitHint)
{
	static DWORD dwCheckPoint = 1;
	BOOL fResult = TRUE;
  
	if (dwCurrentState == SERVICE_START_PENDING)
	{
		ssStatus.dwControlsAccepted = 0;
	} else {
		ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;
	}
    
	ssStatus.dwCurrentState = dwCurrentState;
	ssStatus.dwWin32ExitCode = dwWin32ExitCode;
	ssStatus.dwWaitHint = dwWaitHint;
    
	if((dwCurrentState == SERVICE_RUNNING) || (dwCurrentState == SERVICE_STOPPED))
	{
		ssStatus.dwCheckPoint = 0;
	} else {
		ssStatus.dwCheckPoint = dwCheckPoint++;
	}
	fResult = SetServiceStatus(sshStatusHandle, &ssStatus);
  	return fResult;
}

void CmdInstallService()
{
	SC_HANDLE   schService;
	SC_HANDLE   schSCManager;
	TCHAR szPath[512];

	schSCManager = OpenSCManager(NULL,NULL,	SC_MANAGER_ALL_ACCESS);
	if(schSCManager)
	{
		GetModuleFileName(NULL, szPath, 512);
	
		schService = CreateService(	schSCManager,               // SCManager database
						TEXT(SZSERVICENAME),        // name of service
						TEXT(SZSERVICEDISPLAYNAME), // name to display
						SERVICE_ALL_ACCESS,         // desired access
						SERVICE_WIN32_OWN_PROCESS,  // service type
						SERVICE_AUTO_START, 	    // start type
						SERVICE_ERROR_NORMAL,       // error control type
						szPath,                     // service's binary
						NULL,                       // no load ordering group
						NULL,                       // no tag identifier
						NULL,      		    // dependencies
						NULL,                       // LocalSystem account
						NULL);                      // no password
    
		if(schService)
		{
			printf("%s installed.\n", SZSERVICEDISPLAYNAME);
			CloseServiceHandle(schService);
		} else {
			printf("%s Not Installed\n", SZSERVICEDISPLAYNAME);
		} 
		CloseServiceHandle(schSCManager);
	}
}


void CmdRemoveService()
{
	SC_HANDLE   schService;
	SC_HANDLE   schSCManager;
  
	schSCManager = OpenSCManager	(NULL,	NULL, SC_MANAGER_ALL_ACCESS);
	if(schSCManager)
	{
		schService = OpenService(schSCManager, TEXT(SZSERVICENAME), SERVICE_ALL_ACCESS);
    		if(schService)
		{
			if(ControlService(schService, SERVICE_CONTROL_STOP, &ssStatus))
			{
				printf("Stopping %s\n", SZSERVICEDISPLAYNAME);
				Sleep(1000);
				while(QueryServiceStatus( schService, &ssStatus ))
				{
					if(ssStatus.dwCurrentState == SERVICE_STOP_PENDING)
					{
						printf(".");
						Sleep( 1000 );
					} else {
						break;
					}
				}
	    			if(ssStatus.dwCurrentState == SERVICE_STOPPED)
				{
					printf("\n%s stopped.\n", SZSERVICEDISPLAYNAME);
				} else {
					printf("\n%s failed to stop.\n", SZSERVICEDISPLAYNAME);
				}	
			}
    
			if(DeleteService(schService))
			{
				printf("%s removed.\n", SZSERVICEDISPLAYNAME);
      			}
      			CloseServiceHandle(schService);
		} 
		CloseServiceHandle(schSCManager);
	}
}


BOOL WINAPI ControlHandler (DWORD dwCtrlType)
{
	switch(dwCtrlType)
	{
		case CTRL_BREAK_EVENT:  // use Ctrl+C or Ctrl+Break to simulate
		case CTRL_C_EVENT:      // SERVICE_CONTROL_STOP in debug mode
			printf("Stopping %s.\n", SZSERVICEDISPLAYNAME);
    			return TRUE;
			break;
    
  	}
	return FALSE;
}

FILE * Service_popen(char * Command)
{
	FILE * fp = 0;
	char * lCommand = 0;
	int Status = 0;
	
	if(Command)
	{
		lCommand = dsprintf(lCommand, "%s > MIDAScNT.tmp", Command);
		Status = system(lCommand);
		lCommand = dfree(lCommand);
		
		fp = fopen("MIDAScNT.tmp", "r");
	}
	
	return(fp);
}

int Service_pclose(FILE * fp)
{
	fclose(fp);
	unlink("MIDAScNT.tmp");
	
	return(0);
}

