/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASc
*       Filename : check_external.c
*		Functions for handling external checks (Midas and Netsaint).
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "includes.h"
#include "extern.h"
#include "../Common/Commoninclude.h"

/******************************************************************************
*       char * Check_External_MIDAS(struct XMLLeaf * ClientLeaf)
*		Performs the MIDAS style (Regex based) external check.
*
*	Args :
*		struct XMLLeaf * ClientLeaf	XML Linked list containing 
*						check settings.
*
*       Updated :
******************************************************************************/
char * Check_External_MIDAS(struct XMLLeaf * ClientLeaf)
{	 
	char * Response = 0;
	char * tResponse = 0;
	char * Detail = 0;
	char * tDetail = 0;
	char * Stats = 0;
	char * tStats = 0;
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * CheckLeaf;
	char * COMMAND = 0;
	char * REGEX = 0;
	char * LOGIC = 0;
	char * EXPECT = 0;
	unsigned Count = 0;
	FILE *pp;
	int Status = 1;
	char * CmdString = 0;
	float Value = 0;
	char * StrValue = 0;
	
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
					ReturnXMLLeafValue(ClientLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
					ReturnXMLLeafValue(ClientLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));
					
	LocalLeaf = ClientLeaf;
	CheckLeaf = ClientLeaf;
	
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL"), 1);
		if(CheckLeaf)
		{
			COMMAND = dstrcpy(COMMAND, ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL.COMMAND"));
			REGEX = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL.REGEX");
			LOGIC = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL.LOGIC");
			EXPECT = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL.EXPECT");
			
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_External_MIDAS]-[COMMAND:%s]-[REGEX:%s]-[LOGIC:%s]-[EXPECT:%s]",
								COMMAND, REGEX, LOGIC, EXPECT);
								
			if(COMMAND && LOGIC)
			{
				COMMAND = dsubstrreplace(COMMAND, "$[HOSTNAME]$", ReturnXMLLeafValue(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.HOSTNAME"));
				COMMAND = dsubstrreplace(COMMAND, "$[IP]$", ReturnXMLLeafValue(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.IP"));
				COMMAND = UnEscapeChars(COMMAND);
				
				MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_External_MIDAS]-[COMMAND:%s]", COMMAND);
				
				pp = popen(COMMAND,"r");
				if(pp)
				{
					CmdString = dfiletostring(pp);
					pclose(pp);
					
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_External_MIDAS]-[CmdString:%s]", CmdString);
					 
					if(CmdString)
					{
						if(!CheckStringLogic(CmdString, REGEX, LOGIC, EXPECT))
						{
							Status = 0;
							tDetail = dsprintf(tDetail, "[RED] Output : %s\n", CmdString);
						} else {
							tDetail = dsprintf(tDetail, "[GREEN] Output : %s\n", CmdString);
						}
						
						if(REGEX)
						{
							StrValue = RegexString(StrValue, CmdString, REGEX);
							Value = Myatof(StrValue);
							StrValue = dfree(StrValue);
						} else {
							Value = Myatof(CmdString);
						}
							
						if(Value)
						{
							tStats = dsprintf(tStats, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>%d</ORDER><LABEL>External</LABEL><DATAVAL>%f</DATAVAL></STAT>",
										  ReturnXMLLeafValue(ClientLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
										  ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
										  Count, Value);
							Stats = dstrcat(Stats, tStats);
							tStats = dfree(tStats);
						}
						
						Detail = dstrcat(Detail, tDetail);
						tDetail = dfree(tDetail);
					}
					CmdString = dfree(CmdString);
				} else {
					Status = 0;
					tDetail = dsprintf(tDetail, "[RED] Unable to execute %s\n", COMMAND);
					Detail = dstrcat(Detail, tDetail);
					tDetail = dfree(tDetail);
				}
			}
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);
			
			COMMAND = dfree(COMMAND);
			
		}
		
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}
		
		Count++;
	}
	
	if(!Detail)
	{
		Detail = dstrcpy(Detail, "");
	} 
	
	if(!Stats)
	{
		Stats = dstrcpy(Stats, "");
	}
	
	Detail = CleanXMLValueString(Detail);
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, Stats);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	Detail = dfree(Detail);
	Stats = dfree(Stats);
	return(Response);
}	

/******************************************************************************
*       char * Check_External_NetSaint(struct XMLLeaf * ClientLeaf)
*		Performs Netsaint/Nagios external style checks (Exit Status 
*		based.
*
*	Args :
*		struct XMLLeaf * ClientLeaf	XML Linked list containing 
*						check settings.
*
*
*       Updated :
******************************************************************************/
char * Check_External_NetSaint(struct XMLLeaf * ClientLeaf)
{
	char * Response = 0;
	char * tResponse = 0;
	char * Detail = 0;
	char * tDetail = 0;
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * CheckLeaf;
	char * COMMAND = 0;
	char * REGEX = 0;
	char * Stats = 0;
	char * tStats = 0;
	char * StrValue = 0;
	float Value = 0;
	int Count = 0;
			
	FILE *pp;
	int Status = 1;
	int pstatus = 0;
	char * CmdString = 0;
	
	
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
					ReturnXMLLeafValue(ClientLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
					ReturnXMLLeafValue(ClientLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));
					
	LocalLeaf = ClientLeaf;
	CheckLeaf = ClientLeaf;
	
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL"), 1);
		if(CheckLeaf)
		{
			COMMAND = dstrcpy(COMMAND, ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL.COMMAND"));
			REGEX = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.EXTERNAL.REGEX");
			
			MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_External_NetSaint]-[COMMAND:%s]-[REGEX:%s]", COMMAND, REGEX);
			
			if(COMMAND)
			{
				COMMAND = dsubstrreplace(COMMAND, "$[HOSTNAME]$", ReturnXMLLeafValue(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.HOSTNAME"));
				COMMAND = dsubstrreplace(COMMAND, "$[IP]$", ReturnXMLLeafValue(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.IP"));
				
				COMMAND = UnEscapeChars(COMMAND);
				
				pp = popen(COMMAND,"r");
				if(pp)
				{
					CmdString = dfiletostring(pp);
					pstatus = (int)WEXITSTATUS(pclose(pp));
					
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_External_Netsaint]-[%s]", CmdString);
					
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_External_Netsaint]-[Exit Status:%d]", pstatus);
					
					switch(pstatus)
					{
						case 0:
							Status = 1;
							if(CmdString)
							{
								tDetail = dsprintf(tDetail, "[GREEN] %s\n", CmdString);
							} else {
								tDetail = dsprintf(tDetail, "[GREEN] %s\n", COMMAND);
							}
							break;	                                                                                                                                                                                                                                                                
						case 1:
							Status = 2;
							if(CmdString)
							{
								tDetail = dsprintf(tDetail, "[YELLOW] %s\n", CmdString);
							} else {
								tDetail = dsprintf(tDetail, "[YELLOW] %s\n", COMMAND);
							}
						case 2:
						case -1:
						default:
							Status = 0;
							if(CmdString)
							{
								tDetail = dsprintf(tDetail, "[RED] %s\n", CmdString);
							} else {
								tDetail = dsprintf(tDetail, "[RED] %s\n", COMMAND);
							}
					}
					
					if(CmdString)
					{
						Value = 0;
						if(REGEX)
						{
							StrValue = RegexString(StrValue, CmdString, REGEX);
							Value = Myatof(StrValue);
							StrValue = dfree(StrValue);
						} 
						
						if(Value)
						{
							tStats = dsprintf(tStats, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>%d</ORDER><LABEL>External</LABEL><DATAVAL>%f</DATAVAL></STAT>",
										  ReturnXMLLeafValue(ClientLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
										  ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
										  Count, Value);
							Stats = dstrcat(Stats, tStats);
							tStats = dfree(tStats);
						}
						
						Detail = dstrcat(Detail, tDetail);
						tDetail = dfree(tDetail);
					}
					
					Detail = dstrcat(Detail, tDetail);
					tDetail = dfree(tDetail);
					
					CmdString = dfree(CmdString);
				} else {
					Status = 0;
					tDetail = dsprintf(tDetail, "[RED] Unable to execute %s\n", COMMAND);
					Detail = dstrcat(Detail, tDetail);
					tDetail = dfree(tDetail);
				}
			}
			Count++;
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);
			
			COMMAND = dfree(COMMAND);
		}
		
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}

	if(!Detail)
	{
		Detail = dstrcpy(Detail, "");
	}

	if(!Stats)
	{
		Stats = dstrcpy(Stats, "");
	}
	
	Detail = CleanXMLValueString(Detail);
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, Stats);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	Detail = dfree(Detail);
	Stats = dfree(Stats);
	
	return(Response);
}
