#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "../../../Common/Commoninclude.h"
#include "httpbench.h"

// [TCC] We define some globals used by a few of the common functions.
int LOG_SOCKET = 1;
int LOG_XML = 2;
int LOG_INFO = 4;

int DEBUGLEVEL = 0;
int LOGTOSTDERR = 0;

char * KEYFILE = 0;

int main()
{
	char * InXMLString = 0;
	struct XMLLeaf * ConfigXML = 0;
	char * OutXMLString = 0;
	int ExitStatus = 0;
	
	// [TCC] We Read in the Piped XML Config from STDIN
	InXMLString = ReadPipe(fileno(stdin));

	if(InXMLString)
	{
		// [TCC] We Build the Config XML Tree from the InXMLString.
		ConfigXML = StringToXMLTree(InXMLString);
		
		if(ConfigXML)
		{
			// [TCC] Now we Process the Check and return the Output XML String
			OutXMLString = HTTPBENCH_DoCheck(ConfigXML);
		} else {
			ExitStatus = -1;
		}
	} else {
		ExitStatus = -1;
	}
	
	if(OutXMLString)
	{
		fprintf(stdout, "%s", OutXMLString);
	}
	
	exit(ExitStatus);
}

char * ReturnCheckOptionValue(struct XMLLeaf * ConfigXML, char * TagSuffix)
{
	char * Tag = 0;
	char * Value = 0;
	
	Tag = dsprintf(Tag, "CHECK.OPTION.%s", TagSuffix);
	
	Value = ReturnXMLLeafValue(ConfigXML, Tag);
	Tag = dfree(Tag);

	return(Value);
}

char * HTTPBENCH_DoCheck(struct XMLLeaf * ConfigXML)
{
	char * IP = 0;
	int PORT = 0;
	char * HOSTNAME = 0;
	char * URL = 0;
	char * METHOD = 0;
	char * POSTDATA = 0;
	char * Response = 0;
	char * Stats = 0;
	char * Detail = 0;
	struct HTTPInfo * HTTPInfo = 0;
	
	if(ConfigXML)
	{
		IP = ReturnXMLLeafValue(ConfigXML, "CHECK.IP");
		PORT = Myatoll(ReturnXMLLeafValue(ConfigXML, "CHECK.PORT"));
		
		if(ReturnCheckOptionValue(ConfigXML, "HTTPBENCH.HOSTNAME"))
		{
			HOSTNAME = ReturnCheckOptionValue(ConfigXML, "HTTPBENCH.HOSTNAME");
		} else {
			HOSTNAME = IP;
		}
		
		URL = ReturnCheckOptionValue(ConfigXML, "HTTPBENCH.URL");
		POSTDATA = ReturnCheckOptionValue(ConfigXML, "HTTPBENCH.POSTDATA");
		
		if(ReturnCheckOptionValue(ConfigXML, "HTTPBENCH.METHOD"))
		{
			METHOD = dstrcpy(METHOD, ReturnCheckOptionValue(ConfigXML, "HTTPBENCH.METHOD"));
		} else {
			METHOD = dstrcpy(METHOD, "GET");
		}
		
		if(IP && URL && PORT)
		{
			HTTPInfo = HTTPBENCH_MakeRequest(IP, PORT, HOSTNAME, URL, METHOD, POSTDATA);
			
			if(HTTPInfo)
			{
				Detail = dsprintf(Detail, "<DETAIL>Server Software:%s\nResponse Time: %.2f\nRequest Response:%s</DETAIL>", 
							  HTTPInfo->ServerSoftware, ((float) HTTPInfo->ResponseTime / (float) 1000), HTTPInfo->Response);
							  
				
				Stats = dsprintf(Stats, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>0</ORDER><LABEL>Response Time</LABEL><DATAVAL>%.2f</DATAVAL></STAT>",
							ReturnXMLLeafValue(ConfigXML, "CHECK.CID"),
							ReturnXMLLeafValue(ConfigXML, "CHECK.TID"),
							((float) HTTPInfo->ResponseTime / (float) 1000));
				
				Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID><STATUS>%s</STATUS>%s%s</CHECK>",
						    ReturnXMLLeafValue(ConfigXML, "CHECK.HID"),
						    ReturnXMLLeafValue(ConfigXML, "CHECK.CID"),
						    HTTPInfo->StatusColor,
						    Detail, Stats);
				Stats = dfree(Stats);
				Detail = dfree(Detail);
			}
			
			HTTPInfo = HTTPBENCH_FreeInfo(HTTPInfo);
		}
		
		METHOD = dfree(METHOD);
	}
	
	return(Response);
}

struct HTTPInfo * HTTPBENCH_MakeRequest(char * IP, unsigned Port, char * HostName, char * URL, char * METHOD, char * POSTDATA)
{
	int sock = 0;
	int connected = 0;
	struct sockaddr_in addr;
	struct hostent * hostinfo = 0;
	struct timeval RequestTime;
	
	char * SendBuff = 0;
	char * RecvBuff = 0;
	char * PostHeader = 0;
	int ReadCount = 0;
		
	struct HTTPInfo * HTTPInfo = 0;
			
	if(HostName && Port && URL && METHOD)
	{
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[MakeRequest]-[IP:%s]-[Port:%u]-[Hostname:%s]-[URL:%s]-[Method:%s]-[PostData:%s]",
						      IP, Port, HostName, URL, METHOD, POSTDATA);
						      
		sock = socket(AF_INET, SOCK_STREAM, 0);
		if(sock)
		{
			// [TCC] Start the Clock.
			gettimeofday(&RequestTime, NULL);
			
			hostinfo = gethostbyname(IP);

			addr.sin_family = AF_INET;
			addr.sin_port = htons(Port);
			addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
			memset(&(addr.sin_zero), 0, 8);
			
			connected = connect(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr));
			                                                                                                                                                                                                
			if(connected != -1)
			{
				SendBuff = dsprintf(SendBuff, "%s %s HTTP/1.1\r\nUser-Agent: MIDASc httpbench\r\nHost: %s\r\n", METHOD, URL, HostName);
				
				if(!dstrcasecmp(METHOD, "POST"))
				{
					PostHeader = dsprintf(PostHeader, "Content-Length: %u\r\n\r\n%s\r\n", dstrlen(POSTDATA), POSTDATA);
					SendBuff = dstrcat(SendBuff, PostHeader);
					PostHeader = dfree(PostHeader);
				}
				SendBuff = dstrcat(SendBuff, "\r\n");
				
				SendToSocket(sock, SendBuff, "", 0);
				SendBuff = dfree(SendBuff);

				while(ReadCount < 10 && !RecvBuff)
				{
					RecvBuff = ReadPipe(sock);
					ReadCount++;
					
					if(!RecvBuff)
					{
						usleep(100);
					}
				}

				if(RecvBuff)
				{
					HTTPInfo = HTTPBENCH_ResponseInfo(RecvBuff, RequestTime.tv_usec);
				} else {
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[MakeRequest]-[Nothing returned by server]");
				}
					
				RecvBuff = dfree(RecvBuff);
				
			} else {
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[MakeRequest]-[Failed to connect]");
			}
		}
		close(sock);
	}

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MakeRequest]-[Exit]");
	
	return(HTTPInfo);
}
	
/******************************************************************************
*       char * ReadPipe(int Pipes)
*		Reads the piped output of spawned child process. Used by parent
*		process to read resulting response of childs check.
*
*	Args :
*		int Pipes	Pipe handle of child processe's stdout.
*
*       Updated :
******************************************************************************/
char * ReadPipe(int Pipes)
{
	char * OutStr = 0;
	char ReadBuff[1024];
	int len = 0;		
	struct timeval tv;
	fd_set sel_P;
	int status;
	
	do
	{
		FD_ZERO(&sel_P);
		FD_SET(Pipes, &sel_P);
		
		tv.tv_sec = 0;
		tv.tv_usec = 10;
		status = select(Pipes+1, &sel_P, NULL, NULL, &tv);
		if(status > 0)
		{
			len = read(Pipes, ReadBuff, 1023);
			if(len > 0)
			{
				ReadBuff[len]=0;
				OutStr = dstrcat(OutStr, ReadBuff);
			}
		}
		usleep(100);
	}while(status && status != -1 && len > 0);
	
	return(OutStr);

}

struct HTTPInfo * HTTPBENCH_FreeInfo(struct HTTPInfo * HTTPInfo)
{
	if(HTTPInfo)
	{
		HTTPInfo->StatusColor = dfree(HTTPInfo->StatusColor);
		HTTPInfo->Response = dfree(HTTPInfo->Response);
		HTTPInfo->ServerSoftware = dfree(HTTPInfo->ServerSoftware);
		
		HTTPInfo = (struct HTTPInfo *) dfree((char *) HTTPInfo);
	}
	
	return(HTTPInfo);
}

struct HTTPInfo * HTTPBENCH_ResponseInfo(char * Response, long RequestTime)
{
	struct HTTPInfo * HTTPInfo = 0;
	char * Headers = 0;
	char * Body = 0;
	struct timeval CurrentTime;
	char * HeaderType = 0;
	char * TempResponse = 0;
	
	if(Response)
	{
		HTTPInfo = (struct HTTPInfo *) drealloc((char *) HTTPInfo, sizeof(struct HTTPInfo));
		
		gettimeofday(&CurrentTime, NULL);
		
		if(CurrentTime.tv_usec >= RequestTime)
		{
			HTTPInfo->ResponseTime = CurrentTime.tv_usec - RequestTime;
		} else {
			HTTPInfo->ResponseTime = (CurrentTime.tv_usec + 1000000) - RequestTime;
		}
	
		HTTPInfo->TotalLen = dstrlen(Response);
		
		Headers = dstrsep(Response, "\r\n\r\n", &Body);
		
   		if(Headers)
		{
			HTTPInfo->HeaderLen = dstrlen(Headers);
			HTTPInfo->BodyLen = dstrlen(Body);
			
			TempResponse = dstrcpy(TempResponse, dstrsep(Headers, "\r\n", &Headers));
			HTTPInfo->Response = dstrcpy(HTTPInfo->Response, TempResponse);
			
			dstrsep(TempResponse, " ", &TempResponse);
			HTTPInfo->StatusNum = Myatoi(dstrsep(TempResponse, " ", &TempResponse));
			
			TempResponse = dfree(TempResponse);
			
			if(HTTPInfo->StatusNum >= 200 && HTTPInfo->StatusNum <= 299)
			{
				HTTPInfo->StatusColor = dstrcpy(HTTPInfo->StatusColor, "GREEN");
			} else {
				HTTPInfo->StatusColor = dstrcpy(HTTPInfo->StatusColor, "RED");
			}
			
			do
			{
				HeaderType = dstrsep(Headers, ":", &Headers);
				
				if(!dstrcasecmp(HeaderType, "Server"))
				{
					HTTPInfo->ServerSoftware = dstrcpy(HTTPInfo->ServerSoftware, dstrsep(Headers, "\r\n", (char **) -1));
				}
				dstrsep(Headers, "\r\n", &Headers);
			}while(HeaderType);
		}
	}

	return(HTTPInfo);
}
