/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : alert.c
* 		Functions to queue an alert to be processed by MIDASa
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "../Common/Commoninclude.h"

#include "includes.h"
#include "extern.h"

/******************************************************************************
*       void Alert_Process(struct XMLLeaf * CheckLeaf, char * TID)
*		Inserts an alert into the MIDAS_Alert_Queue table for MIDASa
*		to process.
*
*	Args :
*		struct XMLLeaf * CheckLeaf	XML linked list containing 
*						check options.
*		char * TID			Testing host ID.
*
*       Updated :
******************************************************************************/
void Alert_Process(struct XMLLeaf * CheckLeaf, char * TID, MYSQL * mysql)
{
	struct DBR * dbr;
	struct DBR * dbr2;
        MYSQL_ROW myrow;
        char * query = 0;
        char * HID = 0;
	char * CID = 0;
	char * STATUS = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Alert_Process]-[Enter]");
	
	if(CheckLeaf)
	{
		HID = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.HID");
		CID = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.CID");
		STATUS = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.STATUS");
		
		//
		// [TCC] This is not returning the correct EID in some cases.
		//
		query = dsprintf(query, "SELECT EID FROM MIDAS_Check_Events WHERE TID='%s' AND HID='%s' AND CID='%s' AND STATUS='%s' ORDER BY TimeStamp DESC LIMIT 1", 
						    TID, HID, CID, STATUS);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			if(dbr->num)
			{
				myrow = dbResult(dbr);
				query = dsprintf(query, "INSERT INTO MIDAS_Alert_Queue (EID) VALUES ('%s')", myrow[0]);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
			}
		}
		dbr = dbFree(dbr);
	}
}
