/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : maint.c
*		Main maint. functions. 
*
*       Updated :
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <mysql/mysql.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"


/******************************************************************************
*	void StartMaint(void)
*		Main entry function for Maint. cycle.
*
*	Updates :
******************************************************************************/
void StartMaint(void)
{
	MYSQL mysql;
	pid_t Main_Child = 0;
	pid_t Sub_Child = 0;
	char MAINT_ORDER[][20] = {
					"MAKE_BLUE",
					"PURGE_TABLE",
					"NET_SESSIONS",
					"CLEAN_ORPHANS",
					"UPDATE_RRD"
				};
	int MAINT_NUMBER = 5;
	int i = 0;
		
	if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.INTERVAL"))
	{
		for(;;)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartMaint]-[Starting Maint Cycle]-[Interval:%s]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.INTERVAL"));
    
			Main_Child = fork();
			if(!Main_Child)
			{		
        	        	dbConnect(&mysql, SQLCFG);
				
				for(i=0;i<MAINT_NUMBER;i++)
				{
					Sub_Child = fork();
					
					if(!Sub_Child)
					{
						if(!dstrcmp(MAINT_ORDER[i], "MAKE_BLUE"))
						{
							MAINT_MakeBlue(&mysql);
						}
						
						if(!dstrcmp(MAINT_ORDER[i], "PURGE_TABLE"))
						{
							MAINT_PurgeTable(&mysql, "MIDAS_Check_Events", 
										 "TimeStamp", 
										 ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.CHECKEVENTS"), 
										 "DAY");
							MAINT_PurgeTable(&mysql, "MIDAS_IDS_Events", 
										 "TimeStamp", 
										 ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.IDSEVENTS"), 
										 "DAY");
							MAINT_PurgeTable(&mysql, "MIDAS_MACIP", 
										 "LastSeen", 
										 ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.MACIP"), 
										 "HOUR");
						}
						
						if(!dstrcmp(MAINT_ORDER[i], "NET_SESSIONS"))
						{
							MAINT_PurgeNetSessions(&mysql);
							//MAINT_PurgeNetSessionStats(&mysql);
						}
						
						if(!dstrcmp(MAINT_ORDER[i], "CLEAN_ORPHANS"))
						{
							MAINT_CleanOrphans(&mysql);
						}
						
						if(!dstrcmp(MAINT_ORDER[i], "UPDATE_RRD"))
						{
							MAINT_UpdateRRDStats(&mysql);
						}
						exit(0);
					} else {
						waitpid(Sub_Child, NULL, 0);
					}
				}
				dbClose(&mysql);
				exit(0);
	                } else {
	                	waitpid(Main_Child, NULL, 0);
	                }
	                
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartMaint]-[End Cycle]");
			
	                sleep(Myatoll(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.INTERVAL")));
	        }
	}
}

/******************************************************************************
*	void MAINT_MakeBlue(MYSQL * mysql)
*		Update MIDAS_Check_Current records. Sets any check that is
*		overdue for an update to BLUE status.
*
*	Args :
*		MYSQL * mysql	Mysql connection handle
*
*	Updates :
******************************************************************************/
void MAINT_MakeBlue(MYSQL * mysql)
{
        struct DBR * dbr = 0;
        char * query = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_MakeBlue]-[Enter]");
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.MAKEBLUE")))
	{
        	query = dsprintf(query, "UPDATE MIDAS_Check_Current SET Status='BLUE' WHERE LastCheck <= DATE_SUB(NOW(), INTERVAL %s MINUTE)", 
        				ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.MAKEBLUE"));
        	dbr = dbQuery_P(mysql, query);
        	if(dbr)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_MakeBlue]-[%ld checks BLUE'ed]", dbr->num);
		}
		
        	query = dfree(query);
        	dbr = dbFree(dbr);
        }
        MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_MakeBlue]-[Exit]");
}

/******************************************************************************
*	void MAINT_PurgeTable(MYSQL * mysql, char * Table, char * Field, char * TTL, char * Interval)
*		Generic table purge. Will purge purge records where 'field' 
*		is older then 'TTL Interval's
*
*	Args :
*		MYSQL * mysql		Mysql connection handle
*		char * Table		SQL Table to use
*		char * Field		Field to use as Timestamp value.
*		char * TTL		Time To Live for Records.
*		char * Interval		Interval TTL is expressed in
*
*	Updates :
******************************************************************************/
void MAINT_PurgeTable(MYSQL * mysql, char * Table, char * Field, char * TTL, char * Interval)
{
	char * query = 0;
	struct DBR * dbr = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_PurgeTable]-[Table:%s]-[Field:%s]-[TTL:%s]-[INT:%s]", Table, Field, TTL, Interval);
	if(Myatoi(TTL))
	{
		query = dsprintf(query, "DELETE FROM %s WHERE %s <= DATE_SUB(NOW(), INTERVAL %s %s)",
					Table, Field, TTL, Interval);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		if(dbr)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_PurgeTable]-[%ld Records purged]", dbr->num);
		}
		
		dbr = dbFree(dbr);
	}
	MyDebug(__LINE__, __FILE__, LOG_INFO, "MAINT_PurgeTable]-[Exit]");
}


/******************************************************************************
*	void MAINT_UpdateRRDStats(MYSQL * mysql)
*		Updated RRD Data Values from the RRD_Values_Queue.
*
*	Args :
*		MYSQL * mysql	The MySQL connection handle
*
*	Updated :
*****************************************************************************/
void MAINT_UpdateRRDStats(MYSQL * mysql)
{
	char * query = 0;
	struct DBR * dbr = 0;
	struct DBR * dbr2 = 0;
	int i;
	MYSQL_ROW myrow;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_UpdateRRDStats]-[Enter]");

	query = dsprintf(query, "SELECT DSID, TimeStamp, Value FROM RRD_Values_Queue WHERE HID='%s' ORDER BY TimeStamp, DSID ASC", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	
	if(dbr)
	{
		for(i=0;i<dbr->num;i++)
		{
			myrow = dbResult(dbr);
			
			RRD_UpdateDSValue(Myatoi(myrow[0]), Myatoll(myrow[1]), Myatof(myrow[2]), mysql);
			
			query = dsprintf(query, "DELETE FROM RRD_Values_Queue WHERE DSID='%s' AND HID='%s' AND TimeStamp='%s'", 
						myrow[0], 
						ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"), 
						myrow[1]);
			dbr2 = dbQuery_P(mysql, query);
			query = dfree(query);
			dbr2 = dbFree(dbr2);
		}
	}
	dbr = dbFree(dbr);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_UpdateRRDStats]-[Exit]");

}

/******************************************************************************
*	void MAINT_CleanOrphans(MYSQL * mysql)
*		Removes Orphanded Check history from Check_Current and 
*		Check_Events tables.
*
*	Args :
*		MYSQL * mysql	MySQL connection handle.
*
*	Updated:
******************************************************************************/
void MAINT_CleanOrphans(MYSQL * mysql)
{
	char * query = 0;
	struct DBR * dbr = 0;
	struct DBR * dbr2 = 0;
	int i;
	MYSQL_ROW myrow;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_CleanOrphans]-[Enter]");

	query = dsprintf(query, "SELECT DISTINCT(MIDAS_Check_Current.HID) FROM MIDAS_Check_Current LEFT JOIN MIDAS_Hosts ON (MIDAS_Hosts.HID = MIDAS_Check_Current.HID) WHERE HostName is NULL");
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	
	if(dbr)
	{
		for(i=0;i<dbr->num;i++)
		{
			myrow = dbResult(dbr);
		
			query = dsprintf(query, "DELETE FROM MIDAS_Check_Current WHERE HID='%s' OR TID='%s'", myrow[0], myrow[0]);
			dbr2 = dbQuery_P(mysql, query);
			query = dfree(query);
			dbr2 = dbFree(dbr2);
		}
	}
	dbr = dbFree(dbr);
	
	query = dsprintf(query, "SELECT DISTINCT(MIDAS_Check_Events.HID) FROM MIDAS_Check_Events LEFT JOIN MIDAS_Hosts ON (MIDAS_Hosts.HID = MIDAS_Check_Events.HID) WHERE HostName is NULL");
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	
	if(dbr)
	{
		for(i=0;i<dbr->num;i++)
		{
			myrow = dbResult(dbr);
		
			query = dsprintf(query, "DELETE FROM MIDAS_Check_Events WHERE HID='%s' OR TID='%s'", myrow[0], myrow[0]);
			dbr2 = dbQuery_P(mysql, query);
			query = dfree(query);
			dbr2 = dbFree(dbr2);
		}
	}
	dbr = dbFree(dbr);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_CleanOrphans]-[Enter]");
	
}

/******************************************************************************
*	void MAINT_PurgeShortNetSessions(MYSQL * mysql)
*		Removes short sessions from the MIDAS_NetMon_Sessios tables.
*
*	Args :
*		MYSQL * mysql	MySQL connection handle.
*
*	Updated:
******************************************************************************/
void MAINT_PurgeNetSessions(MYSQL * mysql)
{
	char * query = 0;
	struct DBR * dbr = 0;

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_PurgeShortSessions]-[Enter]");
	
	if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAIN.SHORTSESSIONS"))
	{
		query = dsprintf(query, "DELETE FROM MIDAS_NetMon_Sessions WHERE EndTime < DATE_SUB(NOW(), %s DAY)", 
					ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAINT.SESSIONS"));
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
	}

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_PurgeShortSessions]-[Exit]");
}

/******************************************************************************
*	void MAINT_PurgeNetSessionStats(MYSQL * mysql)
*		Removes Netsession records that only have 1 record in Data 
*		Source
*
*	Args :
*		MYSQL * mysql	Mysql connection handle.
*
*	Updates :
******************************************************************************/
void MAINT_PurgeNetSessionStats(MYSQL * mysql)
{
	char * query = 0;
	struct DBR * dbr = 0;
	int i = 0;
	MYSQL_ROW myrow;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_PurgeNetSessions]-[Enter]");
	
	query = dsprintf(query, "SELECT DSID, COUNT(Sequence) FROM RRD_DataValues GROUP BY DSID ORDER BY 2 ASC");
	dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	if(dbr)
	{
		for(i=0;i<dbr->num;i++)
		{
			myrow = dbResult(dbr);
			
			if(Myatoi(myrow[1]) < 2)
			{
				MAINT_RemoveDSID(mysql, myrow[0]);
			} else {
				break;
			}
		}
	}
	dbr = dbFree(dbr);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_PurgeNetSessions]-[Exit]");
}

/******************************************************************************
*	void MAINT_RemoveDSID(MYSQL msyql, char * DSID)
*		Removes records from DataSource, DataValues, ArchiveSource and
*		ArchiveValues based on a given DSID.
*
*	Args :
*		MYSQL * mysql		Mysql connection handle
*		char * DSID		DSID of records to remove.
*
*	Updates :
******************************************************************************/
void MAINT_RemoveDSID(MYSQL * mysql, char * DSID)
{
	char * query = 0;
	struct DBR * dbr = 0;
	struct DBR * dbr2 = 0;
	int i;
	MYSQL_ROW myrow;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_RemoveDSID]-[DSID:%s]", DSID);
	
	if(DSID)
	{	
		query = dsprintf(query, "DELETE FROM RRD_DataSource WHERE DSID='%s'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
				
		query = dsprintf(query, "DELETE FROM RRD_DataValues WHERE DSID='%s'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
				
		query = dsprintf(query, "SELECT DAID FROM RRD_ArchiveSource WHERE DSID='%s'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
					
		if(dbr)
		{
			for(i=0;i<dbr->num;i++)
			{
				myrow = dbResult(dbr);
				
				query = dsprintf(query, "DELETE FROM RRD_ArchiveValues WHERE DAID='%s'", myrow[0]);
				dbr2 = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr2 = dbFree(dbr2);
			}
		}
		dbr = dbFree(dbr);
	
		query = dsprintf(query, "DELETE FROM RRD_ArchiveSource WHERE DSID='%s'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);
				
		query = dsprintf(query, "DELETE FROM RRD_MIDAS_ID WHERE DSID='%s'", DSID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		dbr = dbFree(dbr);	
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[MAINT_RemoveDSID]-[Exit]");
	
}
