/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : processclient.c
* 		Functions needed to process client requests.
*
*       Updated :
*		03/19/2004 - Fixed update for MIDASs clientsynch portion
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <time.h>

#include "../Common/Commoninclude.h"

#include "includes.h"
#include "extern.h"

//#define dbFree(x) printf("--[%d, %s:%ld]",(x = dbFree(x)),  __FILE__, __LINE__);

/******************************************************************************
*       char * ProcessClientRequest_InSession(struct XMLLeaf * ClientXMLTree)
*		Main entry function for processing client requests during the 
*		client/server session. Fires off specific function based on 
*		value of CHECK.TYPE tag in client request.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML linked list cont-
*							aining check request
*							options.
*
*       Updated :
*		11/20/2003 - Sanity check added. Valid HID required before 
*			     accepting client query.
*		04/10/2004 - Broke Function into 2, InSession and OutOfSession.
******************************************************************************/
char * ProcessClientRequest_InSession(struct XMLLeaf * ClientXMLTree)
{
	struct XMLLeaf * LocalLeaf;
	char * Response = 0;
	char * tResponse = 0;
	char * HID;
	MYSQL mysql;
	
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequiest_InSession]-[Enter]");
	
	Response = dsprintf(Response, "<SERVER><VER>%s</VER><HID>%s</HID>", MIDASVER, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
	
	LocalLeaf = ClientXMLTree;
	if(LocalLeaf)
	{
		dbConnect(&mysql, SQLCFG);
		
		HID = ReturnXMLLeafValue(LocalLeaf, "CLIENT.HID");
		
		if(ValidHID(HID, &mysql))
		{
			while(LocalLeaf)
			{
				LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CLIENT.TYPE");
				if(LocalLeaf)
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequest_InSession]-[HID:%s]-[Type:%s]", HID, LocalLeaf->Value);
						
					if(!dstrcasecmp(LocalLeaf->Value, "RETURNCHECKS"))
					{
						tResponse = ReturnClientChecks(HID, &mysql);
					}
						
					if(!dstrcasecmp(LocalLeaf->Value, "SNORTCONFIG"))
					{
						tResponse = ReturnSnortConfig(ClientXMLTree, HID, &mysql);
					}
						
					if(!dstrcasecmp(LocalLeaf->Value, "HIDQUERY"))
					{
						tResponse = ReturnHIDQuery(LocalLeaf, &mysql);
					}
						
					if(!dstrcasecmp(LocalLeaf->Value, "RETURNNETMON"))
					{
						tResponse = ReturnNetMon(HID, &mysql);
					}
					
					Response = dstrcat(Response, tResponse);
					tResponse = dfree(tResponse);
						
					LocalLeaf = LocalLeaf->NextLeaf;
				}			
			}
			
		}
		dbClose(&mysql);
	} else {
		Response = dstrcat(Response, "<ERROR>INVALID_HID</ERROR>");
	}		
	Response = dstrcat(Response, "</SERVER>");
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequiest_InSession]-[Exit]");
	return(Response);
}

/******************************************************************************
*       char * ProcessClientRequest_OutOfSession(struct XMLLeaf * ClientXMLTree)
*		Main entry function for processing client requests during the 
*		client/server session. Fires off specific function based on 
*		value of CHECK.TYPE tag in client request.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML linked list cont-
*							aining check request
*							options.
*
*       Updated :
******************************************************************************/
char * ProcessClientRequest_OutOfSession(struct XMLLeaf * ClientXMLTree)
{
	struct XMLLeaf * LocalLeaf;
	char * tResponse = 0;
	char * HID;
	MYSQL mysql;
		
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequiest_OutOfSession]-[Enter]");
	
	LocalLeaf = ClientXMLTree;
	if(LocalLeaf)
	{
		dbConnect(&mysql, SQLCFG);
		
		HID = ReturnXMLLeafValue(LocalLeaf, "CLIENT.HID");
		
		if(ValidHID(HID, &mysql))
		{
			while(LocalLeaf)
			{
				LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CLIENT.TYPE");
				if(LocalLeaf)
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequest_OutOfSession]-[HID:%s]-[Type:%s]", HID, LocalLeaf->Value);
						
					if(!dstrcasecmp(LocalLeaf->Value, "CHECKUPDATE"))
					{
						tResponse = UpdateCheckStatus(LocalLeaf, HID, &mysql);
					}
						
					if(!dstrcasecmp(LocalLeaf->Value, "SNORTALERT"))
					{
						tResponse = UpdateSnortAlert(LocalLeaf, HID, &mysql);
					}
						
					if(!dstrcasecmp(LocalLeaf->Value, "NETSTAT"))
					{
						tResponse = UpdateNetStats(LocalLeaf, HID, &mysql);
					}
					
					if(!dstrcasecmp(LocalLeaf->Value, "MACADDRESSES"))
					{
						tResponse = UpdateMACIP(LocalLeaf, HID, &mysql);
					}
					
					if(!dstrcasecmp(LocalLeaf->Value, "NETSESSIONS"))
					{
						tResponse = UpdateNetworkSessions(LocalLeaf, HID, &mysql);
					}
						
					tResponse = dfree(tResponse);
						
					LocalLeaf = LocalLeaf->NextLeaf;
				}			
			}
			
			LocalLeaf = ClientXMLTree;
			if(FindXMLLeafByTag(LocalLeaf,  "CLIENT.STAT"))
			{
				ProcessClientStats(LocalLeaf, "CLIENT", HID, &mysql);
			}
		}
		dbClose(&mysql);
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequiest_OutOfSession]-[Exit]");

	return(NULL);
}

/******************************************************************************
*	int ValidHID(char * HID, struct DBR *)
*		Returns 1 if HID is valid (present) or 0 if not.
*
*	Args :
*		char * HID		Host ID to check.
*		 struct DBR * dbr	DBR handle
*	Updated :
*
******************************************************************************/
int ValidHID(char * HID, MYSQL * mysql)
{
        char * query = 0;
        int Status = 0;
	struct DBR * dbr = 0;

        query = dsprintf(query, "SELECT HID FROM MIDAS_Hosts WHERE HID='%s'", HID);
        dbr = dbQuery_P(mysql, query);
        if(dbr)
        {
        	Status = dbr->num;
        }
        dbr = dbFree(dbr);

        return(Status);
}

/******************************************************************************
*       char * ReturnClientChecks(char * HID, MYSQL * mysql)
*		Response for RETURNCHECKS query. This will build the list of
*		checks for the requesting MIDASc to perform.
*
*	Args :
*		char * HID	The Host ID of the requesting MIDASc.
*
*       Updated :
******************************************************************************/
char * ReturnClientChecks(char * HID, MYSQL * mysql)
{
        MYSQL_ROW myrow;
        char * query = 0;
        int i;
        char * Response = 0;
        char * Segment = 0;
	struct DBR * dbr = 0;
	
	char SELECT[] = "SELECT MIDAS_Host_Checks.HID, MIDAS_Host_Checks.CID, MIDAS_Hosts.HostName, MIDAS_Hosts.IPAddr, MIDAS_Checks.Type, "
			"MIDAS_Host_Checks.Options, MIDAS_Checks.Port, MIDAS_Checks.ConnectOnly "
	                "FROM MIDAS_Host_Checks "
	                "LEFT JOIN MIDAS_Checks ON (MIDAS_Host_Checks.CID = MIDAS_Checks.CID) "
	                "LEFT JOIN MIDAS_Hosts ON (MIDAS_Hosts.HID = MIDAS_Host_Checks.HID) "
	        	"WHERE MIDAS_Host_Checks.TID='%s' AND MIDAS_Host_Checks.Enabled='1'";
		
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnClientChecks]-[HID:%s]", HID);
	
        query = dsprintf(query, SELECT, HID);
        dbr = dbQuery_P(mysql, query);
	query = dfree(query);
	
	if(dbr)
	{
		if(dbr->num)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnClientChecks]-[Number of Checks:%u]", dbr->num);
			Response = dstrcat(Response, "<RETURNCHECKS>");
        		for(i=0;i<dbr->num;i++)
        		{
        		        myrow = dbResult(dbr);
		
				Segment = dsprintf(Segment, "<CHECK><TID>%s</TID><HID>%s</HID><CID>%s</CID><HOSTNAME>%s</HOSTNAME><IP>%s</IP><TYPE>%s</TYPE><OPTION>%s</OPTION><PORT>%s</PORT><CONNECTONLY>%s</CONNECTONLY></CHECK>",
							    HID, myrow[0], myrow[1], myrow[2], myrow[3], myrow[4], myrow[5], myrow[6], myrow[7]);
				Response = dstrcat(Response, Segment);
				Segment = dfree(Segment);
		        }
			Response = dstrcat(Response, "</RETURNCHECKS>");
		}
	}
	dbr = dbFree(dbr);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnClientChecks]-[Exit]");
	
	return(Response);
}

/******************************************************************************
*       char * UpdateCheckStatus(struct XMLLeaf * ClientXMLTree, char * TID)
*		Updats the status of checks based on the results returned by
*		a MIDASc client. This is used for CHECKUPDATE request.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML linked list cont-
*							aining check statuses.
		char * TID				The MIDASc's Host ID.
*
*       Updated :
*		11/18/03 - CurFail now resets when an alert is issued.
*		11/20/03 - Fixed Update record. Now retains FirstCheck value.
*		02/10/04 - Rewrote this function to actualy work correctly.
******************************************************************************/
char * UpdateCheckStatus(struct XMLLeaf * ClientXMLTree, char * TID, MYSQL * mysql)
{
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * CheckLeaf = 0;
	char * query = 0;
	char * Response = 0;
	char * HID = 0;
	char * CID = 0;
	char * STATUS = 0;
	char * DETAIL = 0;
	unsigned CheckCount = 0;
	struct DBR * dbr = 0;
	int IssueAlert = 0;
	int NewRecord = 0;
	char * PrevStatus = 0;
	int LogCurrent = 0;
	int LogEvent = 0;
	
	LocalLeaf = ClientXMLTree;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[TID:%s]", TID);
	
	LocalLeaf = ClientXMLTree;
		
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "CLIENT.CHECKUPDATE.CHECK"), 1);
		if(CheckLeaf)
		{
			CheckCount++;
			HID = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.HID");
			CID = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.CID");
			STATUS = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.STATUS");
			DETAIL = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.DETAIL");
			
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
			
			IssueAlert = 0;
			NewRecord = 0;
			LogCurrent = 1;
			LogEvent = 0;
			
			PrevStatus = UpdateCheckStatus_PrevStatus(TID, HID, CID, mysql);
			
			// If No Prev Status, then its a virgin Check and we log/alert every thing.
			if(!PrevStatus)
			{
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[New Event records, Issues Alert, And Logging event]");
				
				IssueAlert = 1;
				NewRecord = 1;
				LogEvent = 1;
			}
			
			/* [TCC] 03/09/04
				Flow for GREEN :
					... Reset CurFail to 0.
					... If STATUS != PrevStatus and isnt a BLUE We LogEvent. 
						... If a Red Alert has been issues issue a Green alert. Reset RedAlertIssuedFlag to 0;
			*/
			if(!dstrcasecmp(STATUS, "GREEN"))
			{
				// Green Status, we reset the CurFail to 0.
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[GREEN Status, reseting CurFail to 0]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
				
				query = dsprintf(query, "UPDATE MIDAS_Host_Checks SET CurFail=0 WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
				dbr = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr = dbFree(dbr);
				
				// If Current status is diff then prev status and if PrefStatus != BLUE issue Alert and Log
				if(dstrcasecmp(STATUS, PrevStatus) && dstrcasecmp(PrevStatus, "BLUE"))
				{
 					LogEvent = 1;
					
					// New Status so we update the FirstCheck to the current time.
					query = dsprintf(query, "UPDATE MIDAS_Check_Current SET FirstCheck=NOW() WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr = dbFree(dbr);
					
					// Has an Alert been issued for this check? If not we do not issue a green alert.
					if(UpdateCheckStatus_RedAlertIssued(TID, HID, CID, mysql))
					{
						MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Status Change. Issuing alert]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s -> %s]", TID, HID, CID, PrevStatus, STATUS);
						IssueAlert = 1;
						
						// Reset the RedAlertFlag back to 0.
						query = dsprintf(query, "UPDATE MIDAS_Host_Checks SET RedAlertIssued=0 WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
						dbr = dbQuery_P(mysql, query);
						query = dfree(query);
						dbr = dbFree(dbr);
					}
				}
			}
			
			/* [TCC] 03/09/04
				Flow for RED :
					... Increment CurFail and TotFail.
					... If AlertOnRed toggled on? If so IssueAlert and LogEvent
					... If this is a reaccuring Status (ie PrevStatus = STATUS) we see if we have reached our max. If so we Log/Issue.
					... If this is a transition we Update Check_Current and Log.
						... If we have reached the MinFail we Alert.
			*/
			if(!dstrcasecmp(STATUS, "RED") || !dstrcasecmp(STATUS, "YELLOW"))
			{
				// Red status Increment of CurFail and TotFail
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Incrementing CurFail and TotFail]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
				
				query = dsprintf(query, "UPDATE MIDAS_Host_Checks SET CurFail=CurFail+1, TotFail=TotFail+1 WHERE TID='%s' AND HID='%s' AND CID='%s'",
							TID, HID, CID);
				dbr = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr = dbFree(dbr);
			
				// Is Status RED and AlertOnRed toggled on?
				if(UpdateCheckStatus_AlertOnRed(CID, mysql))
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[AlertOnRed Flagged issuing alert]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
					LogEvent = 1;
					IssueAlert = 1;
				}
			
				// Is PrevStatus = CurStatus and is CurFail>=MinFail? If so Log/Alert
				if(!dstrcasecmp(PrevStatus, STATUS) && UpdateCheckStatus_ReachMaxAlerts(TID, HID, CID, mysql, 1))
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Status Red and CurFai>=MinFail. Issuing alert]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
					IssueAlert = 1;
					LogEvent = 1;
				}
				
				// If Current status is diff then prev status and if PrefStatus != BLUE Log event
				if(dstrcasecmp(STATUS, PrevStatus) && dstrcasecmp(PrevStatus, "BLUE"))
				{
					// New Status so we update the FirstCheck to the current time.
					query = dsprintf(query, "UPDATE MIDAS_Check_Current SET FirstCheck=NOW() WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr = dbFree(dbr);
					
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Status Change. Adding to Event Log]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s -> %s]", TID, HID, CID, PrevStatus, STATUS);
					LogEvent = 1;
					
					if(UpdateCheckStatus_ReachMaxAlerts(TID, HID, CID, mysql, 0))
					{
						IssueAlert = 1;
					}
				}
			}
			PrevStatus = dfree(PrevStatus);
			
			if(LogCurrent)
			{
				if(NewRecord)
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Virgin Check Status]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
					
					query = dsprintf(query, "INSERT INTO MIDAS_Check_Current (TID, HID, CID, Response, Status, LastCheck, FirstCheck) VALUES ('%s', '%s', '%s', '%s', '%s', NOW(), NOW())",
								TID, HID, CID, DETAIL, STATUS);	
				} else {
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Updating Current Status]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
					
					query = dsprintf(query, "UPDATE MIDAS_Check_Current SET Status='%s', Response='%s', LastCheck=NOW() WHERE TID='%s' AND HID='%s' AND CID='%s'",
								STATUS, DETAIL, TID, HID, CID);
				}
				dbr = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr = dbFree(dbr);
			}
			
			if(LogEvent)
			{
				// Add Event records
				query = dsprintf(query, "INSERT INTO MIDAS_Check_Events (TID, HID, CID, Response, Status, TimeStamp) VALUES ('%s', '%s', '%s', '%s', '%s', NOW())",
								TID, HID, CID, DETAIL, STATUS);
				dbr = dbQuery_P(mysql, query);
				query = dfree(query);
				dbr = dbFree(dbr);
			}
			
			// If IssueAlert then issue an Alert
			if(IssueAlert)
			{
				// Is DoAlert toggled on?
				if(UpdateCheckStatus_DoAlert(TID, HID, CID, mysql))
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Issueing Alert]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
					
					// Reset CurFail to 0.			
					query = dsprintf(query, "UPDATE MIDAS_Host_Checks SET CurFail=0 WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr = dbFree(dbr);
					
					// If Status is not GREEN then we toggle the RedAlertIssed on and Set the RedAlertIssuesOn to NOW()
					if(dstrcasecmp(STATUS, "GREEN"))
					{
						query = dsprintf(query, "UPDATE MIDAS_Host_Checks SET RedAlertIssued=1, RedAlertIssuedOn=NOW() WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
						dbr = dbQuery_P(mysql, query);
						query = dfree(query);
						dbr = dbFree(dbr);
					}
					
					// Queue the Alert
					Alert_Process(CheckLeaf, TID, mysql);
				} else {
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[DoAlert Toggled off. Ignoring alert]-[TID:%s]-[HID:%s]-[CID:%s]-[STATUS:%s]", TID, HID, CID, STATUS);
				}
			}
			
			// Process the stats for this check update.
			if(FindXMLLeafByTag(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.STAT"))
			{
				ProcessClientStats(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK", HID, mysql);
			}
			
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);
		}
		
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}		
	}
	
	Response = dstrcpy(Response, "<TYPE>CHECKUPDATE</TYPE><CHECKUPDATE>FINISHED</CHECKUPDATE>");
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Checks Updated:%u]-[Exiting]", CheckCount);
	return(Response);	
}	

/******************************************************************************
*	char * UpdateCheckStatus_PrevStatus(char * TID, char * HID, char * CID, MYSQL * mysql)	
*		Returns the current Check Status.
*	Args :
*		char * TID	Tester ID
*		char * HID	Host ID
*		char * CID	Check ID
*		MYSQL * mysql	Mysql connection handle
*
*       Updated :
******************************************************************************/
char * UpdateCheckStatus_PrevStatus(char * TID, char * HID, char * CID, MYSQL * mysql)
{
	char * Status = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	MYSQL_ROW myrow;
	
	if(TID && HID && CID)
	{
		query = dsprintf(query, "SELECT Status FROM MIDAS_Check_Current WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			if(dbr->num)
			{
				myrow = dbResult(dbr);
				
				Status = dstrcpy(Status, myrow[0]);
			}
		}
		dbr = dbFree(dbr);
	}
	
	return(Status);
}

/******************************************************************************
*	int UpdateCheckStatus_AlertOnRed(char * CID, MYSQL * mysql)	
*		Returns 0 if AlertOnRed is not flagged, 1 if it is.
*
*	Args :
*		char * CID	Check ID
*		MYSQL * mysql	Mysql connection handle
*
*       Updated :
******************************************************************************/		
int UpdateCheckStatus_AlertOnRed(char * CID, MYSQL * mysql)
{
	int Status = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	
	if(CID)
	{
		query = dsprintf(query, "SELECT AlertOnRed FROM MIDAS_Checks WHERE CID='%s' AND AlertOnRed=1", CID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
	
		if(dbr)
		{
			Status = dbr->num;
		}
		dbr = dbFree(dbr);
	}
	
	return(Status);
}

/******************************************************************************
*	int UpdateCheckStatus_ReachMaxAlerts(char * TID, char * HID, char * CID, MYSQL * mysql)	
*		Returns 0 if CurFail >= MinFail, otherwise returns 1.
*
*	Args :
*		char * TID	Tester ID
*		char * HID	Host ID
*		char * CID	Check ID
*		MYSQL * mysql	Mysql connection handle
*		
*       Updated :
******************************************************************************/
int UpdateCheckStatus_ReachMaxAlerts(char * TID, char * HID, char * CID, MYSQL * mysql, int RequireMinFail)
{
	int Status = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	
	if(TID && HID && CID)
	{
		if(RequireMinFail)
		{
			query = dsprintf(query, "SELECT CurFail FROM MIDAS_Host_Checks WHERE TID='%s' AND HID='%s' AND CID='%s' AND CurFail >= MinFail AND MinFail > 0", TID, HID, CID);
		} else {
			query = dsprintf(query, "SELECT CurFail FROM MIDAS_Host_Checks WHERE TID='%s' AND HID='%s' AND CID='%s' AND CurFail >= MinFail", TID, HID, CID);
		}
		
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
	
		if(dbr)
		{
			Status = dbr->num;
		}
		dbr = dbFree(dbr);
	}
	
	return(Status);
}

/******************************************************************************
*	int UpdateCheckStatus_DoAlert(char * TID, char * HID, char * CID, MYSQL * mysql)	
*		Returns 1 if DoAlert is toggled on, or 0 if off.
*
*	Args :
*		char * TID	Tester ID
*		char * HID	Host ID
*		char * CID	Check ID
*		MYSQL * mysql	Mysql connection handle
*		
*       Updated :
******************************************************************************/
int UpdateCheckStatus_DoAlert(char * TID, char * HID, char * CID, MYSQL * mysql)
{
	int Status = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	
	if(TID && HID && CID)
	{
		query = dsprintf(query, "SELECT DoAlert FROM MIDAS_Host_Checks WHERE TID='%s' AND HID='%s' AND CID='%s' AND DoAlert=1", TID, HID, CID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
	
		if(dbr)
		{
			Status = dbr->num;
		}
		dbr = dbFree(dbr);
	}
	
	return(Status);
}

/******************************************************************************
*	int UpdateCheckStatus_RedAlertIssued(char * TID, char * HID, char * CID, MYSQL * mysql)	
*		Returns 1 if RedAlertIssed is on, or 0 if off.
*
*	Args :
*		char * TID	Tester ID
*		char * HID	Host ID
*		char * CID	Check ID
*		MYSQL * mysql	Mysql connection handle
*		
*       Updated :
******************************************************************************/
int UpdateCheckStatus_RedAlertIssued(char * TID, char * HID, char * CID, MYSQL * mysql)
{
	int Status = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	
	if(TID && HID && CID)
	{
		query = dsprintf(query, "SELECT RedAlertIssued FROM MIDAS_Host_Checks WHERE TID='%s' AND HID='%s' AND CID='%s' AND RedAlertIssued=1", TID, HID, CID);
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
	
		if(dbr)
		{
			Status = dbr->num;
		}
		dbr = dbFree(dbr);
	}
	
	return(Status);
}

	
/******************************************************************************
*       char * ReturnSnortConfig(char * HID)
*		Builds the config response for MIDASs when it performs a 
*		SNORTCONFIG query.
*
*	Args :
*		char * HID	The Host id of the requesting MIDASs client.
*
*       Updated :
		2004-03		Rodney Caudle
			updated this to support new MIDASs configuration module
******************************************************************************/
char * ReturnSnortConfig(struct XMLLeaf * ClientXMLTree, char * HID, MYSQL * mysql)
{
	MYSQL_ROW myrow;
	char * query = 0;
	char * Response = 0;
	char * SnortConfig = 0;
	char * B64SnortConfig = 0;
	struct DBR * dbr = 0;
	unsigned i = 0;
	char * VERSION;
        VERSION = ReturnXMLLeafValue(ClientXMLTree, "CLIENT.SNORTCONFIG.VERSION");
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnSnortConfig]-[HID:%s]-[VERSION:%s]", HID, VERSION);

// 	Need to check the STATUS first so we don't send a partial configuration
  	query = dsprintf(query, "SELECT * from MIDAS_SNORT_Status where HID='%s' and STATUS='FULLY_CONFIGURED' or STATUS='CLIENT_N_SYNCH'", HID);
  	MyDebug(__LINE__, __FILE__, LOG_SQL, "[ReturnSnortConfig]-[QUERY:%s]", query);
  	dbr = dbQuery_P(mysql, query);
  	query = dfree(query);

  	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnSnortConfig]-[Available Configs %d]", dbr->num);

// 	If it's a good config then start grabbing it
  	if (dbr) {
		if (dbr->num > 0) {
// 			Free the DB structure
  			dbr = dbFree(dbr);
    
//  			query = dsprintf(query, "SELECT Config FROM MIDAS_SNORT_Conf WHERE HID='%s'", HID);
  			query = dsprintf(query, "SELECT Section,Details FROM MIDAS_SNORT_Sections WHERE HID=%s ORDER BY NORDER", HID);
  			MyDebug(__LINE__, __FILE__, LOG_SQL, "[ReturnSnortConfig]-[QUERY:%s]", query);
  			dbr = dbQuery_P(mysql, query);
	  		query = dfree(query);
  			
  			MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnSnortConfig]-[Matching %d lines]", dbr->num);
	
  			if(dbr) {
    				for(i=0; i < dbr->num; i++) {
      					myrow = dbResult(dbr);
      					SnortConfig = dstrcat(SnortConfig, myrow[1]);
      					SnortConfig = dstrcat(SnortConfig, "\n");
      					MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnSnortConfig]-[AddToConfig]-[%d]-[%s] %s", i,myrow[0],myrow[1]);
    				}
  			}
// 			Free the DB structure
  			dbr = dbFree(dbr);
	
//  			Add a spacer for the ruleset
  			SnortConfig = dstrcat(SnortConfig, "#\n## Ruleset Starting\n#\n");
  			MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnSnortConfig]-[spaceradded]-[%s]", SnortConfig);
	
  			query = dsprintf(query, "select MIDAS_SNORT_Rulebase.Rule%s from MIDAS_SNORT_Rulebase, MIDAS_SNORT_HID_x_RID where MIDAS_SNORT_Rulebase.RID = MIDAS_SNORT_HID_x_RID.RID and MIDAS_SNORT_HID_x_RID.HID=%s", VERSION, HID);

  			MyDebug(__LINE__, __FILE__, LOG_SQL, "[ReturnSnortConfig]-[QUERY:%s]", query);
  			dbr = dbQuery_P(mysql, query);
  			query = dfree(query);
  			if(dbr) {
    				for(i=0;i<dbr->num;i++) {
      					myrow = dbResult(dbr);
      					SnortConfig = dstrcat(SnortConfig, myrow[0]);
      					SnortConfig = dstrcat(SnortConfig, "\n");
    				}
    				query = dsprintf(query, "UPDATE MIDAS_SNORT_Status SET Status='CLIENT_N_SYNCH', LastSynch=NOW() WHERE HID='%s'", HID);
    				dbr = dbQuery_P(mysql, query);
    				query = dfree(query);
    				dbr = dbFree(dbr);
  			}
		} else {
			SnortConfig = dstrcat(SnortConfig, "ERROR: This sensor is not fully configured.  Please return to the configuration wizard and complete the configuration before continuing.");
		}
	} else {
		SnortConfig = dstrcat(SnortConfig, "ERROR: This sensor is not fully configured.  Please return to the configuration wizard and complete the configuration before continuing.");
	}

//	Now encode and send
  	B64SnortConfig = Base64Enc(SnortConfig, dstrlen(SnortConfig));
  	SnortConfig = dfree(SnortConfig);
  	SnortConfig = B64SnortConfig;

  	Response = dsprintf(Response, "<TYPE>SNORTCONFIG</TYPE><SNORTCONFIG><CONFIG>%s</CONFIG></SNORTCONFIG>", SnortConfig);
  	SnortConfig = dfree(SnortConfig);
  	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturningConfig]-[%s] %s", HID,Response);
  	return(Response);
}

/******************************************************************************
*       char * UpdateSnortAlert(struct XMLLeaf * ClientXMLTree, char * HID)
*		Inserts IDS event record into MIDAS_IDS_Events. This is 
*		triggered on SNORTALERT MIDASs queries.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML Linked list cont-
*							aining the MIDASs 
*							query information.
*		char * HID				The Host ID of the 
*							MIDASs client.
*
*       Updated :
*		11/20/2003 - Sanity Check. Now only accepts Alerts from HID's
*			     that have isSniffer toggled on.
******************************************************************************/
char * UpdateSnortAlert(struct XMLLeaf * ClientXMLTree, char * HID, MYSQL * mysql)
{
	char * query = 0;
	char * Response = 0;
	char * SRCMAC = 0;
	char * DSTMAC = 0;
	char * SRCIP = 0;
	char * SRCPORT = 0;
	char * DSTIP = 0;
	char * DSTPORT = 0;
	char * PROTOCOL = 0;
	char * SEQUENCE = 0;
	char * PAYLOAD = 0;
	char * ALERTDESC = 0;
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * AlertLeaf;
	int Status = 0;
	struct DBR * dbr = 0;
		
	LocalLeaf = ClientXMLTree;

	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[UpdateSnortAlert]-[Enter]");

	if(SnortIsSniffer(HID, mysql))
	{
		while(LocalLeaf)
		{
			AlertLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "CLIENT.SNORTALERT.SNORT"), 1);
			if(AlertLeaf)
			{
				SRCMAC = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SRCMAC");
				DSTMAC = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.DSTMAC");
				SRCIP = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SRCIP");
				SRCPORT = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SRCPORT");
				DSTIP = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.DSTIP");
				DSTPORT = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.DSTPORT");
				PROTOCOL = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.PROTOCOL");
				SEQUENCE = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SEQUENCE");
				PAYLOAD = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.PAYLOAD");
				ALERTDESC = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.ALERTDESC");
			
				if(SRCMAC && DSTMAC && SRCIP && SRCPORT && DSTIP && DSTPORT && PROTOCOL && SEQUENCE)
				{
					query = dsprintf(query, "INSERT INTO MIDAS_IDS_Events (HID, SMAC, SIP, SPort, DMAC, DIP, DPort, Protocol, Descr, Payload, TimeStamp, Seq) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s' , '%s', '%s', '%s', NOW(), '%s')",
										      HID, SRCMAC, SRCIP, SRCPORT, DSTMAC, DSTIP, DSTPORT, PROTOCOL, ALERTDESC, PAYLOAD, SEQUENCE);
					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr = dbFree(dbr);
				}
				LocalLeaf = FindXMLLeafByID(LocalLeaf, AlertLeaf->ID);
				AlertLeaf = DelSubXMLTree(AlertLeaf);
		
				if(FindXMLLeafByTag(AlertLeaf, "CLIENT.SNORTALERT.SNORT.STAT"))
				{
					ProcessClientStats(AlertLeaf, "CLIENT.SNORTALERT.SNORT", HID, mysql);
				}
			}
			
			if(LocalLeaf)
			{
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
		Status = 1;
	}
	
	if(Status)
	{
		Response = dstrcpy(Response, "<TYPE>SNORTALERT</TYPE><SNORTALERT>UPDATED</SNORTALERT>");
	} else {
		Response = dstrcpy(Response, "<TYPE>SNORTALERT</TYPE><SNORTALERT>FAILED</SNORTALERT>");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[UpdateSnortAlert]-[Exit]");
	return(Response);
}

/******************************************************************************
*	int SnortIsSniffer(char * HID)
*		Returns 1 if HID has IsSniffer Toggled on, 0 if off.
*
*	Args :
*		char * HID	Host ID to check.
*
*	Updated :
*******************************************************************************/
int SnortIsSniffer(char * HID, MYSQL * mysql)
{
	char * query = 0;
	int Status = 0;
	struct DBR * dbr = 0;
		
	query = dsprintf(query, "SELECT HID FROM MIDAS_Hosts WHERE HID='%s' AND IsSniffer=1", HID);
	dbr = dbQuery_P(mysql, query);
	if(dbr)
	{
		Status = dbr->num;
	}
	dbr = dbFree(dbr);
	
	return(Status);
}

/******************************************************************************
*       char * ReturnHIDQuery(struct XMLLeaf * ClientLeaf)
*		Builds the response for HIDQUERY. This will take a host name  
*		and return the Host ID (HID) associated to it.
*
*	Args :
*		struct XMLLeaf * ClientLeaf		XML linked list cont-
*							aining the query options.
*
*       Updated :
******************************************************************************/
char * ReturnHIDQuery(struct XMLLeaf * ClientLeaf, MYSQL * mysql)
{
	char * Response = 0;
	MYSQL_ROW myrow;
	char * query = 0;
	struct DBR * dbr = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ReturnHIDQuery]-[Enter]");
	
	if(ClientLeaf)
	{
		MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ReturnHIDQuery]-[Host:%s]", 
							ReturnXMLLeafValue(ClientLeaf, "CLIENT.HOSTNAME"));
							
		Response = dstrcpy(Response, "<TYPE>HIDQUERY</TYPE><HIDQUERY><HID>");
		
		query = dsprintf(query, "SELECT HID FROM MIDAS_Hosts WHERE HostName='%s'",
					ReturnXMLLeafValue(ClientLeaf, "CLIENT.HOSTNAME"));
		dbr = dbQuery_P(mysql, query);
		query = dfree(query);
		
		if(dbr)
		{
			myrow = dbResult(dbr);
			Response = dstrcat(Response, myrow[0]);
		}
		dbr = dbFree(dbr);
		
		Response = dstrcat(Response, "</HID></HIDQUERY>");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ReturnHIDQuery]-[Exit]");
	
	return(Response);
}

/******************************************************************************
*       void ProcessClientStats(struct XMLLeaf * ClientLeaf, char * SegmentName, char * HID)
*		Process the client STAT responses. Update RRD_* tables with
*		statistical information provided by the various client apps.
*
*	Args :
*		struct XMLLeaf * ClientLeaf	XML linked list containing the 
*						query options.
*		char * SegmentName		The unique Tag segment used to 
*						build the correct tag names.
*		char * HID			The Host ID of the client 
*						performing the request.
*
*       Updated :
******************************************************************************/
void ProcessClientStats(struct XMLLeaf * ClientLeaf, char * SegmentName, char * HID, MYSQL * mysql)
{
	struct XMLLeaf * LocalLeaf = 0;
	char * XMLStat = 0;
	char * XMLType = 0;
	char * query = 0;
	char * XMLID = 0;
	char * XMLDataVal = 0;
	char * XMLOrder = 0;
	char * XMLLabel = 0;
	char * XMLFormat = 0;
	int NewDSNeeded = 0;
	unsigned TimeStamp = 0;
	unsigned BaseTS = 0;
	struct RRDCreateDS * CreateDS = 0;
	struct RRDCreateArch * CreateArchMonthly = 0;
	struct RRDCreateArch * CreateArchYearly = 0;
	MYSQL_ROW myrow;		
	int DSID = 0;
	struct DBR * dbr = 0;
	int Found = 0;
	char * FormatValue = 0;
		
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ProcessClientStats]-[Enter]");
	
	LocalLeaf = ClientLeaf;
	if(LocalLeaf)
	{
		XMLStat = dsprintf(XMLStat, "%s.STAT", SegmentName);	
		XMLType = dsprintf(XMLType, "%s.STAT.TYPE", SegmentName);
		XMLID = dsprintf(XMLID, "%s.STAT.ID", SegmentName);
		XMLDataVal = dsprintf(XMLDataVal, "%s.STAT.DATAVAL", SegmentName);
		XMLOrder = dsprintf(XMLOrder, "%s.STAT.ORDER", SegmentName);
		XMLLabel = dsprintf(XMLLabel, "%s.STAT.LABEL", SegmentName);
		XMLFormat = dsprintf(XMLFormat, "%s.STAT.FORMAT", SegmentName);
		
		if(ReturnXMLLeafValue(LocalLeaf, XMLFormat))
		{
			FormatValue = dstrcpy(FormatValue,  ReturnXMLLeafValue(LocalLeaf, XMLFormat));
		} else {
			FormatValue = dstrcpy(FormatValue, "NONE");
		}
		
		while(LocalLeaf)
		{
			NewDSNeeded = 0;
			
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, XMLStat);
			if(LocalLeaf)
			{
				query = dsprintf(query, "SELECT DSID FROM RRD_MIDAS_ID WHERE Type='%s' AND ID1='%s' AND ID2='%s' AND ID3='%s'",
							ReturnXMLLeafValue(LocalLeaf, XMLType),
							HID,
							ReturnXMLLeafValue(LocalLeaf, XMLID),
							ReturnXMLLeafValue(LocalLeaf, XMLOrder));
				dbr = dbQuery_P(mysql, query);
				
				query = dfree(query);
				
				if(dbr)
				{
					if(dbr->num)
					{
						myrow = dbResult(dbr);
						DSID = Myatoll(myrow[0]);
					} else {
						NewDSNeeded = 1;
					}
				} else {
					NewDSNeeded = 1;
				}
				dbr = dbFree(dbr);
				
				if(NewDSNeeded)
				{
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ProcessClientStats]-[Creating New DataSource]");
					
					CreateDS = (struct RRDCreateDS *) drealloc((char *) CreateDS, sizeof(struct RRDCreateDS));
					CreateArchMonthly = (struct RRDCreateArch *) drealloc((char *) CreateArchMonthly, sizeof(struct RRDCreateArch));
					CreateArchYearly = (struct RRDCreateArch *) drealloc((char *) CreateArchYearly, sizeof(struct RRDCreateArch));
					
					CreateDS->Title = dstrcpy(CreateDS->Title, ReturnXMLLeafValue(LocalLeaf, XMLLabel));
					CreateDS->Format = dstrcpy(CreateDS->Format, FormatValue);
					
					CreateDS->Sequences = 288;
					CreateDS->Interval = 300;
					CreateDS->Archive = CreateArchMonthly;
				
					CreateArchMonthly->Sequences = 122;
					CreateArchMonthly->Interval = 21600;
					CreateArchMonthly->NextArch = CreateArchYearly;
				
					CreateArchYearly->Sequences = 365;
					CreateArchYearly->Interval = 86400;
					
					DSID = RRD_CreateDataSource(CreateDS, mysql);
					
					CreateDS->Title = dfree(CreateDS->Title);
					CreateDS->Format = dfree(CreateDS->Format);
					CreateDS = (struct RRDCreateDS *) dfree((char *) CreateDS);
					CreateArchMonthly = (struct RRDCreateArch *) dfree((char *) CreateArchMonthly);
					CreateArchYearly = (struct RRDCreateArch *) dfree((char *) CreateArchYearly);

					query = dsprintf(query, "INSERT INTO RRD_MIDAS_ID (DSID, Type, ID1, ID2, ID3) VALUES ('%d', '%s', '%s', '%s', '%s')",
								DSID, 
								ReturnXMLLeafValue(LocalLeaf, XMLType),
								HID,
								ReturnXMLLeafValue(LocalLeaf, XMLID),
								ReturnXMLLeafValue(LocalLeaf, XMLOrder));
					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr = dbFree(dbr);
					
					NewDSNeeded = 0;
				}
				
				if(DSID)
				{
					TimeStamp = time(NULL);
					BaseTS = (time(NULL) / 300) * 300;
				
					query = dsprintf(query, "SELECT HID, DSID, TimeStamp FROM RRD_Values_Queue WHERE TimeStamp >= %u AND TimeStamp <= %u AND HID='%s' AND DSID='%d' ", BaseTS, (BaseTS + 300), HID, DSID);
					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
				
					Found = 0;
					if(dbr)
					{
						if(dbr->num)
						{
							Found = 1;
						}
					}
					dbr = dbFree(dbr);
					
					if(Found)
					{
						query = dsprintf(query, "UPDATE RRD_Values_Queue SET Value=(Value + %f) / 2, TimeStamp=TimeStamp WHERE DSID='%d' AND HID='%s' AND TimeStamp >= %u AND TimeStamp <= %u", 
									 Myatof(ReturnXMLLeafValue(LocalLeaf, XMLDataVal)), 
									 DSID, 
									 ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"), 
									 BaseTS, 
									 (BaseTS + 300));
						dbr = dbQuery_P(mysql, query);
						query = dfree(query);
						dbr = dbFree(dbr);
					} else {
						query = dsprintf(query, "INSERT INTO RRD_Values_Queue (DSID, HID, TimeStamp, Value) VALUES ('%d', '%s', '%u', '%f')", 
									DSID, 
									ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"), 
									BaseTS, 
									Myatof(ReturnXMLLeafValue(LocalLeaf, XMLDataVal)));
						dbr = dbQuery_P(mysql, query);
						query = dfree(query);
						dbr = dbFree(dbr);
					}
					
					if(ReturnXMLLeafValue(LocalLeaf, XMLLabel))
					{
						query = dsprintf(query, "UPDATE RRD_DataSource SET Title='%s' WHERE DSID='%d'", 
									ReturnXMLLeafValue(LocalLeaf, XMLLabel), DSID);
						dbr = dbQuery_P(mysql, query);
						query = dfree(query);
						dbr = dbFree(dbr);
					}
					
					if(ReturnXMLLeafValue(LocalLeaf, XMLFormat))
					{
						query = dsprintf(query, "UPDATE RRD_DataSource SET Format='%s' WHERE DSID='%d'",
                                                			ReturnXMLLeafValue(LocalLeaf, XMLFormat), DSID);
					} else {
						 query = dsprintf(query, "UPDATE RRD_DataSource SET Format='NONE' WHERE DSID='%d'", DSID);
					}

					dbr = dbQuery_P(mysql, query);
					query = dfree(query);
					dbr = dbFree(dbr);
				}			
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
		XMLStat = dfree(XMLStat);
		XMLType = dfree(XMLType);
		XMLID = dfree(XMLID);
		XMLDataVal = dfree(XMLDataVal);
		XMLOrder = dfree(XMLOrder);
		XMLLabel = dfree(XMLLabel);
		XMLFormat = dfree(XMLFormat);
		
		FormatValue = dfree(FormatValue);
		
	}
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ProcessClientStats]-[Exit]");
}

/******************************************************************************
*       char * UpdateNetStats(struct XMLLeaf * LocalLeaf, char * HID)
*		Simple front end to ProcessClientStats for MIDASn.
*
*	Args :
*		struct XMLLeaf * LocalLeaf	XML linked list containing the 
*						query options.
*		char * HID			The Host ID of the MIDASn client.
*
*       Updated :
******************************************************************************/
char * UpdateNetStats(struct XMLLeaf * LocalLeaf, char * HID, MYSQL * mysql)
{
	char * Response = 0;
	
	ProcessClientStats(LocalLeaf, "CLIENT.NETSTAT", HID, mysql);
	
	Response = dstrcpy(Response, "<NETSTAT>COMPLETE</NETSTAT>");
	
	return(Response);
}

/******************************************************************************
*       char * ReturnNetMon(char * HID)
*		Builds the list of Ports for MIDASn to monitor. 
*
*	Args :
*		char * HID	The Host ID of the MIDASn client.
*
*       Updated :
******************************************************************************/
char * ReturnNetMon(char * HID, MYSQL * mysql)
{
	char * Response = 0;
	MYSQL_ROW myrow;
	char * query = 0;
	struct DBR * dbr = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnNetMon]-[Enter]");
	
	if(HID)
	{
		Response = dstrcpy(Response, "<TYPE>NETMON</TYPE><NETMON>");
		
		query = dsprintf(query, "SELECT Options FROM MIDAS_NetMon_Ports WHERE HID='%s'", HID);
		dbr = dbQuery_P(mysql, query);
		
		if(dbr->num)
		{
			myrow = dbResult(dbr);
			Response = dstrcat(Response, myrow[0]);
		}
		
		dbr = dbFree(dbr);
		
		Response = dstrcat(Response, "</NETMON>");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnNetMon]-[Exit]");
	
	return(Response);
}

/******************************************************************************
*	char * UpdateMACIP(struct XMLLeaf * CheckLeaf, char * HID, MYSQL * mysql)
*		Updates the MAP->IP lists stored in the SQL DB.
*
*	Args :
*		struct XMLLeaf * CheckLeaf	XML Leaf containing update info
*		char * HID			HID of client send update
*		MYSQL * mysql			Mysql handle
*
*       Updated :
******************************************************************************/
char * UpdateMACIP(struct XMLLeaf * CheckLeaf, char * HID, MYSQL * mysql)
{
	char * query = 0;
	char * Out = 0;
	char * MAC = 0;
	char * IP = 0;
	struct DBR * dbr = 0;
	struct XMLLeaf * LocalLeaf = 0;
	int Found = 0;
			
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateMACIP]-[Enter]");
	
	LocalLeaf = FindXMLLeafByTag(CheckLeaf, "CLIENT.MACADDRESSES.MACADDRESS");
	while(LocalLeaf)
	{
		if(LocalLeaf)
		{
			MAC = ReturnXMLLeafValue(LocalLeaf, "CLIENT.MACADDRESSES.MACADDRESS.MAC");
			IP = ReturnXMLLeafValue(LocalLeaf, "CLIENT.MACADDRESSES.MACADDRESS.IP");
			
			query = dsprintf(query, "SELECT HID FROM MIDAS_MACIP WHERE MAC='%s' AND IP='%s' AND HID='%s'", MAC, IP, HID);
			dbr = dbQuery_P(mysql, query);
			query = dfree(query);
			if(dbr)
			{
				if(dbr->num)
				{
					Found = 1;	
				}
			}
			dbr = dbFree(dbr);
			
			if(Found)
			{
				query = dsprintf(query, "UPDATE MIDAS_MACIP SET LastSeen=NOW(), FoundOn=FoundOn WHERE HID='%s' AND MAC='%s' AND IP='%s'", HID, MAC, IP);
			} else {
				query = dsprintf(query, "INSERT INTO MIDAS_MACIP (HID, MAC, IP, FoundOn, LastSeen) VALUES ('%s', '%s', '%s', NOW(), NOW())", HID, MAC, IP);
			}

			dbr = dbQuery_P(mysql, query);
			dbr = dbFree(dbr);
			
			LocalLeaf = LocalLeaf->NextLeaf;
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CLIENT.MACADDRESSES.MACADDRESS");
		}
	}
	
	Out = dstrcpy(Out, "<TYPE>MACUPDATE</TYPE><MACUPDATE>COMPLETE</MACUPDATE>");
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateMACIP]-[Exit]");
	
	return(Out);
}

/******************************************************************************
*	char * UpdateNetworkSessions(struct XMLLeaf * CheckLeaf, char * HID, MYSQL * mysql)
*		Updates/Inserts recorded network sessions.
*
*	Args :
*		struct XMLLeaf * CheckLeaf	XML Leaf containing update info
*		char * HID			HID of client send update
*		MYSQL * mysql			Mysql handle
*
*       Updated :
******************************************************************************/
char * UpdateNetworkSessions(struct XMLLeaf * CheckLeaf, char * HID, MYSQL * mysql)
{
	char * query = 0;
	char * Out = 0;
	char * SrcIP = 0;
	char * DstIP = 0;
	char * DstPort = 0;
	char * Proto = 0;
	char * Count = 0;
	char * Bytes = 0;
	char * SrcPort = 0;
	unsigned StartTime = 0;
	unsigned EndTime = 0;
	char * tStartTime = 0;
	char * tEndTime = 0;
	char * tTTLTime = 0;
	struct DBR * dbr = 0;
	struct XMLLeaf * LocalLeaf = 0;
	int Found = 0;
			
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateNetworkSessions]-[Enter]");
	
	LocalLeaf = FindXMLLeafByTag(CheckLeaf, "CLIENT.NETSESSIONS.NETSESSION");
	while(LocalLeaf)
	{
		if(LocalLeaf)
		{
			SrcIP = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.SRCIP");
			SrcPort = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.SRCPORT");
			DstIP = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.DSTIP");
			DstPort = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.DSTPORT");
			Proto = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.PROTO");
			Count = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.COUNT");
			Bytes = ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.BYTES");
			StartTime = Myatoll(ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.STARTTIME"));
			EndTime = Myatoll(ReturnXMLLeafValue(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION.ENDTIME"));
			
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateNetworkSessions]-[SrcIP:%s]-[DstIP:%s]-[DstPort:%s]-[Proto:%s]-[Count:%s]-[Bytes:%s]-[Start:%s]-[End:%s]",
								SrcIP, DstIP, DstPort, Proto, Count, Bytes, StartTime, EndTime);
			
			tTTLTime = UnixTime2SQLTime(StartTime - 300);
			query = dsprintf(query, "SELECT HID FROM MIDAS_NetMon_Sessions WHERE SrcIP='%s' AND SrcPort='%s' AND DstIP='%s' AND DstPort='%s' AND Protocol='%s' AND HID='%s' AND EndTime >= '%s'", 
						SrcIP, SrcPort, DstIP, DstPort, Proto, HID, tTTLTime);
			tTTLTime = dfree(tTTLTime);
			dbr = dbQuery_P(mysql, query);
			query = dfree(query);
			
			if(dbr)
			{
				if(dbr->num)
				{
					Found = 1;	
				}
			}
			dbr = dbFree(dbr);
			
			tStartTime = UnixTime2SQLTime(StartTime);
			tEndTime = UnixTime2SQLTime(EndTime);
			
			if(Found)
			{
				query = dsprintf(query, "UPDATE MIDAS_NetMon_Sessions SET PacketCount=PacketCount+'%s', Bytes=Bytes+'%s', StartTime=StartTime, EndTime='%s', UpdateCount=UpdateCount+1 WHERE SrcIP='%s' AND SrcPort='%s' AND DstIP='%s' AND DstPort='%s' AND Protocol='%s' AND HID='%s'",
							Count, Bytes, tEndTime, SrcIP, SrcPort, DstIP, DstPort, Proto, HID);
			} else {
				query = dsprintf(query, "INSERT INTO MIDAS_NetMon_Sessions (HID, SrcIP, SrcPort, DstIP, DstPort, Protocol, Bytes, PacketCount, StartTime, EndTime, UpdateCount) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', 1)",
							HID, SrcIP, SrcPort, DstIP, DstPort, Proto, Bytes, Count, tStartTime, tEndTime);
			}
			
			tStartTime = dfree(tStartTime);
			tEndTime = dfree(tEndTime);
			
			dbr = dbQuery_P(mysql, query);
			dbr = dbFree(dbr);
			
			LocalLeaf = LocalLeaf->NextLeaf;
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CLIENT.NETSESSIONS.NETSESSION");
		}
	}
	
	Out = dstrcpy(Out, "<TYPE>NETSESSIONUPDATE</TYPE><NETSESSIONUPDATE>COMPLETE</NETSESSIONUPDATE>");
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateNetworkSessions]-[Exit]");
	
	return(Out);
}
