/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : server.c
* 		Function required for the 'Server' portions of MIDASd
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <mysql/mysql.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

#ifdef __WIN32__
// [TCC] Windows produces a mem leak if close() and gethostbyname() is used.
//       Requires the use of closesocket() instead.
#define close(x) closesocket(x)
#endif

/******************************************************************************
*       int StartServer(void)
*		Starts up the server and waits for client connects.
*
*
*       Updated :
******************************************************************************/
int StartServer(void)
{
	int sockfd, clientsockfd, len;
	struct sockaddr_in serv_addr, cli_addr;
	char *tCIP = 0;
	int FreeChildrenCount;
	unsigned CHILDREN[255];
	unsigned BIRTH[255];
	int on = 1;
	pid_t child;
	char * SendBuff = 0;
	struct XMLLeaf * ClientXMLRequest;
	char * ClientString = 0;
	char * ClientResponse = 0;
	struct hostent * hostinfo = 0;
	      
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Starting]");
	
	bzero(&CHILDREN, sizeof(CHILDREN));
	bzero(&BIRTH, sizeof(BIRTH));
	
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof on);
	
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	
	if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"))
	{
		hostinfo = gethostbyname(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"));
		serv_addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
	} else {
		serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	}
	
	serv_addr.sin_port = htons(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.Port")));
		
	if(bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr)) == -1)
	{
		MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SockServer]-[Unable to open Port for listening. Exiting.]");
		close(sockfd);
		return(-1);
	}
	
	if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"))
	{
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Interface %s:%s bound]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.BINDIP"), ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PORT"));
	} else {
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Port %s bound]",  ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PORT"));
	}
	
	listen(sockfd, 250);
	len = sizeof(cli_addr);
	for(;;)
	{
		clientsockfd = accept(sockfd,(struct sockaddr*)&cli_addr,&len);
		if(clientsockfd > 0)
		{
			tCIP = ConvertL2IP(cli_addr.sin_addr.s_addr);
			if(CheckIPAccess(cli_addr.sin_addr.s_addr))
			{
				MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[Client connected]-[%s]", tCIP);
				do
				{
					FreeChildrenCount = FreeChild(CHILDREN, BIRTH);
					if(FreeChildrenCount != -1)
					{
						child = fork();
						if(child == 0)
						{
							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Forked]");
							
							SendBuff = dsprintf(SendBuff, "MIDASd v%s connection accepted\n", MIDASVER);
							SendToSocket(clientsockfd, SendBuff, MIDAS_EOL, 0);
							SendBuff = dfree(SendBuff);
							
							ClientString = RecvFromSocket(clientsockfd, MIDAS_EOL, 1);
							if(ClientString)
							{
								MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Sent %s]", ClientString);
								
								ClientXMLRequest = StringToXMLTree(ClientString);
								ClientString = dfree(ClientString);

								if(ClientXMLRequest)
								{
									ClientResponse = ProcessClientRequest_InSession(ClientXMLRequest);
									if(ClientResponse)
									{
										SendToSocket(clientsockfd, ClientResponse, MIDAS_EOL, 1);
									}
									ClientResponse = dfree(ClientResponse);
								} else {
									MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Invalid Client Request]");
									SendToSocket(clientsockfd, "<SERVER><TYPE>INVALID</TYPE></SERVER>", MIDAS_EOL, 1);
								}
								
								
								// Wait for the client <CLOSE>
								ClientString = RecvFromSocket(clientsockfd, MIDAS_EOL, 1);
								ClientString = dfree(ClientString);
								
								MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Closed]");
								close(sockfd);
								
								MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Processing Out of Session Requests]");
								ProcessClientRequest_OutOfSession(ClientXMLRequest);
								
								ClientXMLRequest = DelXMLLeafTree(ClientXMLRequest);
							} else {
								 MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Nothing sent by Client]");
								close(sockfd);
							}
							exit(0);
						} else {
							close(clientsockfd);
							AddChild(child, CHILDREN, BIRTH);
						}
					} else {
						usleep(100);
						continue;
					}

					
				}while(FreeChildrenCount == -1);
			} else {
				MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[IP Access denied to : %s]", tCIP);
				close(clientsockfd);
			}
			tCIP = dfree(tCIP);
		}
  	}
	
	MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[Exit]");
	
	return(0);
}

/******************************************************************************
*       int CheckIPAccess(unsigned clientip)
*		Checks the connecting IP address against the defined IP blocks 
*		to verify that its allowed to connect. Returns 1 if connection
*		allowed or 0 if not allowed.
*
*	Args :
*		unsigned clientip	The connecting (numeric) IP address.
*
*       Updated :
******************************************************************************/						        
int CheckIPAccess(unsigned ClientIP)
{
	int  status = 0;
	struct XMLLeaf * LocalLeaf;
	unsigned NumIP = 0;
	unsigned NumMask = 0;
	char * IP = 0;
	char * Mask = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[CheckIPAccess]-[Enter]");

	LocalLeaf = ConfigXMLTree;
	while(LocalLeaf)
	{
		LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.ACCESSIP.IP");
		if(LocalLeaf)
		{
			IP = ReturnXMLLeafValue(LocalLeaf, "CONFIG.ACCESSIP.IP");
			Mask = ReturnXMLLeafValue(LocalLeaf, "CONFIG.ACCESSIP.IP.MASK");

			NumIP = ConvertIP2L(IP);
			NumMask = FixIP(MyPow(2, Myatoi(Mask)) - 1);
			
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[CheckIPAccess]-[IP:%s]-[MASK:%s]", IP, Mask);

			if(IP && Mask)
			{
				if((ClientIP & NumMask) == NumIP)
				{
					status = 1;
					break;
				}
			}
			
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}
	return(status);	
}

/******************************************************************************
*	unsigned FixIP(unsigned Decimal)
*	Plays musical bits on an unsigned int. Will swap bits order 
*	within each 4 byte block. This is required when appling mask bits 
*	to it.	
*
*	Args :
*		unsigned IP	Numeric IP value.
*
*       Updated :
******************************************************************************/						        
unsigned FixIP(unsigned IP)
{
	unsigned OutIP = 0;
	int i = 0;
	int ii = 0;
	
	for(i=0;i<32;i=i+8)
	{
		for(ii=0;ii<8;ii++)
		{
			if((IP & MyPow(2, i+ii)) == MyPow(2, i+ii))
			{
				OutIP = OutIP + MyPow(2, (i + (7 - ii)));
			}
		}
	}

	return(OutIP);
}
