/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASn
*       Filename : MIDASn.c
* 		Funstions for main portion of MIDASn application
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*		12/31/2003 - Added SOCK_HANDLE to store socket handle as a 
*			     global.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>

#ifdef __sunos__
#include <sys/sockio.h>
#endif

#include <sys/wait.h>
#include <fcntl.h>
#include <sys/time.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>

#include "../Common/Commoninclude.h"
#include "includes.h"

int DEBUGLEVEL 	= 0;
int LOGTOSTDERR = 0;
int LOG_INFO 	= 1;
int LOG_SOCKET 	= 2;
int LOG_XML	= 4;
int LOG_CHECK   = 8;
int LOG_FORK    = 16;

struct XMLLeaf * ConfigXMLTree = 0;
char APPNAME[] = "MIDASn";
char * CFGPATH = 0;
char * KEYFILE = 0;
int SENDDELAY = 0;
pcap_t * PCAP_HANDLE = 0;

extern char MIDAS_EOL[];

/******************************************************************************
*       int main(int argc, char *argv[])
*		The main() function for MIDASn
*
*	Args :
*		int argc	Argument count.
*		char * argv[]	Argument value array.
*
*       Updated : 
******************************************************************************/
int main(int argc, char *argv[])
{
	(void) signal(SIGHUP, CatchSignal);
	(void) signal(SIGTERM, CatchSignal);

	CFGPATH = dsprintf(CFGPATH, "%s/etc/MIDASn.cf", APPROOT);		
	ConfigXMLTree = FileToXMLTree(CFGPATH);
	
	if(!ConfigXMLTree)
	{
		printf("! Unable to open %s\n", CFGPATH);
		exit(0);
	}
	
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));
	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	SENDDELAY = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SENDINTERVAL"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASn %s starting]", MIDASVER);
	Sniffer();
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASn %s exiting]", MIDASVER);
	
	DelXMLLeafTree(ConfigXMLTree);
	
	exit(0);
}

/******************************************************************************
*       pcap_t initdevice(void)
*		Builds socket and puts specified ethernet device into 
*		promiscuous mode.
*
*	Args :
*		char * device		Device to modify
*
*       Updated :
*		12/31/03 - Changed from SOCK_PACKET to SOCK_RAW
*		12/31/03 - Only sets prom. mode if not already set.
*		01/03/03 - Moved to using libpcap.
******************************************************************************/
pcap_t * initdevice(void)
{
	pcap_t * pcap = 0;
	char * Device = 0;
	int Promisc = 0;
	char error[PCAP_ERRBUF_SIZE];
	
	Device = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.ETHDEV");
	Promisc = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PROMISC"));

	if(Device)
	{
		pcap = pcap_open_live(Device, sizeof(struct etherpacket), Promisc, 0, error);
	}
	
	return(pcap);
}

/******************************************************************************
*       int deinitdevice(pcap_t * pcap)
*		Turns promiscuous mode off on socket
*
*	Args :
*		int if_fd	Socket handle.
*
*       Updated :
******************************************************************************/
void deinitdevice(pcap_t * pcap)
{
	struct passwd * pw;
	
	if(pcap)
	{
		pw = getpwnam("root");
		
		seteuid(pw->pw_uid);
		setegid(pw->pw_gid);
		
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[deinitdevice]-[Switching back to root for deinit [UID:%d]", geteuid());
	
		pcap_close(pcap);
	}
}

/******************************************************************************
*	void Sniffer(struct XMLLeaf * PortLeaf)       
*		Packet sniffer function. This grabs the packets and builds stats 
*
*       Updated :
******************************************************************************/
void Sniffer(void)
{
	pcap_t * pcap;
	struct pcap_pkthdr pkthdr;
	
	struct etherpacket * ep;
	struct NetStat * NetworkStat = 0;
	unsigned long LastStatSend = 0;
	
	struct passwd * pw;
	struct passwd * pw2;
	struct XMLLeaf * PortLeaf = 0;
	
	
	NetworkStat = (struct NetStat *) drealloc((char *)NetworkStat, sizeof(struct NetStat));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Enter]-[Dev:%s]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.ETHDEV"));
	
	pcap = initdevice();
	if(pcap)
	{
		PCAP_HANDLE = pcap;
		
		if(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.USER"))
		{
			pw = getpwnam(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.USER"));
			if(pw)
			{
				seteuid(pw->pw_uid);
				setegid(pw->pw_gid);
			
				pw2 = getpwuid(geteuid());
					
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Switching to user : %s UID:%d]", pw2->pw_name, pw2->pw_uid);
			}
		}
		

		PortLeaf = QueryNetmon(PortLeaf);
		NetworkStat = UpdateStats_InitPorts(NetworkStat, PortLeaf);
		
		LastStatSend = time(NULL);
		
		for(;;)
		{
			ep = (struct etherpacket *) pcap_next(pcap, &pkthdr);
			
			// [TCC] Right now we only support IP. Should add support for others (Ie IPX, Appletalk, etc) 
			// in the future.
			if(ep->eth.h_proto == ntohs(ETH_P_IP))
			{
				NetworkStat = UpdateStats(ep, NetworkStat);
				NetworkStat = UpdateStats_Ports(ep, NetworkStat);
				NetworkStat = UpdateStats_Mac(ep, NetworkStat);
				NetworkStat = UpdateStats_Session(ep, NetworkStat);
			}
				
			if(time(NULL) >= LastStatSend + SENDDELAY)
			{
				SendStats(NetworkStat);

				// Reset the current stats
				NetworkStat = UpdateStats_PurgePortStats(NetworkStat);
				NetworkStat = UpdateStats_PurgeMac(NetworkStat);
				NetworkStat = UpdateStats_PurgeSession(NetworkStat);
					
				memset(NetworkStat, 0, sizeof(struct NetStat));
				LastStatSend = time(NULL);
				
				// Re query for the latest Netmon rules from MIDASd
				PortLeaf = DelXMLLeafTree(PortLeaf);
				PortLeaf = QueryNetmon(PortLeaf);
				
				// Init the Port Stats and Free PortLeaf XML Tree.
				NetworkStat = UpdateStats_InitPorts(NetworkStat, PortLeaf);
				PortLeaf = DelXMLLeafTree(PortLeaf);
			}
		}

	} else {
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Unable to open %s]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.ETHDEV"));
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Exit]");
	
}

/******************************************************************************
*       int ParseCommandLine(int argc, char *argv[])
*		Parses the command line arguments.
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
	pcap_t * pcap = 0;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "VhD:TPL");
		switch(opt)
		{
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D#\tSet the Debug level\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-T\tShow configuration\n");
				printf("\t-V\tShow version\n");
				printf("\t-P\tDe-Promisouos network interface\n");
				printf("\t-L\tSend Debug to STDERR\n");
				
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
			case 'P':
				pcap = initdevice();
				if(pcap)
				{
					deinitdevice(pcap);
				}
				Status = 0;
				break;
			case 'L':
				LOGTOSTDERR = 1;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Shows current configurations. This is triggered with the -T
*		command line arg.
*
*
*       Updated :
******************************************************************************/
void ShowConfig(void)
{
	struct XMLLeaf * PortLeaf = 0;
	
        printf("%s %s\n", APPNAME, MIDASVER);
        printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME);
        DebugXMLTree("MIDASn.cf", ConfigXMLTree, 0);
	
	printf("------------------------------------------------------------------\n");
	printf("[Port monitors configured]\n");
	PortLeaf = QueryNetmon(PortLeaf);
	DebugXMLTree("PortLeaf", PortLeaf, 0);
}

/******************************************************************************
*       void CatchSignal(int sig)
*               The signal() catch function. Handles, TERM, KILL, CHLD and HUP
*
*       Args :
*               int sig         Signal that caused trigger.
*
*       Updated :
******************************************************************************/
void CatchSignal(int sig)
{
        switch(sig)
        {
                case SIGHUP:
                        MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchSignal]-[Caught HUP Reloading config file : %s]", CFGPATH);
                        ConfigXMLTree = FileToXMLTree(CFGPATH);
                        break;
                case SIGTERM:
                        MyDebug(__LINE__, __FILE__,  LOG_INFO, "[CatchHUP]-[Caught SIGTERM exiting]");
                        deinitdevice(PCAP_HANDLE);
                        exit(0);
        }
}

/******************************************************************************
*	struct XMLTree * QueryNetmon()
*		Queries MIDASd server and returns to netmon rules
*
*
*       Updated :
******************************************************************************/
struct XMLLeaf * QueryNetmon(struct XMLLeaf * PortLeaf)
{
	char * Query = 0;
	char * Response = 0;

	Query = dsprintf(Query, "<CLIENT><HID>%s</HID><TYPE>RETURNNETMON</TYPE></CLIENT>", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
	Response = SendToServer(Query);
	Query = dfree(Query);

	if(Response)
	{
		PortLeaf = StringToXMLTree(Response);
	}
	
	Response = dfree(Response);
		
	return(PortLeaf);
}
