#define u_int8_t unsigned char
#define u_int16_t unsigned short
#define u_int32_t unsigned int

#define ETH_ALEN        6               /* Octets in one ethernet addr   */
#define ETH_P_IP        0x0800          /* Internet Protocol packet     */
#define ETH_P_ALL       0x0003		/* All protocols */

/**************************************************************************************************
*
*	Ethernet packet header
*
**************************************************************************************************/
struct ethhdr
{
	unsigned char   h_dest[ETH_ALEN];       /* destination eth addr */
	unsigned char   h_source[ETH_ALEN];     /* source ether addr    */
	unsigned short  h_proto;                /* packet type ID field */
};

/**************************************************************************************************
*
*	IP packet header
*
**************************************************************************************************/
struct iphdr
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	unsigned int ihl:4;
	unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
	unsigned int version:4;
	unsigned int ihl:4;
#else
# error "Please fix <bits/endian.h>"
#endif
	u_int8_t tos;
	u_int16_t tot_len;
	u_int16_t id;
	u_int16_t frag_off;
	u_int8_t ttl;
	u_int8_t protocol;
	u_int16_t check;
	u_int32_t saddr;
	u_int32_t daddr;
	/*The options start here. */
};

/**************************************************************************************************
*
*	tcp packet header
*
**************************************************************************************************/
struct tcphdr
{
	u_int16_t source;
	u_int16_t dest;
	u_int32_t seq;
	u_int32_t ack_seq;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int16_t res1:4;
	u_int16_t doff:4;
	u_int16_t fin:1;
	u_int16_t syn:1;
	u_int16_t rst:1;
	u_int16_t psh:1;
	u_int16_t ack:1;
	u_int16_t urg:1;
	u_int16_t res2:2;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t doff:4;
	u_int16_t res1:4;
	u_int16_t res2:2;
	u_int16_t urg:1;
	u_int16_t ack:1;
	u_int16_t psh:1;
	u_int16_t rst:1;
	u_int16_t syn:1;
	u_int16_t fin:1;
#else
#   error "Adjust your <bits/endian.h> defines"
#endif
	u_int16_t window;
	u_int16_t check;
	u_int16_t urg_ptr;
};

/**************************************************************************************************
*
*	UDP packet header
*
**************************************************************************************************/
struct udphdr 
{
	u_int16_t     source;
	u_int16_t     dest;
	u_int16_t     len;
	u_int16_t     check;
};

/**************************************************************************************************
*
*	ICMP packet header
*
**************************************************************************************************/
struct icmphdr
{
	u_int8_t type;                /* message type */
	u_int8_t code;                /* type sub-code */
	u_int16_t checksum;
	union
	{
		struct
		{
			u_int16_t id;
			u_int16_t sequence;
		} echo;                     /* echo datagram */
		u_int32_t   gateway;        /* gateway address */
		struct
		{
			u_int16_t unused;
			u_int16_t mtu;
		} frag;                     /* path mtu discovery */
	} un;
};
