/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASn
*       Filename : stats_mac.c
*		Function used to gather MAC/IP statistics.
*
*       Updated :
*               01/04/2004 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

struct NetStat * UpdateStats_Mac(struct etherpacket * ep, struct NetStat * NetworkStats)
{
	char * SMAC = 0;
	char * DMAC = 0;
	char * SIP = 0;
	char * DIP = 0;
	struct iphdr * ip = 0;
	
	if(ep)
	{
		SMAC = BuildMac(ep->eth.h_source);
		DMAC = BuildMac(ep->eth.h_dest);
		
		if(SMAC && DMAC)
		{
			ip = (struct iphdr *) (((unsigned long) &ep->ip) - 2);
		
			if(ip)
			{
			
				SIP = BuildIP(ip->saddr);
				DIP = BuildIP(ip->daddr);

				if(SIP && DIP)
				{
					NetworkStats = UpdateMacs(NetworkStats, SMAC, SIP);
					NetworkStats = UpdateMacs(NetworkStats, DMAC, DIP);
				}
				
				SIP = dfree(SIP);
				DIP = dfree(DIP);
			}
		}
		
		SMAC = dfree(SMAC);
		DMAC = dfree(DMAC);
	}
	
	return(NetworkStats);	
}


struct NetStat * UpdateMacs(struct NetStat * NetworkStats, char * MAC, char * IP)
{
	int Found = 0;
	struct MacStat * LocalMac = 0;
	struct MacStat * NewMac = 0;
	
	if(IP && MAC)
	{
		if(NetworkStats)
		{
			LocalMac = NetworkStats->MacStats;
			while(LocalMac)
			{
				if(!dstrcmp(LocalMac->Mac, MAC))
				{
					LocalMac->IP = dfree(LocalMac->IP);
					LocalMac->IP = dstrcpy(LocalMac->IP, IP);
					
					Found = 1;
					break;
				}
				LocalMac = LocalMac->NextMac;
			}
		}
	
		if(!Found)
		{
			NewMac = (struct MacStat *) drealloc((char *) NewMac, sizeof(struct MacStat));
			NewMac->IP = dstrcpy(NewMac->IP, IP);
			NewMac->Mac = dstrcpy(NewMac->Mac, MAC);
			
			LocalMac = NetworkStats->MacStats;
			if(LocalMac)
			{
				while(LocalMac->NextMac)
				{
					LocalMac = LocalMac->NextMac;
				}
				
				LocalMac->NextMac = NewMac;
			} else {
				NetworkStats->MacStats = NewMac;
			}
		}
	}

	return(NetworkStats);
}

char * UpdateStats_BuildMacQuery(struct NetStat * NetworkStats)
{
	struct MacStat * LocalMac = 0;
	char * Query = 0;
	char * tQuery = 0;
	
	if(NetworkStats)
	{
		Query = dstrcpy(Query, "<TYPE>MACADDRESSES</TYPE><MACADDRESSES>");
		
		LocalMac = NetworkStats->MacStats;
		while(LocalMac)
		{
			tQuery = dsprintf(tQuery, "<MACADDRESS><MAC>%s</MAC><IP>%s</IP></MACADDRESS>", LocalMac->Mac, LocalMac->IP);
			Query = dstrcat(Query, tQuery);
			tQuery = dfree(tQuery);
			
			LocalMac = LocalMac->NextMac;
		}
		Query = dstrcat(Query, "</MACADDRESSES>");
	}

	return(Query);
}

struct NetStat * UpdateStats_PurgeMac(struct NetStat * NetworkStats)
{
	struct MacStat * LocalMac = 0;
	struct MacStat * NextMac = 0;
	unsigned Count = 0;
		
	MyDebug(__LINE__,__FILE__, LOG_INFO, "[UpdateStats_PurgeMac]-[Enter]");
	
	if(NetworkStats)
	{
		LocalMac = NetworkStats->MacStats;
		if(LocalMac)
		{
			while(LocalMac->NextMac)
			{
				Count++;
				NextMac = LocalMac->NextMac;
				
				LocalMac->IP = dfree(LocalMac->IP);
				LocalMac->Mac = dfree(LocalMac->Mac);
				LocalMac = (struct MacStat *) dfree((char *) LocalMac);
			
				LocalMac = NextMac;
			}
		
			if(LocalMac)
			{
				LocalMac->IP = dfree(LocalMac->IP);
				LocalMac->Mac = dfree(LocalMac->Mac);
				LocalMac = (struct MacStat *) dfree((char *) LocalMac);
			}
		}
	}
	
	MyDebug(__LINE__,__FILE__, LOG_INFO, "[UpdateStats_PurgeMac]-[Exit]");
	
	return(NetworkStats);
}

char * BuildMac(unsigned char * String)
{
	char * MAC = 0;
	
	if(String)
	{
		MAC = dsprintf(MAC, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
				    String[0],String[1],String[2],
				    String[3],String[4],String[5]);
	}
	return(MAC);
}

char * BuildIP(unsigned long NumericIP)
{
	struct in_addr addr;
	char * IP = 0;
	
	if(NumericIP)
	{
		addr.s_addr = NumericIP;
		IP = dstrcpy(IP, inet_ntoa(addr));
	}
	
	return(IP);
}
