/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASn
*       Filename : stats_session.c
*		Function used to gather Session statistics.
*
*       Updated :
*               01/04/2004 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

struct NetStat * UpdateStats_Session(struct etherpacket * ep, struct NetStat * NetworkStats)
{
	char * SIP = 0;
	char * DIP = 0;
	unsigned SPort = 0;
	unsigned DPort = 0;
	unsigned Len = 0;
	int Found = 0;
	struct tcphdr * tcp;
	struct udphdr * udp;
	char * Proto = 0;
	struct iphdr * ip = 0;
	
	if(ep)
	{
		ip = (struct iphdr *) (((unsigned long) &ep->ip) - 2);
		
		SIP = BuildIP(ip->saddr);
		DIP = BuildIP(ip->daddr);
		Len = htons(ip->tot_len);
		
		switch(ip->protocol)
		{
			case 6:
				Proto = dstrcpy(Proto, "TCP");
				tcp = (struct tcphdr *) (((unsigned long) &ep->tcp) - 2);
				SPort = htons(tcp->source);
				DPort = htons(tcp->dest);
				Found = 1;
				break;
			case 7:
				Proto = dstrcpy(Proto, "UDP");
				udp = (struct udphdr *) (((unsigned long) &ep->tcp) - 2);
				SPort = htons(udp->source);
				DPort = htons(udp->dest);
				Found = 1;
				break;
		}
		
		if(Found)
		{
			NetworkStats = UpdateSession(NetworkStats, SIP, DIP, Proto, SPort, DPort, Len);
		}
		
		SIP = dfree(SIP);
		DIP = dfree(DIP);
		Proto = dfree(Proto);
	}
	
	return(NetworkStats);	
}


struct NetStat * UpdateSession(struct NetStat * NetworkStats, char * SIP, char * DIP, char * Proto, unsigned SrcPort, unsigned DstPort, unsigned Len)
{
	int Found = 0;
	struct SessStat * LocalSess = 0;
	struct SessStat * NewSess = 0;
		
	if(NetworkStats)
	{
		LocalSess = NetworkStats->SessStats;
		while(LocalSess)
		{
			if(!dstrcmp(LocalSess->SrcIP, SIP) && !dstrcmp(LocalSess->DstIP, DIP) &&
			   LocalSess->SrcPort == SrcPort && LocalSess->DstPort == DstPort &&
			   !dstrcmp(LocalSess->Protocol, Proto))
			{
				LocalSess->Count++;
				LocalSess->Bytes+=Len;
				LocalSess->EndTime = (unsigned) time(NULL);
				
				Found = 1;
				break;
			}
			LocalSess = LocalSess->NextSess;
		}
	}
	
	if(!Found)
	{
		NewSess = (struct SessStat *) drealloc((char *) NewSess, sizeof(struct SessStat));
		NewSess->SrcIP = dstrcpy(NewSess->SrcIP, SIP);
		NewSess->DstIP = dstrcpy(NewSess->DstIP, DIP);
		NewSess->Protocol = dstrcpy(NewSess->Protocol, Proto);
		NewSess->SrcPort = SrcPort;
		NewSess->DstPort = DstPort;
		NewSess->Count = 1;
		NewSess->Bytes = Len;
		NewSess->StartTime = (unsigned) time(NULL);
		NewSess->EndTime = (unsigned) time(NULL);
		
		LocalSess = NetworkStats->SessStats;
		if(LocalSess)
		{
			while(LocalSess->NextSess)
			{
				LocalSess = LocalSess->NextSess;
			}
			
			LocalSess->NextSess = NewSess;
		} else {
			NetworkStats->SessStats = NewSess;
		}
	}

	return(NetworkStats);
}

char * UpdateStats_BuildSessionQuery(struct NetStat * NetworkStats)
{
	struct SessStat * LocalSess = 0;
	char * Query = 0;
	char * tQuery = 0;
	
	if(NetworkStats)
	{
		Query = dstrcpy(Query, "<TYPE>NETSESSIONS</TYPE><NETSESSIONS>");
		
		LocalSess = NetworkStats->SessStats;
		while(LocalSess)
		{
			tQuery = dsprintf(tQuery, "<NETSESSION><SRCIP>%s</SRCIP><SRCPORT>%u</SRCPORT><DSTIP>%s</DSTIP><DSTPORT>%u</DSTPORT><PROTO>%s</PROTO><COUNT>%u</COUNT><BYTES>%u</BYTES><STARTTIME>%u</STARTTIME><ENDTIME>%u</ENDTIME></NETSESSION>",
				 LocalSess->SrcIP, LocalSess->SrcPort,
				 LocalSess->DstIP, LocalSess->DstPort,
				 LocalSess->Protocol,
				 LocalSess->Count, LocalSess->Bytes,
				 LocalSess->StartTime, LocalSess->EndTime);
			Query = dstrcat(Query, tQuery);
			tQuery = dfree(tQuery);
			LocalSess = LocalSess->NextSess;
		}
		Query = dstrcat(Query, "</NETSESSIONS>");
	}

	return(Query);
}

char * UpdateStats_BuildSessionStats(struct NetStat * NetworkStats)
{
	struct SessStat * LocalSess = 0;
	char * Query = 0;
	char * tQuery = 0;
	
	if(NetworkStats)
	{
		LocalSess = NetworkStats->SessStats;
		while(LocalSess)
		{
			tQuery = dsprintf(tQuery, "<STAT><TYPE>NETSESSION</TYPE><ID>%s_%s_%u_%s</ID><ORDER>0</ORDER><LABEL>[%s] %s -> %s:%u (Count)</LABEL><DATAVAL>%u</DATAVAL></STAT>",
				 		LocalSess->SrcIP,
				 		LocalSess->DstIP, LocalSess->DstPort,
				 		LocalSess->Protocol, LocalSess->Protocol,
				 		LocalSess->SrcIP,
				 		LocalSess->DstIP, LocalSess->DstPort,
				 		LocalSess->Count);
			Query = dstrcat(Query, tQuery);
			tQuery = dfree(tQuery);
			
			tQuery = dsprintf(tQuery, "<STAT><TYPE>NETSESSION</TYPE><ID>%s_%s_%u_%s</ID><ORDER>1</ORDER><LABEL>[%s] %s -> %s:%u (Bytes)</LABEL><DATAVAL>%u</DATAVAL></STAT>",
				 		LocalSess->SrcIP,
				 		LocalSess->DstIP, LocalSess->DstPort,
				 		LocalSess->Protocol, LocalSess->Protocol,
				 		LocalSess->SrcIP,
				 		LocalSess->DstIP, LocalSess->DstPort,
				 		LocalSess->Bytes);
			Query = dstrcat(Query, tQuery);
			tQuery = dfree(tQuery);
			
			LocalSess = LocalSess->NextSess;
		}
	}

	return(Query);
}


struct NetStat * UpdateStats_PurgeSession(struct NetStat * NetworkStats)
{
	struct SessStat * LocalSess = 0;
	struct SessStat * NextSess = 0;
	
	MyDebug(__LINE__,__FILE__, LOG_INFO, "[UpdateStats_PurgeSess]-[Enter]");
	
	if(NetworkStats)
	{
		LocalSess = NetworkStats->SessStats;
		if(LocalSess)
		{
			while(LocalSess)
			{
				NextSess = LocalSess->NextSess;
				
				LocalSess->SrcIP = dfree(LocalSess->SrcIP);
				LocalSess->DstIP = dfree(LocalSess->DstIP);
				LocalSess->Protocol = dfree(LocalSess->Protocol);
							
				LocalSess = (struct SessStat *) dfree((char *) LocalSess);
				
				LocalSess = NextSess;
			}
		}
	}
	
	MyDebug(__LINE__,__FILE__, LOG_INFO, "[UpdateStats_PurgeSess]-[Exit]");
	
	return(NetworkStats);
}
