/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASs
*       Filename : snort.c
* 		Function for handling the interaction between MIDASs and 
*		SNORT.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

/******************************************************************************
*       void Snort_Start(void)
*		Starts SNORT, reads the output and sends alerts to MIDASd
*
*       Updated :
*		11/20/2003 - Fixed Stat issue. Now returns correct values 
*			     instead of string pointer value ;-).
******************************************************************************/
int Snort_Start(void)
{
	FILE * pp;
	char Line[255];
	struct PacketInfo * Packet;
	char * AlertDesc = 0;
        long Sequence = 0;
        char * Protocol = 0;
        char * SNORTCMD = 0;
        char * UpdateMsg = 0;
        char * ServerResponse = 0;
	char * Payload = 0;
	char * tStatInfo = 0;
	char * StatInfo = 0;
	char * tPacketInfo = 0;
	int Status = 1;
		
	// [TCC] Mininum # of seconds between stat sents.
	int MINSTATDELAY = 5;

	struct SNORTStats {
		unsigned int StartTime;
		unsigned int TotalAlerts;
		unsigned int CacheMisses;
		unsigned int TotalPackets;
		unsigned int LastStatSent;
	};
	
	struct SNORTStats SnortStats;
	
	MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_Start]-[Enter]");
	
	if(Snort_BuildConfig())
        {
		while(1==1)
		{
			SNORTCMD = dsprintf(SNORTCMD, "%s -c %s", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.CMD"),
								  ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.CONFIG"));
				
			MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_Start]-[SNORTCMD:%s]", SNORTCMD);
			
			SnortStats.LastStatSent = time(NULL);
  			SnortStats.TotalPackets = 0;
                        SnortStats.TotalAlerts = 0;
                        SnortStats.CacheMisses = 0;
                                        			
			pp = popen(SNORTCMD,"r");
			if(pp)
			{
				do
				{
					fgets(Line, 254, pp);
					if(!feof(pp))
					{
//               	        	         	MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_Start]-[Misc Line]-[LINE:%s]", Line);
						if(strstr(Line, "[**]"))
						{
							SnortStats.TotalAlerts++;
						
        				        	strtok(Line,".");
        				                Sequence = Myatol(strtok(NULL, " "));
        			       		        strtok(NULL, " ");
        			       	                strtok(NULL, " ");
        			                        AlertDesc = dstrcpy(AlertDesc, strtok(NULL, "["));
        			                        AlertDesc = CleanXMLValueString(AlertDesc);
        			                        strtok(NULL, "{");
        			                        Protocol = dstrcpy(Protocol, strtok(NULL, "}"));
	
       	 					        Packet = Snort_FindPacket(Sequence);
       		        		    		if(Packet)
       	        		                 	{
               	        	         			MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_Start]-[Sending Alert]-[Seq:%ld]-[Protocol:%s]", Sequence, Protocol);
               	                 			
               	        	         			Payload = dstrcpy(Payload, Packet->PayLoad);
								Payload = CleanXMLValueString(Payload);
               	        	         			Payload = dsubstrreplace(Payload, "'", " ");
               	        	         			Payload = dsubstrreplace(Payload, "\"", " ");
               	                 			
               	        	         			tPacketInfo = dsprintf(tPacketInfo, "<SNORTALERT><SNORT><SRCMAC>%s</SRCMAC><DSTMAC>%s</DSTMAC><SRCIP>%s</SRCIP><SRCPORT>%d</SRCPORT><DSTIP>%s</DSTIP><DSTPORT>%d</DSTPORT><PROTOCOL>%s</PROTOCOL><SEQUENCE>%ld</SEQUENCE><PAYLOAD>%s</PAYLOAD><ALERTDESC>%s</ALERTDESC></SNORT></SNORTALERT>",
                        	                						Packet->SrcMAC, Packet->DstMAC, 
                        	                						Packet->SrcIP, Packet->SrcPort, 
                        	                						Packet->DstIP, Packet->DstPort, 
                        	                						Protocol, Packet->Seq, 
                        	                						Payload, AlertDesc);
                                        		
                        	                		if(time(NULL) >= SnortStats.LastStatSent + MINSTATDELAY)
                        	                		{
                        	                			tStatInfo = dsprintf(tStatInfo, "<STAT><TYPE>SNORT</TYPE><ID>STAT</ID><ORDER>0</ORDER><LABEL>Total Packets</LABEL><DATAVAL>%u</DATAVAL></STAT>",
                        	                							SnortStats.TotalPackets);
                        	                			StatInfo = dstrcat(StatInfo, tStatInfo);
                        	                			tStatInfo = dfree(tStatInfo);
                        	                		
                        	                			tStatInfo = dsprintf(tStatInfo, "<STAT><TYPE>SNORT</TYPE><ID>STAT</ID><ORDER>1</ORDER><LABEL>Total Alerts</LABEL><DATAVAL>%u</DATAVAL></STAT>",
                        	                							SnortStats.TotalAlerts);
                        	                			StatInfo = dstrcat(StatInfo, tStatInfo);
                        	                			tStatInfo = dfree(tStatInfo);
                        	                		
                        	                			tStatInfo = dsprintf(tStatInfo, "<STAT><TYPE>SNORT</TYPE><ID>STAT</ID><ORDER>2</ORDER><LABEL>Cache Misses</LABEL><DATAVAL>%u</DATAVAL></STAT>",
                        	                							SnortStats.CacheMisses);
                        	                			StatInfo = dstrcat(StatInfo, tStatInfo);
                        	                			tStatInfo = dfree(tStatInfo);
                        	                			
                        	                			SnortStats.LastStatSent = time(NULL);
                        	                			SnortStats.TotalPackets = 0;
                        	                			SnortStats.TotalAlerts = 0;
                        	                			SnortStats.CacheMisses = 0;
                        	                		} else {
                               		         			StatInfo = dstrcpy(StatInfo, "");
                               		         		}
                                        		
                                	        		UpdateMsg = dsprintf(UpdateMsg, "<CLIENT><HID>%s</HID><TYPE>SNORTALERT</TYPE>%s%s</CLIENT>",
                                	        						ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
                                	        						tPacketInfo, StatInfo);
                                	        		tPacketInfo = dfree(tPacketInfo);
                                	        		StatInfo = dfree(StatInfo);
                                	        		
								ServerResponse = SendToServer(UpdateMsg);
								ServerResponse = dfree(ServerResponse);
								Payload = dfree(Payload);
								UpdateMsg = dfree(UpdateMsg);
								free(Packet);
								Packet = NULL;							
							} else {
								SnortStats.CacheMisses++;
							}
								
							AlertDesc = dfree(AlertDesc);
							Protocol = dfree(Protocol);
						} else {
							if(strncmp(Line, "=+=", 3)) 
							{
								if(strstr(Line, "->") && strstr(Line, "type:"))
								{
									Packet = Snort_ParsePacket(Line, pp);
									if(Packet)
									{
										SnortStats.TotalPackets++;
										
										Snort_AddPacket(Packet);
										free(Packet);
										Packet = NULL;
									}
								}
							}
						}
					}
					
					usleep(100);
				}while(!feof(pp));
				pclose(pp);
			} else {
				MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_Start]-[Unable to start %s]", SNORTCMD);
				Status = 0;
				break;
			}
		}
	} else {
		MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_Start]-[Unable to retrieve snort config from MIDASd]");
	}

	return(Status);
}	

/******************************************************************************
*       struct PacketInfo * Snort_ParsePacket(char *line, FILE * pp)
*		Parses outut of Snort and returns the PacketInfo struct.
*
*	Args :
*		char *line	1st line of pip read.
*		FILE * pp	Handle of pipe to snorts output.
*
*       Updated :
******************************************************************************/
struct PacketInfo * Snort_ParsePacket(char *line, FILE * pp)
{
	char line2[255];
	struct PacketInfo * Packet;
	
	char tSIP[40];
	char tDIP[40];
	
	Packet = (struct PacketInfo *) malloc(sizeof(struct PacketInfo));
	bzero(Packet, sizeof(struct PacketInfo));
		
	sprintf(Packet->TimeStamp, "%s", strtok(line, "."));
	Packet->Seq = Myatol(strtok(NULL, " "));	
	sprintf(Packet->SrcMAC, "%s", strtok(NULL, " "));
	strtok(NULL, " ");
	sprintf(Packet->DstMAC, "%s", strtok(NULL, " "));

	fgets(line2, 255, pp);
	
	sprintf(tSIP, "%s", strtok(line2, " "));
	strtok(NULL, " ");
	sprintf(tDIP, "%s", strtok(NULL, " "));						
	sprintf(Packet->Proto, "%s", strtok(NULL, " "));

	if(strchr(tSIP, ':'))
	{
		sprintf(Packet->SrcIP, "%s", strtok(tSIP,":"));
		Packet->SrcPort = Myatoi(strtok(NULL, "\0"));
		
	} else {
		strcpy(Packet->SrcIP, tSIP);
		Packet->SrcPort = 0;
	}
	
	if(strchr(tDIP, ':'))
	{
		sprintf(Packet->DstIP, "%s", strtok(tDIP,":"));
		Packet->DstPort = Myatoi(strtok(NULL, "\0"));
		
	} else {
		strcpy(Packet->DstIP, tDIP);
		Packet->DstPort = 0;
	}
	
	fgets(line2, 255, pp);
							
	do
	{
		fgets(line2, 255, pp);
		if(strncmp(line2, "=+=", 3))
		{
		        if(strlen(line2) + strlen(Packet->PayLoad) < 2048)
		        {       
		                strcat(Packet->PayLoad, line2);
		        }
		}
	}while(strncmp(line2, "=+=", 3));
	
	if(!strlen(Packet->PayLoad))
	{
		strcpy(Packet->PayLoad, " ");
	}
	
	return(Packet);
}

/******************************************************************************
*       struct PacketInfo * Snort_FindPacket(long Seq)
*		Finds packet stored in local DB. Returns Null if not found.
*
*	Args :
*		long Seq	Packet Sequence number to look for.
*
*       Updated :
******************************************************************************/
struct PacketInfo * Snort_FindPacket(long Seq)
{
	FILE *fp;
	struct PacketInfo * PacketLocal;
        int i = 0;
	char * SNORTDB = 0;
        
	PacketLocal = (struct PacketInfo *) malloc(sizeof(struct PacketInfo));
	bzero(PacketLocal, sizeof(struct PacketInfo));

	SNORTDB = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.DB");
	fp = fopen(SNORTDB, "r+b");
        if(fp)
        {
        	fread(PacketLocal, sizeof(struct PacketInfo), 1, fp);
                while(!feof(fp))
                {
                        i++;
                        fread(PacketLocal, sizeof(struct PacketInfo), 1, fp);
                        if(PacketLocal->Seq == Seq)
                        {
                                fclose(fp);
                                return(PacketLocal);
                        }
                };
                fclose(fp);
        }
        
	if(PacketLocal)
	{
		free(PacketLocal);
	}
	
	return(NULL);
}

/******************************************************************************
*       int Snort_AddPacket(struct PacketInfo * Packet)
*		Adds Packet information to local DB.
*
*	Args :
*		struct PacketInfo * Packet	Packet info struct to add to DB.
*
*
*       Updated :
******************************************************************************/	
int Snort_AddPacket(struct PacketInfo * Packet)
{
	FILE *fp;
	struct PacketInfo * PacketLocal;
	char * SNORTDB = 0;
	        	
	PacketLocal = (struct PacketInfo *) malloc(sizeof(struct PacketInfo));
	bzero(PacketLocal, sizeof(struct PacketInfo));

	SNORTDB = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.DB");
	fp = fopen(SNORTDB, "r+b");   
	if(!fp)
	{
		fp = fopen(SNORTDB, "w+b");
		if(fp)
		{
			PacketLocal->Seq = 1;
			fwrite(PacketLocal, sizeof(struct PacketInfo), 1, fp);
			fwrite(Packet, sizeof(struct PacketInfo), 1, fp);
			fclose(fp);
			free(PacketLocal);
			return(1);
		}
	        fclose(fp);
		return(0);
	} else {
		fread(PacketLocal, sizeof(struct PacketInfo), 1, fp);
		
		if(PacketLocal->Seq > Myatoll(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.DB.CACHESIZE")))
		{
			PacketLocal->Seq = 1;
		}
		
		fseek(fp, (PacketLocal->Seq * sizeof(struct PacketInfo)), SEEK_SET);
		fwrite(Packet, sizeof(struct PacketInfo), 1, fp);
		
		PacketLocal->Seq++;
		rewind(fp);
		fwrite(PacketLocal, sizeof(struct PacketInfo), 1, fp);
		fclose(fp);

		free(PacketLocal);
		return(1);		
	}
}	

/******************************************************************************
*       int Snort_BuildConfig(void)
*		Queries MIDASd for snort.conf/rules and builds local config.
*
*       Updated :
******************************************************************************/
int Snort_BuildConfig(void)
{
        FILE * fp;
	//FILE * pp;
	//char * CMD = 0;
        int Status = 0;
	char * Query = 0;        
	char * ServerResponse = 0;
	char * SNORTCFG = 0;
	struct XMLLeaf * ServerXMLLeaf = 0;
	char * SnortConfig = 0;
	struct BinData * B64SnortConfig;	
	
	MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_BuildConfig]-[Enter]");
	
	Query = dsprintf(Query, "<CLIENT><HID>%s</HID><SNORTCONFIG><VERSION>%s</VERSION></SNORTCONFIG><TYPE>SNORTCONFIG</TYPE></CLIENT>", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"), ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.VERSION"));
	ServerResponse = SendToServer(Query);
	Query = dfree(Query);
	
	if(ServerResponse)
	{
		MyDebug(__LINE__, __FILE__, LOG_SNORT, "[Snort_BuildConfig]-[Serverresponse_strlen:%ld]", dstrlen(ServerResponse));

		ServerXMLLeaf = StringToXMLTree(ServerResponse);
		ServerResponse = dfree(ServerResponse);
		
		if(ServerXMLLeaf)
		{
			if(ReturnXMLLeafValue(ServerXMLLeaf, "SERVER.SNORTCONFIG.CONFIG"))
			{
				SNORTCFG = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.SNORT.CONFIG");
			
				MyDebug(__LINE__, __FILE__, LOG_SNORT, "[SnortBuildConfig]-[Writing:%s]", SNORTCFG);
			
				fp = fopen(SNORTCFG, "w");
				if(fp)
				{
					SnortConfig = dstrcpy(SnortConfig, ReturnXMLLeafValue(ServerXMLLeaf, "SERVER.SNORTCONFIG.CONFIG"));
					
					B64SnortConfig = Base64Dec(SnortConfig);
					B64SnortConfig->Data[B64SnortConfig->Len] = 0;
					
					SnortConfig = dfree(SnortConfig);
					SnortConfig = dstrcpy(SnortConfig, B64SnortConfig->Data);
					
					B64SnortConfig->Data = dfree(B64SnortConfig->Data);
					free(B64SnortConfig);
					
					// Make sure the SnortConfig is a UNIX format (removing \r)
					SnortConfig = dsubstrreplace(SnortConfig, "\r\n", "$[REPLACE]$");
					SnortConfig = dsubstrreplace(SnortConfig, "$[REPLACE]$", "\n");

					if(SnortConfig)
					{
						fprintf(fp, "%s", SnortConfig);
					}
					fclose(fp);
					
					// Free the memory
					dfree(SnortConfig);

				}
				Status = 1;
			} else {
				Status = 0;
			}	
			ServerXMLLeaf = DelXMLLeafTree(ServerXMLLeaf);
		} else {
			Status = 0;
		}	
	} else {
		Status = 0;
	}
	
	return(Status);
}

