###############################################################################
#
#	MIDAS Tables.
#

#
# Hosts
#	Host defs
#
DROP TABLE IF EXISTS MIDAS_Hosts;
CREATE TABLE MIDAS_Hosts (
	HID		int NOT NULL AUTO_INCREMENT,
	GID		int,
	AGID		int,
	HostName	char(255),
	IPAddr		char(255),
	Descr		blob,
	IsSniffer	int,
	Enabled		int,
	Primary Key (HID)
);

#
# Groups
#	Host group defs
#
DROP TABLE IF EXISTS MIDAS_Groups;
CREATE TABLE MIDAS_Groups (
	GID		int NOT NULL AUTO_INCREMENT,
	Name		char(50),
	Descr		char(255),
	Enabled		int,
	Primary Key (GID)
);

#  Check_Types
#	The Check types
DROP TABLE IF EXISTS MIDAS_Check_Types;
CREATE TABLE MIDAS_Check_Types (
	CheckType	int,
	Name		char(255)
);

#
# Insert the default Check_Types
#
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (1, 'TCP');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (2, 'UDP');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (3, 'PING');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (4, 'Disk');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (5, 'CPU');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (6, 'Processes');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (7, 'Log');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (8, 'SNMP');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (9, 'MIDAS External');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (10, 'NETSAINT External');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (11, 'RAM');
INSERT INTO MIDAS_Check_Types (CheckType, Name) VALUES (12, 'MD5');


#
# Checks
#	Check defs
#
DROP TABLE IF EXISTS MIDAS_Checks;
CREATE TABLE MIDAS_Checks (
	CID		int NOT NULL AUTO_INCREMENT,
	Name		char(50) NOT NULL,
	Descr		char(255),
	Type		int,
	Port		int,
        Options         blob,
	ConnectOnly	int,
	AlertOnRed	int,
	Enabled		int,
	Primary Key (CID)
);

#
# Insert default checks
#
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (1, 'HTTP', 'HTTP', 1, 80, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (2, 'FTP', 'FTP', 1, 21, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (3, 'HTTPS', 'HTTPS', 1, 443, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (4, 'DNS', 'DNS', 1, 53, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (5, 'POP3', 'POP3', 1, 110, '\n', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (6, 'SMTP', 'SMTP', 1, 25, '', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (7, 'NNTP', 'NNTP', 1, 119, '', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (8, 'MsSQL', 'MsSQL', 1, 1433, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (9, 'VNC', 'VNC', 1, 5900, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (10, 'MySQL', 'MySQL', 1, 3306, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (11, 'PPTP', 'PPTP', 1, 1723, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (12, 'Telnet', 'Telnet', 1, 23, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (13, 'SSH', 'SSH', 1, 22, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (14, 'SNPP', 'SNPP', 1, 444, '', 1, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (15, 'PCAnyWhere', 'PCAnyWhere', 1, 5631, '', 1, 0, 1);

INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
        VALUES (16, 'PING', 'PING', 3, 0, '<!--\n<PING>\n  <COUNT>10</COUNT>\n</PING>\n-->', 1, 0, 1);

INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (17, 'Disks', 'Disk Space check', 4, 0, '<!--\n<DISK>\n  <PARTITION>/dev/hda1</PARTITION>\n  <SPACEFREE>10240</SPACEFREE>\n</PARTITION>\n-->', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (18, 'CPU', 'CPU Load Check', 5, 0, '<!--\n<CPU>\n  <MAXLOAD>1.5</MAXLOAD>\n</CPU>\n-->', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (19, 'Process', 'Process Check', 6, 0, '<!--\n<PROC>\n  <TYPE>COUNT</TYPE>\n  <LIMIT>20</LIMIT>\n</PROC>\n-->', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (20, 'LOG', 'Event/System Log Check', 7, 0, '<!--\n<LOG>\n  <LOGFILE>/var/adm/syslog</LOGFILE>\n  <REGEX>kernel</REGEX>\n  <LOGIC>EQUAL</LOGIC>\n  <EXPECT>kernel</EXPECT>\n</LOG>\n-->', 0, 1, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (21, 'SNMP', 'SNMP Check', 8, 0, '<!--\n<SNMP>\n  <OID>1.2.3.4.5.6.7</OID>\n  <COMMUNITY>public</COMMUNITY>\n  <LOGIC>LESS</LOGIC>\n  <EXPECT>1024</EXPECT>\n</SNMP>\n-->', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (22, 'RAM', 'RAM Check', 11, 0, '<!--\n<RAM>\n  <RAMFREE>10240</RAMFREE>\n  <SWAPFREE>102400</SWAPFREE>\n</RAM>\n-->', 0, 0, 1);
INSERT INTO MIDAS_Checks (CID, Name, Descr, Type, Port, Options, ConnectOnly, AlertOnRed, Enabled) 
	VALUES (23, 'MD5', 'MD5 File Check', 12, 0, '<!--\n<MD5>\n  <PATH>/etc</PATH>\n  <RECURSIVE>1</RECURSIVE>\n</MD5>\n-->', 0, 1, 1);

# 
# Host_Checks
#	Host Check Defs
# 
DROP TABLE IF EXISTS MIDAS_Host_Checks;
CREATE TABLE MIDAS_Host_Checks (
	HID			int NOT NULL,
	TID			int NOT NULL,
	CID			int NOT NULL,
	DoAlert			int,
	MinFail			int,
	CurFail			int,
	TotFail			int(4),
	Options			blob,
	RedAlertIssued		int,
	RedAlertIssuedOn	TIMESTAMP(14),
	Enabled			int
);

#
# Check_Events
#	Historical events of checks
#
DROP TABLE IF EXISTS MIDAS_Check_Events;
CREATE TABLE MIDAS_Check_Events (
	EID		int NOT NULL AUTO_INCREMENT,
	HID		int,
	CID		int,	
	TID		int,	
	Status		char(21),
	Response	longblob,
	TimeStamp	DATETIME,
	Primary Key (EID),
	KEY HID (HID),
	KEY CID (CID),
	KEY TID (TID),
	KEY TimeStamp (TimeStamp)
);

#
# Check_Current
#	Current status of checks
#
DROP TABLE IF EXISTS MIDAS_Check_Current;
CREATE TABLE MIDAS_Check_Current (
	TID		int NOT NULL,
	HID		int NOT NULL,
	CID		int NOT NULL,
	Status		char(21),
	Response	longblob,
	LastCheck	DATETIME,
	FirstCheck	DATETIME,
	Primary Key (TID, HID, CID)
);

#
# Users
#	Login defs for MIDAS
#
DROP TABLE IF EXISTS MIDAS_Users;
CREATE TABLE MIDAS_Users (
  uid 		int(11) NOT NULL auto_increment,
  username 	varchar(40) NOT NULL default '',
  passwd 	varchar(40) NOT NULL default '',
  email 	varchar(255) default NULL,
  accessLevel 	varchar(11) NOT NULL default '0',
  overwrite     INT(1) NOT NULL DEFAULT '0',
  lastAccess 	int(20) default NULL,
  enabled 	int(1) NOT NULL default '0',
  PRIMARY KEY (uid),
  UNIQUE KEY username (username)
);
INSERT INTO MIDAS_Users (username, passwd, email, accessLevel) 
                        VALUES ('admin', MD5('password'), 'noreply@localhost', 255);

#
# MIDAS_Alert_Groups
#	Defines Alert Groups
#
DROP TABLE IF EXISTS MIDAS_Alert_Groups;
CREATE TABLE MIDAS_Alert_Groups (
	AGID	int NOT NULL AUTO_INCREMENT,
	Name	char(255),
	Descr	char(255),
	Primary Key (AGID)
);
INSERT INTO MIDAS_Alert_Groups (AGID, Name, Descr) VALUES (1, 'Default', 'The Default Alert Group');

#
# MIDAS_Alert_Contacts
#	Used to store defined alerts.
#
DROP TABLE IF EXISTS MIDAS_Alert_Contacts;
CREATE TABLE MIDAS_Alert_Contacts (
	ACID		int NOT NULL AUTO_INCREMENT,
	Name		char(255),
	Descr		char(255),
	AlertType	int,
	AlertOption	blob,
	OnTime		int UNSIGNED,
	OffTime		int UNSIGNED,
	Primary Key(ACID)
);

#
# MIDAS_Alert_Contact_Groups
#	Used to define Alert groups
#
DROP TABLE IF EXISTS MIDAS_Alert_Contact_Groups;
CREATE TABLE MIDAS_Alert_Contact_Groups (
	ACID	int NOT NULL,
	AGID	int NOT NULL,
	Primary Key(ACID, AGID)
);



DROP TABLE IF EXISTS MIDAS_Alert_Queue;
CREATE TABLE MIDAS_Alert_Queue (
	AID		int NOT NULL AUTO_INCREMENT,
	EID		int,
	Primary Key(AID)
);

###############################################################################
#
#	IDS Related Tables
#

#
# IDS_Events
#	Where we store the Snort Alerts
#	
DROP TABLE IF EXISTS MIDAS_IDS_Events;
CREATE TABLE MIDAS_IDS_Events (
	EID		int NOT NULL AUTO_INCREMENT,
	HID		int,
	SMAC		char(40),
	SIP		char(20),
	SPort		int,
	DMAC		char(40),
	DIP		char(20),
	DPort		int,
	Protocol	char(30),
	Descr		longblob,
	Payload		longblob,
	TimeStamp	DATETIME,
	Seq		int(4),
	Primary Key (EID),
      KEY TimeStamp (TimeStamp)
);

#
# Individual Rule Definitions
#
DROP TABLE IF EXISTS MIDAS_SNORT_Rulebase;
CREATE TABLE MIDAS_SNORT_Rulebase (
	RID		int NOT NULL AUTO_INCREMENT,
	Name		char(255),
	Description	blob,
	Category	char(255),
	Rule2_0		blob,
	Rule2_1		blob,
	Enabled		int,
	Primary Key (RID)
);

#
# Group the rules together by HID
#
DROP TABLE IF EXISTS MIDAS_SNORT_HID_x_RID;
CREATE TABLE MIDAS_SNORT_HID_x_RID (
	RID		int NOT NULL,
	HID		int NOT NULL,
	Primary Key(RID,HID)
);

#
# Provide preprocessors/output plugins for various versions of SNORT
#
DROP TABLE IF EXISTS MIDAS_SNORT_Mapping;
CREATE TABLE MIDAS_SNORT_Mapping (
	MID		int NOT NULL AUTO_INCREMENT,
	Data		blob,
	Short		blob,
	Version		char(255),
	MTYPE		enum('preprocessor','output plugin','classification') NOT NULL default 'preprocessor',
	NORDER		int not null,
	Primary Key(MID)
);

#
#
#
DROP TABLE IF EXISTS MIDAS_SNORT_Sections;
CREATE TABLE MIDAS_SNORT_Sections (
	HID		int NOT NULL AUTO_INCREMENT,
	Section		char(255),
	Details		longblob,
	NORDER		int NOT NULL,
	Primary Key(HID,Section)
);

#
#
#
DROP TABLE IF EXISTS MIDAS_SNORT_Groups;
CREATE TABLE MIDAS_SNORT_Groups (
	GID		int NOT NULL AUTO_INCREMENT,
	Name		char(255) NOT NULL,
	Description	longblob,
	Primary Key(GID)
);
INSERT INTO MIDAS_SNORT_Groups (GID,Name,Description) VALUES (1000000, 'CUSTOM-RULES', 'This is a default group created for custom rules.');

#
#
#
DROP TABLE IF EXISTS MIDAS_SNORT_Status;
CREATE TABLE MIDAS_SNORT_Status (
	HID		int NOT NULL,
	Status		enum('NO_CONFIGURATION', 'PARTIALLY_CONFIGURED', 'FULLY_CONFIGURED', 'READY_TO_SYNCH', 'CLIENT_N_SYNCH') NOT NULL default 'NO_CONFIGURATION',
	LastUpdate	datetime,
	LastSynch	datetime,
	AutoUpdate	int(11),
	Primary Key(HID)
);

###############################################################################
#
#	RRD Related Tables
#

# RRD_MIDAS_ID
#	DSID		=	DataSource ID
#	Type		=	Stat Type (CHECK, NETMON, SNORT)
#	ID1		= 	MIDAS ID1
#	ID2		=	MIDAS ID2
#	ID3		=	MIDAS ID3
#
#
#	Standard STAT update ID usage:
#
#		ID1 = HID of Host
#		ID2 = CID_TID
#		ID3 = Order of Check Option (Starting at 0)
#
DROP TABLE IF EXISTS RRD_MIDAS_ID;
#
CREATE TABLE RRD_MIDAS_ID (
	DSID	int NOT NULL,
	Type	char(50) NOT NULL,
	ID1	char(50) NOT NULL,
	ID2	char(50) NOT NULL,
	ID3	char(50) NOT NULL,
	Primary Key(DSID)
);

# RRD_Values_Queue
#	HID		=	Host ID
#	DSID		=	Data Source ID
#	TimeStamp	=	TimeStamp of Data value.
#	Value		= 	Value
DROP TABLE IF EXISTS RRD_Values_Queue;
#
CREATE TABLE RRD_Values_Queue (
	HID		int NOT NULL,
	DSID		int NOT NULL,
	TimeStamp	int NOT NULL,
	Value		float,
	Primary Key(HID, DSID, TimeStamp)
);

# RRD_DataSource
#	DSID		=	DataSource ID
#	Sequences	=	Number of Sequences in Data Source
#	Interval	=	Interval Between Sources.
#	Title		=	Title (Display) for Data Source
#	Format 		= 	Format Data should be desiplayed as (Ie
#				NONE, TIMETICKS, BYTES, TIMESTAMP).
#
DROP TABLE IF EXISTS RRD_DataSource;
#
CREATE TABLE RRD_DataSource (
	DSID		int NOT NULL AUTO_INCREMENT,
	Sequences	int,
	Intervals	int,
	Title		char(255),
	Format		char(20),	
	Primary Key(DSID)
);

# RRD_DataValues
#	DSID		=	DataSource ID
#	Sequence	=	Sequence number
#	TimeStamp	= 	TimeStamp of Data.
#	Value		=	Value
#	AverageValue	=	Average (Over all sequences) value
#	DerivedValue	=	Difference from last Data Value.
#
DROP TABLE IF EXISTS RRD_DataValues;
#
CREATE TABLE RRD_DataValues (
	DSID		int NOT NULL,
	Sequence	int NOT NULL,
	TimeStamp	int,
	Value		float,
	AverageValue	float,
	DerivedValue	float,
	Primary Key(DSID, Sequence)
);

# RRD_ArchiveSource
#	DAID 		=	DataSource Archive ID
#	DSID		=	DataSource ID (Where to Pull the originating Data from)
#	Sequences	=	Number of Sequences in this archive.
#	Interval	= 	Interval in seconds to find average from DataSource.
#
DROP TABLE IF EXISTS RRD_ArchiveSource;
#	
CREATE TABLE RRD_ArchiveSource (
	DAID 		int NOT NULL AUTO_INCREMENT,
	DSID		int,
	Sequences	int,
	Intervals	int,
	Primary Key(DAID)
);

# RRD_ArchiveValues
#	DAID		=	Data Archive ID
#	Sequence	=	Sequence number
#	TimeStamp	= 	TimeStamp of Data.
#	Value		=	Value
#	AverageValue	=	Average (Over all sequences) value
#	DerivedValue	=	Difference from last Data Value.
#
DROP TABLE IF EXISTS RRD_ArchiveValues;
#
CREATE TABLE RRD_ArchiveValues (
	DAID		int NOT NULL,
	Sequence	int NOT NULL,
	TimeStamp	int,
	Value		float,
	AverageValue	float,
	DerivedValue	float,
	Primary Key(DAID, Sequence)
);


DROP TABLE IF EXISTS MIDAS_Reg_Modules;
#
CREATE TABLE MIDAS_Reg_Modules (
  module_name 		varchar(100) NOT NULL default '',
  module_loc 		varchar(100) NOT NULL default '',
  module_req 		varchar(255) NOT NULL default '',
  module_priority 	int(2) NOT NULL default '9',
  module_init 		varchar(100) NOT NULL default '',
  PRIMARY KEY (module_name)
);

DROP TABLE IF EXISTS MIDAS_User_Activity;
#
CREATE TABLE MIDAS_User_Activity (
  uid int(11) NOT NULL default '0',
  accessLevel char(3) default NULL,
  remoteaddr varchar(15) default NULL,
  sesskey varchar(255) NOT NULL default '',
  stamp int(20) NOT NULL default '0',
  PRIMARY KEY  (uid),
  KEY sesskey (sesskey)
);

###########################################################
#
#	File Monitoring tables
#
#	[TCC] Commented out till 2.3
#
#DROP TABLE IF EXISTS MIDAS_FMON_Directories;
#
#CREATE TABLE MIDAS_FMON_Directories (
#	DID	int NOT NULL AUTO_INCREMENT,
#	HID	int,
#	PDID	int,
#	Path	char(255),
#	UID	int,
#	GID	int,
#	aTime	TIMESTAMP(14),
#	cTime	TIMESTAMP(14),
#	mTime	TIMESTAMP(14),
#	Status	int,
#	Primary Key(DID)
#);

#
#DROP TABLE IF EXISTS MIDAS_FMON_Files;
#
#CREATE TABLE MIDAS_FMON_Files (
#	FID		int NOT NULL AUTO_INCREMENT,
#	DID		int,
#	HID		int,
#	FileName	char(255),
#	UID		int,
#	GID		int,
#	FileSize	int,
#	aTime		int,
#	cTime		int,
#	mTime		int,
#	Mode		int,
#	Status		int,
#	Primary Key(FID)
#);

###########################################################
#
#	Network Monitoring Tables
#
#
#
# MIDAS_NetMon_Ports
#	Netowrk monitoring configuration.
#
DROP TABLE IF EXISTS MIDAS_NetMon_Ports;
#
CREATE TABLE MIDAS_NetMon_Ports (
	HID		int NOT NULL,
	Options		blob,
	Primary Key(HID)
);

#
# MIDAS_MACIP
#	Used to store the recorded MAC/IP list
#
DROP TABLE IF EXISTS MIDAS_MACIP;
#
CREATE TABLE MIDAS_MACIP (
	HID		int NOT NULL,
	MAC		char(40) NOT NULL,
	IP		char(17) NOT NULL,
	FoundOn		TIMESTAMP(14),
	LastSeen	TIMESTAMP(14),
	Primary Key(HID, MAC, IP)
);

#
# MIDAS_NetMon_Sessions
#	Used to Store Network Sessions
#
DROP TABLE IF EXISTS MIDAS_NetMon_Sessions;
#
CREATE TABLE MIDAS_NetMon_Sessions (
	SID		int NOT NULL AUTO_INCREMENT,
	HID		int NOT NULL,
	SrcIP		char(40),
	SrcPort		int,
	DstIP		char(40),
	DstPort		int,
	Protocol	char(20),
	Bytes		int,
	PacketCount	int,
	StartTime	TIMESTAMP(14),
	EndTime		TIMESTAMP(14),
	UpdateCount	int,
	Primary Key(SID, HID)
);


###########################################################
#
#	MIDAS Client Information
#

#
# MIDAS_Client_Info
#	Stores client application information
#
#DROP TABLE IF EXISTS MIDAS_Client_Info;
#
#CREATE TABLE MIDAS_Client_Info (
#	HID		int NOT NULL,
#	ClientType	char(40) NOT NULL,
#	Version		char(10),
#	LastContact	TIMESTAMP(14),
#	Config		blob,
#	ConfigTimeStamp	TIMESTAMP(14),
#	Primary Key(HID, ClientType)
#);
#
#
# MIDAS_Client_Stats;
#	Stores Client application specific stats.
#
#DROP TABLE IF EXISTS MIDAS_Client_Stats;
#
#CREATE TABLE MIDAS_Client_Stats (
#	HID		int NOT NULL,
#	ClientType	char(40) NOT NULL,
#	StatTag		char(40) NOT NULL,
#	StatValue	blob,
#	TimeStamp	TIMESTAMP(14),
#	Primary Key(HID, ClientType, StatTag)
#);

#
# MIDAS_Client_Stat_Defs
#	Stores the Client stat def's
#
#DROP TABLE IF EXISTS MIDAS_Clent_Stat_Defs
#
#CREATE TABLE MIDAS_Client_Stat_Defs (
#	ClientType	char(40) NOT NULL,
#	StatTag		char(40) NOT NULL,
#	StatTitle	char(255),
#	Primary Key(ClientType, StatTag)
#);
#
#INSERT INTO MIDAS_Client_Stat_Defs (ClientType, StatTag, StatTitle) VALUES, 'MIDASC', 'UPTIME', 'Uptime');
#INSERT INTO MIDAS_Client_Stat_Defs (ClientType, StatTag, StatTitle) VALUES, 'MIDASC', 'CHECKTIME', 'Check Cycle Time');
#INSERT INTO MIDAS_Client_Stat_Defs (ClientType, StatTag, StatTitle) VALUES, 'MIDASC', 'NUMCHECKS', 'Checks Performed');
#INSERT INTO MIDAS_Client_Stat_Defs (ClientType, StatTag, StatTitle) VALUES, 'MIDASC', 'VERSION', 'Version');
#INSERT INTO MIDAS_Client_Stat_Defs (ClientType, StatTag, StatTitle) VALUES, 'MIDASC', 'CLIENTAPPMD5', 'MD5 Signature of Client');
#INSERT INTO MIDAS_Client_Stat_Defs (ClientType, StatTag, StatTitle) VALUES, 'MIDASC', 'LASTUPDATE', 'Last connect');

