#
# [TCC] Change the DSID to a Auto-Inc type.
#
ALTER TABLE RRD_DataSource MODIFY DSID int NOT NULL AUTO_INCREMENT;

#
# [TCC] Dropping the old Snort tables
#
# Unregister the old IDS modules.
DELETE FROM MIDAS_Reg_Modules WHERE module_name='IDS View';
DELETE FROM MIDAS_Reg_Modules WHERE module_name='Web Configuration (IDS)';
#
# Drop the old tables.
DROP TABLE IF EXISTS MIDAS_SNORT_Conf;
DROP TABLE IF EXISTS MIDAS_SNORT_Rules;

#
# New SNORT Tables
#
#
# Individual Rule Definitions
#
DROP TABLE IF EXISTS MIDAS_SNORT_Rulebase;
CREATE TABLE MIDAS_SNORT_Rulebase (
	RID		int NOT NULL AUTO_INCREMENT,
	Name		char(255),
	Description	blob,
	Category	char(255),
	Rule2_0		blob,
	Rule2_1		blob,
	Enabled		int,
	Primary Key (RID)
);

#
# Group the rules together by HID
#
DROP TABLE IF EXISTS MIDAS_SNORT_HID_x_RID;
CREATE TABLE MIDAS_SNORT_HID_x_RID (
	RID		int NOT NULL,
	HID		int NOT NULL,
	Primary Key(RID,HID)
);

#
# Provide preprocessors/output plugins for various versions of SNORT
#
DROP TABLE IF EXISTS MIDAS_SNORT_Mapping;
CREATE TABLE MIDAS_SNORT_Mapping (
	MID		int NOT NULL AUTO_INCREMENT,
	Data		blob,
	Short		blob,
	Version		char(255),
	MTYPE		enum('preprocessor','output plugin','classification') NOT NULL default 'preprocessor',
	NORDER		int not null,
	Primary Key(MID)
);

#
#
#
DROP TABLE IF EXISTS MIDAS_SNORT_Sections;
CREATE TABLE MIDAS_SNORT_Sections (
	HID		int NOT NULL AUTO_INCREMENT,
	Section		char(255),
	Details		longblob,
	NORDER		int NOT NULL,
	Primary Key(HID,Section)
);

#
#
#
DROP TABLE IF EXISTS MIDAS_SNORT_Groups;
CREATE TABLE MIDAS_SNORT_Groups (
	GID		int NOT NULL AUTO_INCREMENT,
	Name		char(255) NOT NULL,
	Description	longblob,
	Primary Key(GID)
);
INSERT INTO MIDAS_SNORT_Groups (GID,Name,Description) VALUES (1000000, 'CUSTOM-RULES', 'This is a default group created for custom rules.');

#
#
#
DROP TABLE IF EXISTS MIDAS_SNORT_Status;
CREATE TABLE MIDAS_SNORT_Status (
	HID		int NOT NULL,
	Status		enum('NO_CONFIGURATION', 'PARTIALLY_CONFIGURED', 'FULLY_CONFIGURED', 'READY_TO_SYNCH', 'CLIENT_N_SYNCH') NOT NULL default 'NO_CONFIGURATION',
	LastUpdate	datetime,
	LastSynch	datetime,
	AutoUpdate	int(11),
	Primary Key(HID)
);
